package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.*;

public abstract class AbstractBuiltInFunc implements BuiltInFunc {

  private String getArgPlural(int size) {
    return size == 1 ? " argument" : " arguments";
  }

  private String argSizeMessage(String funcName, int expectedSize,
                                int actualSize) {
    StringBuffer b = new StringBuffer("Built-in function '");
    b.append(funcName);
    b.append("(...)' expects ");
    b.append(expectedSize);
    b.append(getArgPlural(expectedSize));
    b.append("; ");
    b.append(actualSize);
    b.append(" given.");

    return b.toString();
  }

  /**
   * Checks if specified args has the specified number of
   * arguments. The funcName is just fo error reporting.
   *
   * @param funcName
   * @param args
   * @param expectedSize
   */
  protected void argSizeCheck(String funcName, ASTarglist args,
                              int expectedSize) {
    if (args == null && expectedSize == 0) return;
    if (args == null && expectedSize > 0) {
      throw ErrorFactory.
              createCompilerException(argSizeMessage(funcName, expectedSize, 0));
    }

    if (args.jjtGetNumChildren() != expectedSize) {
      throw ErrorFactory.
              createCompilerException(argSizeMessage(funcName, expectedSize,
                                                     args.jjtGetNumChildren()));

    }
  }

  /**
   * Checks if specified arg is either a number, boolean, or String.
   * The funcName is for error reporting and nothing else.
   *
   * @param funcName
   * @param arg
   */
  protected void argNumBoolStringCheck(String funcName, NQPyClass arg) {
    int id = arg.id();
    if (id != NQPyClass.LONG_CLASS && id != NQPyClass.FLOAT_CLASS &&
            id != NQPyClass.DOUBLE_CLASS && id != NQPyClass.BOOLEAN_CLASS &&
            id != NQPyClass.INT_CLASS && id != NQPyClass.STRING_CLASS) {
      String message = "Built-in function '" + funcName + "(...)' " +
              "expects a number, boolean, or string as its argument.";
      throw ErrorFactory.createCompilerException(message);
    }
  }
}
