package uchicago.src.nqpy.compiler;

//import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.*;

public abstract class AbstractNQPyClass implements NQPyClass {

  protected Class jclass;
  protected String jclassName;
  protected int id = -1;
  protected boolean canBeBool = false;

  public int id() {
    return id;
  }


  /**
   * Returns a String representation of the Node associated with this
   * NQPyClass
   *
  public String toJString() {
    if (node == null) return "None";
    return node.getInfo() == null ? "None" : node.getInfo().toString();
  }
  */

  /**
   * Gets the Java Class wrapped by this NQPyClass
   */
  public Class getJClass() {
    return jclass;
  }

  /**
   * Gets the name of the Java Class wrapped by this NQPyClass
   */
  public String getJClassName() {
    return jclassName;
  }

  /**
   * Returns whether this NQPyClass can be used as a boolean.
   */
  public boolean canBeBool() {
    return canBeBool;
  }

  /**
   * Makes a TypeDescriptor for the java primitive or class that this
   * NQPyClass represents. This will use the Class of the java primitive
   * or class that this represents to create the descriptor.
   */
  public TypeDescriptor makeTypeDescriptor() {
    return new TypeDescriptor(jclass);
  }

  /**
   * Returns whether the java primitive or class represented by this
   * NQPyClass is assignable from that of the specified NQPyClass.
   */
  public boolean isAssignableFrom(NQPyClass clazz) {
    Class rhs = clazz.getJClass();
    if (rhs == null && clazz.id() == NQPyClass.DYNAMIC_CLASS) rhs = ((NQPyDynamicClass)clazz).superClass;
    return jclass.isAssignableFrom(rhs);
  }

   /**
   * Returns whether the Java object represented by this NQPyClass
   * has the specified constructor.
   */
  public boolean hasConstructor(NQPyClass[] args) {
    return false;
  }

  /**
   * Returns true if this NQPyClass is the same as the specified NQPyClass
   * <b>or</b> if the java Class represented by the specified NQPyClass
   * is a parent of the Java Class represented by this one.
   */
  public boolean inObjHeirarchy(NQPyClass nqclass) {
    if (equals(nqclass)) return true;
    return nqclass.getJClass().isAssignableFrom(jclass);
    /*
    Class parent = jclass.getSuperclass();
    Class nqJClass = nqclass.getJClass();
    if (nqJClass == null) return false;
    //System.out.println("this: " + getJClassName());
    //System.out.println("nqclass: " + nqclass.getJClassName());

    while (parent != null) {
      //System.out.println("parent: " + parent);
      if (parent.equals(nqJClass)) return true;
      parent = parent.getSuperclass();
    }

    Class[] interfaces = jclass.getInterfaces();
    for (int i = 0; i < interfaces.length; i++) {
      System.out.println("interface: " + interfaces[i]);
      System.out.println("nqclass: " + nqclass.getJClass());
      if (nqclass.getJClass().equals(interfaces[i])) return true;
    }
    return false;
    */
  }

  /**
   * Emit the byte code to coerce this NQPyClass to the
   * specifed NQPyClass.
   */
  public void coerce(NQPyBlockCompiler compiler, NQPyClass toCoerce) {
    throw new RuntimeException("Coercion not implemented on " + this);
  }

  /**
   * Returns true if this NQPyClass can be coerced to the specified
   * NQPYClass. By default this returns false.
   */
  public boolean isCoercePossible(NQPyClass toCoerce) {
    return false;
  }

  /**
   * Performs a match of this NQPyClass to the specified NQPyClass. If there
   * is an exact match then this returns this NQPyClass. If there
   * is a match, but requires conversion of the java primitive or class
   * represented by this NQPyClass then the passed in NQPyClass is returned.
   * If there is no match at all, then null is returned.<p>
   */
  public NQPyClass match(NQPyClass nqclass) {
    if (inObjHeirarchy(nqclass)) return this;
    //return convert(nqclass);
    return null;
  }

  public boolean equals(Object o) {
    if (o instanceof AbstractNQPyClass) {
      AbstractNQPyClass nq = (AbstractNQPyClass)o;
      return nq.id == id;
    }

    return false;
  }

  /**
   * Returns the MethodCall object for the constructor of the
   * java object represented by the NQPyClass. If no such constructor
   * exists, then this returns ErrorMethodCall. This default implementation
   * always returns an ErrorMethodCall.
   */
  public MethodCall getConstructorCall(NQPyClass[] args) {
    return new ErrorMethodCall();
  }

  /**
   * Returns the MethodCall object for the specified method of the
   * java object represented by the NQPyClass. If no such method exists,
   * then this returns ErrorMethodCall. This default implementation always
   * returns an ErrorMethodCall.
   */
  public MethodCall getMethodCall(String name, NQPyClass[] args) {
    return new ErrorMethodCall();
  }

  /**
   * Return the FieldCall object for the specified field of the java
   * object represented by this NQPyClass. If no such field exists,
   * this returns null.
   */
  public FieldCall getFieldCall(String name, String callerClassName) {
    return null;
  }

  /**
   * Gets the Emitter appropriate for this NQPyClass.
   */
  public Emitter getEmitter() {
    String message = "getEmitter unsupported on " + this;
    throw ErrorFactory.createCompilerException(message);
  }

  /**
   * Visits the equals method of the passed in NQPyClass.
   */
  public void eq(NQPyClass nqclass) {
    throw ErrorFactory.createCompilerException("eq unsupported on " + this);
  }

  /*
  public Emitter equalsCmp(NQPyIntClass jnqclass) {
    String message = "Comparison not supported between " + getJClassName()
      + " and " + jnqclass.getJClassName();
    throw ErrorFactory.createCompilerException(message);
  }

  public Emitter equalsCmp(NQPyClass nqclass) {
    String message = "Comparison not supported between " + getJClassName()
      + " and " + nqclass.getJClassName();
    throw ErrorFactory.createCompilerException(message);
  }
  */
}
