package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.Label;

public class BooleanEmitter extends AbstractEmitter {
  
  public BooleanEmitter(NQPyClass nqclass) {
    super(nqclass);
  }

  /**
   * Gets the result type (as an NQPyClass) of arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType(NQPyClass rhs) {
    switch (rhs.id()) {
    case NQPyClass.STRING_CLASS:
      return NQPyClassFactory.makeStringClass();
    default:
      String message = "Arithmetic operations not supported between " +
	nqPyClass.getJClassName() + " and " + rhs.getJClassName();
      throw ErrorFactory.createCompilerException(message);
    }
  }

  /**
   * Emits the appropriatxe byte code to go to the specified branch
   * if the specified node compiles to true. This tests whether
   * the result of node.compile() is equal to zero, and branches if the
   * result is not equal to zero.
   */
  public void branchOnTrue(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitNotEqZero(branch);
  }

  /**
   * Emits the appropriate byte code to go to the specified branch
   * if the single node in nodes compiles to false. This tests whether
   * the result of node.compile() is equal to zero, and branches if so.
   */
  public void branchOnFalse(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitEqZero(branch);
  }

  public void eqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitIntComp(branch, "==");
  }

   public void notEqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitIntComp(branch, "!=");
  }

  public void addString(SimpleNode lhsn, SimpleNode rhsn,
			NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {boolean.class});
    rhsn.compile(compiler);
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class}); 
  }
}
