package uchicago.src.nqpy.compiler;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * A RePast specific extension intended to wrap other extensions. Simplifies
 * exception handling.
 */
public class CompilerException extends AbstractCompilerException {

  private Exception ex;
  private String message;

  public CompilerException(String message, Exception ex) {
    this.ex = ex;
    this.message = message;
  }

  public CompilerException(String message) {
    this.message = message;
  }

  public void resetMessage(String message) {
    this.message = message;
  }

  private String formatMessage() {
    StringBuffer b = new StringBuffer("Block: ");
    b.append(blockName);
    if (lineNum >= 0) {
      b.append(", line: ");
      b.append(lineNum);
    }
    b.append("\nCompiler Error: ");
    b.append(message);
    b.append("\n  ");
    b.append(line);
    return b.toString();
  }

  /**
   * Because of the way Throwable works we need an explicit method for
   * getting the actual stack trace. 
   */
  public void printSuperStackTrace() {
    super.printStackTrace(System.err);
  }

  public void printStackTrace() {
    System.out.println(formatMessage());
    if (ex != null) ex.printStackTrace();
  }

  public void printStackTrace(PrintStream s) {
    s.println(formatMessage());
    if (ex != null) ex.printStackTrace(s);
  }

  public void printStackTrace(PrintWriter w) {
    w.println(formatMessage());
    if (ex != null) ex.printStackTrace(w);
  }

  public String getMessage() {
    if (ex != null) return formatMessage() + "\n" + ex.getMessage();
    return formatMessage();
  }
}
