package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.Label;

public class DoubleEmitter extends AbstractEmitter {
  
  public DoubleEmitter(NQPyClass nqclass) {
    super(nqclass);
  }

  /**
   * Gets the result type (as an NQPyClass) of single arg arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType() {
    return nqPyClass;
  }


  /**
   * Gets the result type (as an NQPyClass) of arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType(NQPyClass rhs) {
    switch (rhs.id()) {
    case NQPyClass.INT_CLASS:
      return NQPyClassFactory.makeDoubleClass();
    case NQPyClass.DOUBLE_CLASS:
      return NQPyClassFactory.makeDoubleClass();
    case NQPyClass.LONG_CLASS:
      return NQPyClassFactory.makeDoubleClass();
    case NQPyClass.FLOAT_CLASS:
      return NQPyClassFactory.makeDoubleClass();
    case NQPyClass.STRING_CLASS:
      return NQPyClassFactory.makeStringClass();
    default:
      String message = "Arithmetic operations not supported between " +
	nqPyClass.getJClassName() + " and " + rhs.getJClassName();
      throw ErrorFactory.createCompilerException(message);
    }
  }

  /**
   * Emits the appropriatxe byte code to go to the specified branch
   * if the specified node compiles to true. This tests whether
   * the result of node.compile() is equal to zero, and branches if the
   * result is not equal to zero.
   */
  public void branchOnTrue(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitPrimitiveConversion(double.class, int.class);
    compiler.emitNotEqZero(branch);
  }

  /**
   * Emits the appropriate byte code to go to the specified branch
   * if the single node in nodes compiles to false. This tests whether
   * the result of node.compile() is equal to zero, and branches if so.
   */
  public void branchOnFalse(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitPrimitiveConversion(double.class, int.class);
    compiler.emitEqZero(branch);
  }
  
  private void intOp(SimpleNode lhsn, SimpleNode rhsn,
		     NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitPrimitiveConversion(int.class, double.class);
  }

  private void doubleOp(SimpleNode lhsn, SimpleNode rhsn,
			NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
  }

  private void floatOp(SimpleNode lhsn, SimpleNode rhsn,
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitPrimitiveConversion(float.class, double.class);
  }

  private void longOp(SimpleNode lhsn, SimpleNode rhsn,
		      NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitPrimitiveConversion(long.class, double.class);
  }
  
  public void eqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitEqZero(branch);
  }
  
  public void eqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitEqZero(branch);
  }
  
  public void eqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitEqZero(branch);
  }
  
  public void eqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitEqZero(branch);
  }

  public void notEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitNotEqZero(branch);
  }
  
  public void notEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitNotEqZero(branch);

  }
  
  public void notEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitNotEqZero(branch);
  }
  
  public void notEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitNotEqZero(branch);
  }
  
  public void gtCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtZero(branch);
  }
  
  public void gtCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtZero(branch);

  }
  
  public void gtCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtZero(branch);
  }
  
  public void gtCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtZero(branch);
  }

  public void gtEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtEqZero(branch);
  }
  
  public void gtEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtEqZero(branch);

  }
  
  public void gtEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtEqZero(branch);
  }
  
  public void gtEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitGtEqZero(branch);
  }

  public void ltCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtZero(branch);
  }
  
  public void ltCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtZero(branch);

  }
  
  public void ltCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtZero(branch);
  }
  
  public void ltCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtZero(branch);
  }

  public void ltEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtEqZero(branch);
  }
  
  public void ltEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtEqZero(branch);

  }
  
  public void ltEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtEqZero(branch);
  }
  
  public void ltEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.emitDCMPL();
    compiler.emitLtEqZero(branch);
  }

  public void addInt(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.doubleAdd();
  }

  public void addDouble(SimpleNode lhsn, SimpleNode rhsn, 
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.doubleAdd();
  }

  public void addLong(SimpleNode lhsn, SimpleNode rhsn, 
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.doubleAdd();
  }

  public void addFloat(SimpleNode lhsn, SimpleNode rhsn, 
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.doubleAdd();
  }

  public void addString(SimpleNode lhsn, SimpleNode rhsn,
			NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {double.class});
    rhsn.compile(compiler);
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class}); 
  }

  public void subInt(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.doubleSub();
  }

  public void subDouble(SimpleNode lhsn, SimpleNode rhsn, 
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.doubleSub();
  }

  public void subLong(SimpleNode lhsn, SimpleNode rhsn, 
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.doubleSub();
  }

  public void subFloat(SimpleNode lhsn, SimpleNode rhsn, 
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.doubleSub();
  }

  public void multInt(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.doubleMult();
  }

  public void multDouble(SimpleNode lhsn, SimpleNode rhsn, 
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.doubleMult();
  }

  public void multLong(SimpleNode lhsn, SimpleNode rhsn, 
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.doubleMult();
  }

  public void multFloat(SimpleNode lhsn, SimpleNode rhsn, 
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.doubleMult();
  }

  public void divInt(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    intOp(lhsn, rhsn, compiler);
    compiler.doubleDiv();
  }

  public void divDouble(SimpleNode lhsn, SimpleNode rhsn, 
			  NQPyBlockCompiler compiler)
  {
    doubleOp(lhsn, rhsn, compiler);
    compiler.doubleDiv();
  }

  public void divLong(SimpleNode lhsn, SimpleNode rhsn, 
			NQPyBlockCompiler compiler)
  {
    longOp(lhsn, rhsn, compiler);
    compiler.doubleDiv();
  }

  public void divFloat(SimpleNode lhsn, SimpleNode rhsn, 
			 NQPyBlockCompiler compiler)
  {
    floatOp(lhsn, rhsn, compiler);
    compiler.doubleDiv();
  }
  
  public void modInt(SimpleNode lhsn, SimpleNode rhsn, NQPyBlockCompiler compiler) {
    intOp(lhsn, rhsn, compiler);
    compiler.doubleMod();
  }
  
  public void modDouble(SimpleNode lhsn, SimpleNode rhsn, NQPyBlockCompiler compiler) {
    doubleOp(lhsn, rhsn, compiler);
    compiler.doubleMod();
  }
  
  public void modFloat(SimpleNode lhsn, SimpleNode rhsn, NQPyBlockCompiler compiler) {
    floatOp(lhsn, rhsn, compiler);
    compiler.doubleMod();
  }
  
  public void modLong(SimpleNode lhsn, SimpleNode rhsn, NQPyBlockCompiler compiler) {
    longOp(lhsn, rhsn, compiler);
    compiler.doubleMod();
  }

  public void neg(SimpleNode node, NQPyBlockCompiler compiler) {
    node.compile(compiler);
    compiler.doubleNeg();
  }  
}
