package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.Label;

public interface Emitter {
  
  /**
   * Gets the result type (as an NQPyClass) of arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType(NQPyClass rhs);

  /**
   * Gets the result type (as an NQPyClass) of single arg arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType();
  
  /**
   * Emits the appropriatxe byte code to go to the specified branch
   * if the specified node compiles to true. This tests whether
   * the result of node.compile() is equal to zero, and branches if not so.
   */
  public void branchOnTrue(SimpleNode node, NQPyBlockCompiler compiler,
			   Label branch);

  /**
   * Emits the appropriate byte code to go to the specified branch
   * if the single node in nodes compiles to false. This tests whether
   * the result of node.compile() is equal to zero, and branches if so.
   */
  public void branchOnFalse(SimpleNode node, NQPyBlockCompiler compiler,
			    Label branch);
  
  public void eq(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn, Label branch,
		 NQPyBlockCompiler compiler);
  public void eqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void eqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void eqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void eqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void eqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void eqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void eqCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void notEq(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		    Label branch, NQPyBlockCompiler compiler);
  public void notEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void notEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void notEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void notEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void notEqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void notEqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void notEqCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void gt(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		    Label branch, NQPyBlockCompiler compiler);
  public void gtCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void gtCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void gtCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void gtCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void gtCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void gtCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void gtCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void lt(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		    Label branch, NQPyBlockCompiler compiler);
  public void ltCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void ltCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void ltCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void ltCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void ltCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void ltCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void ltCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void gtEq(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		    Label branch, NQPyBlockCompiler compiler);
  public void gtEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void gtEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void gtEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void gtEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void gtEqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void gtEqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void gtEqCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void ltEq(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		    Label branch, NQPyBlockCompiler compiler);
  public void ltEqCmpInt(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler);
  public void ltEqCmpDouble(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void ltEqCmpLong(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);
  public void ltEqCmpFloat(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			 NQPyBlockCompiler compiler);
  public void ltEqCmpBoolean(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			   NQPyBlockCompiler compiler);
  public void ltEqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			  NQPyBlockCompiler compiler);
  public void ltEqCmpJava(SimpleNode lhsn, SimpleNode rhsn, Label branch,
			NQPyBlockCompiler compiler);

  public void add(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		  NQPyBlockCompiler compiler);
  public void sub(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		  NQPyBlockCompiler compiler);
  public void mult(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		  NQPyBlockCompiler compiler);
  public void div(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		  NQPyBlockCompiler compiler);
  public void mod(NQPyClass rhs, SimpleNode lhsn, SimpleNode rhsn,
		  NQPyBlockCompiler compiler);

  /**
   * Negative operation (-2).
   */
  public void neg(SimpleNode node, NQPyBlockCompiler compiler);
}
