package uchicago.src.nqpy.compiler;

import java.util.ArrayList;
import java.io.*;

import uchicago.src.nqpy.parser.SimpleNode;
import uchicago.src.nqpy.parser.ParseException;

/**
 * Creates compiler, and parser exceptions
 */

public class ErrorFactory {

  private static SimpleNode curNode = null;
  private static String blockName = "";
  private static String code = "";

  private ErrorFactory() {}

  public static void setBlockName(String val) {
    blockName = val;
  }

  public static void setCode(String val) {
    code = val;
  }

  public static void setCurNode(SimpleNode val) {
    curNode = val;
  }

  public static CompilerException createCompilerException(String message) {
    return createCompilerException(message, null);
  }

  public static CompilerException createCompilerException(String message,
							  Exception exc)
  {
    CompilerException ex = new CompilerException(message, exc);
    ex.setBlockName(blockName);
    if (curNode != null) {
      ex.setLineNum(curNode.beginLine);
      ex.setLine(createLine(curNode.beginLine));
    }
    return ex;
  }

  private static String createLine(int lineNum) {
    String line = "";
    try {
	BufferedReader in = new BufferedReader(new StringReader(code));
	for (int i = 0; i < lineNum; i++) {
	  line = in.readLine();
	  
	}
      } catch (IOException ex1) {}
    return line;
  }

  public static ParserException createParserException(ParseException ex)
  {
    
    ArrayList expected = new ArrayList();
    int[][] tokseq = ex.expectedTokenSequences;
    for (int i = 0; i < tokseq.length; i++) {
      for (int j = 0; j < tokseq[i].length; j++) {
	expected.add(ex.tokenImage[tokseq[i][j]]);
      }
    }

    String[] exp = new String[expected.size()];
    for (int i = 0; i < expected.size(); i++) {
      exp[i] = (String)expected.get(i);
    }
      

    return new ParserException(createLine(ex.currentToken.beginLine),
			       ex.currentToken.beginLine,
			       ex.currentToken.endColumn,
			       exp, blockName);
  } 
}
