package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.*;

/**
 * Handles the built-in function float(x) where x is a number or a
 * string.
 */
public class FloatBuiltInFunc extends AbstractBuiltInFunc {

  private NQPyClass arg;
  private NQPyClass retType;
  private ASTarglist args;

  public NQPyClass getNQPyClass(NQPyBlockCompiler compiler, ASTarglist args) {
    argSizeCheck("float", args, 1);
    arg = args.getArgTypes(compiler)[0];
    argNumBoolStringCheck("float", arg);
    this.args = args;
    retType = NQPyClassFactory.makeDoubleClass();
    return retType;
  }

  public void compile(NQPyBlockCompiler compiler) {
    args.jjtGetChild(0).compile(compiler);
    // do conversion depending on type
    switch (arg.id()) {
      case NQPyClass.LONG_CLASS:
        compiler.emitPrimitiveConversion(long.class, double.class);
        break;

      case NQPyClass.INT_CLASS:
      case NQPyClass.BOOLEAN_CLASS:
        compiler.emitPrimitiveConversion(int.class, double.class);
        break;


      case NQPyClass.FLOAT_CLASS:
        compiler.emitPrimitiveConversion(float.class, double.class);
        break;

      case NQPyClass.STRING_CLASS:
        compiler.emitInvokeStatic("java.lang.Double", "parseDouble",
                                  retType, new Class[]{String.class});
        break;


      case NQPyClass.DOUBLE_CLASS:
        break;

      default:
        String message = "Argument to int() must be a long, double, float, " +
                "String or boolean";
        throw ErrorFactory.createCompilerException(message);
    }
  }
}
