package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.*;

/**
 * Handles the built-in function int(x) where x is a number or a
 * string.
 */
public class IntBuiltInFunc extends AbstractBuiltInFunc {

  private NQPyClass arg;
  private NQPyClass retType;
  private ASTarglist args;

  public NQPyClass getNQPyClass(NQPyBlockCompiler compiler, ASTarglist args) {
    argSizeCheck("int", args, 1);
    arg = args.getArgTypes(compiler)[0];
    argNumBoolStringCheck("int", arg);
    this.args = args;
    retType = NQPyClassFactory.makeIntegerClass();
    return retType;
  }
  
  public void compile(NQPyBlockCompiler compiler) {
    args.jjtGetChild(0).compile(compiler);
    // do conversion depending on type
    switch (arg.id()) {
    case NQPyClass.LONG_CLASS:
	compiler.emitPrimitiveConversion(long.class, int.class);
	break;

    case NQPyClass.DOUBLE_CLASS:
      compiler.emitPrimitiveConversion(double.class, int.class);
      break;

    case NQPyClass.FLOAT_CLASS:
      compiler.emitPrimitiveConversion(float.class, int.class);
      break;
      
    case NQPyClass.STRING_CLASS:
      compiler.emitInvokeStatic("java.lang.Integer", "parseInt",
				retType, new Class[]{String.class});
      break;

    case NQPyClass.INT_CLASS:
    case NQPyClass.BOOLEAN_CLASS:
      break;

    default:
      String message = "Argument to int() must be a long, double, float, " +
	"String or boolean";
      throw ErrorFactory.createCompilerException(message);
    }
  }
}
