package uchicago.src.nqpy.compiler;

//import java.lang.reflect.*;

import java.util.ArrayList;

public class JavaMethod {

  private ArrayList args = new ArrayList();
  private Class retClass;
  private String retClassName;
  private String name;

  public JavaMethod(String name) {
    this.name = name;
  }

  public void addArgs(Class[] cs, Class decClass, int mods, String retType) {
    JavaArgs ja = new JavaArgs(cs, mods, retType);
    ja.setDeclaringClass(decClass);
    args.add(ja);

  }

  public void addArgs(String[] str, String decClassName, int mods, String retType) {
    JavaArgs ja = new JavaArgs(str, mods, retType);
    ja.setDeclaringClass(decClassName);
    args.add(ja);
  }

  public void setRetType(Class c) {
    retClass = c;
  }

  public void setRetType(String name) {
    retClassName = name;
  }

  public String getName() {
    return name;
  }

  public MethodCall getMethodCall(NQPyClass[] margs) {
    ArrayList mcalls = new ArrayList();
    for (int i = 0; i < args.size(); i++) {
      JavaArgs jargs = (JavaArgs) args.get(i);
      //System.out.println("meth name: " + name);
      //System.out.println(jargs);
      if (jargs.size() == margs.length) {
        MethodCall mc = jargs.methodCall(margs);
        
        /*
        if (retClass == null)
          mc = jargs.methodCall(margs, retClassName);
        else
          mc = jargs.methodCall(margs, retClass);
          */
        if (mc.isValid()) {
          mc.setMethodName(name);
          mcalls.add(mc);
        }
      }
    }

    //System.out.println("mcalls.size: " + mcalls.size());
    
    if (mcalls.size() == 1)
      return (MethodCall) mcalls.get(0);
    else if (mcalls.size() > 1) {
      MethodCall closest = (MethodCall) mcalls.get(0);
      for (int i = 1; i < mcalls.size(); i++) {
        MethodCall mc = (MethodCall) mcalls.get(i);
        if (mc.size() < closest.size())
          closest = mc;
      }
      return closest;
    }

    MethodCall call = new ErrorMethodCall();
    call.setMethodName(name);
    return call;
  }
}

class JavaArgs {

  NQPyClass[] args;
  Class decClass;
  int mods;

  // used if declaring class is dynamically created.
  String decClassName;
  String retType;

  public JavaArgs(Class[] cs, int mods, String retType) {
    int length = cs.length;
    args = new NQPyClass[length];
    for (int i = 0; i < length; i++) {
      Class c = cs[i];
      args[i] = NQPyClassFactory.makeNQPyClass(c);
    }
    this.mods = mods;
    this.retType = retType;
  }

  public JavaArgs(String[] strs, int mods, String retType) {
    int length = strs.length;
    args = new NQPyClass[length];
    for (int i = 0; i < length; i++) {
      args[i] = NQPyClassFactory.makeNQPyClass(strs[i]);
    }

    this.mods = mods;
    this.retType = retType;
  }

  public void setDeclaringClass(Class c) {
    decClass = c;
  }

  public void setDeclaringClass(String clzname) {
    decClassName = clzname;
  }

  public int size() {
    return args.length;
  }

  /*
   * Will return null if no match. Assumes that nargs.length == args.length.
   */
  public MethodCall methodCall(NQPyClass[] nargs) {
    MethodCall mc = new MethodCall();
    mc.setRetType(retType);
    return searchForMethod(nargs, mc);
  }

  private MethodCall searchForMethod(NQPyClass[] nargs, MethodCall mc) {
    for (int i = 0; i < nargs.length; i++) {
      //System.out.println("args[i]: " + args[i]);
      //System.out.println("nargs[i]: " + nargs[i]);
      NQPyClass matchClass = nargs[i].match(args[i]);
      //System.out.println("matchClass: " + matchClass);
      if (matchClass == null) return new ErrorMethodCall();
      if (!matchClass.equals(nargs[i])) mc.addArgConversion(i, matchClass);
    }

    mc.setValid(true);
    mc.setArgs(args);
    mc.setModifiers(mods);
    if (decClass == null) {
      mc.setDeclaringClass(NQPyClassFactory.makeNQPyClass(decClassName));
    } else {
      mc.setDeclaringClass(NQPyClassFactory.makeNQPyClass(decClass));
    }

    return mc;
  }

  public String toString() {
    StringBuffer b = new StringBuffer("(");
    for (int i = 0; i < args.length; i++) {
      if (i != 0) b.append(", ");
      b.append(args[i].getJClassName());
    }

    b.append(")");
    return b.toString();
  }
}
      
