package uchicago.src.nqpy.compiler;

//import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.TypeDescriptor;

public interface NQPyClass {

  public static final int LONG_CLASS = 0;
  public static final int INT_CLASS = 1;
  public static final int DOUBLE_CLASS = 2;
  public static final int FLOAT_CLASS = 3;
  public static final int BOOLEAN_CLASS = 4;
  public static final int STRING_CLASS = 5;
  public static final int JAVA_CLASS = 6;
  public static final int NULL_CLASS = 7;
  public static final int VOID_CLASS = 8;
  public static final int DYNAMIC_CLASS = 9;
  public static final int NQPY_CLASS = -1;

  public int id();

  /**
   * Returns a String representation of the Node associated with this
   * NQPyClass
   */
  //public String toJString();

  /**
   * Returns the Java class associated with this NQPyClass. This may
   * sometimes be null.
   */
  public Class getJClass();

  /**
   * Returns the name of the Java class associated with this NQPyClass.
   * This should never be null.
   */
  public String getJClassName();

  /**
   * Makes a TypeDescriptor for the java primitive or class that this
   * NQPyClass represents.
   */
  public TypeDescriptor makeTypeDescriptor();

  /**
   * Returns whether the java primitive or class represented by this
   * NQPyClass is assignable from that of the specified NQPyClass.
   */
  public boolean isAssignableFrom(NQPyClass clazz);

  /**
   * Returns whether this NQPyClass can be used as a boolean.
   */
  public boolean canBeBool();


  /**
   * Performs a match of this NQPyClass to the specified NQPyClass. If there
   * is an exact match then this returns this NQPyClass. If there
   * is a match, but requires conversion of the java primitive or class
   * represented by this NQPyClass then the passed in NQPyClass is returned.
   * If there is no match at all, then null is returned.<p>
   */
  public NQPyClass match(NQPyClass nqclass);

  /**
   * Returns the MethodCall object for the constructor of the
   * java object represented by the NQPyClass. If no such constructor
   * exists, then this returns ErrorMethodCall.
   */
  public MethodCall getConstructorCall(NQPyClass[] args);

  /**
   * Returns the MethodCall object for the specified method of the
   * java object represented by the NQPyClass. If no such method exists,
   * then this returns ErrorMethodCall.
   */
  public MethodCall getMethodCall(String name, NQPyClass[] args);

  /**
   * Return the FieldCall object for the specified field of the java
   * object represented by this NQPyClass. If no such field exists,
   * this returns null.
   */
  public FieldCall getFieldCall(String name, String callerClassName);

  /**
   * Returns true if this NQPyClass is the same as the specified NQPyClass
   * <b>or</b> if the java Class represented by the specified NQPyClass
   * is a parent of the Java Class represented by this one.
   */
  public boolean inObjHeirarchy(NQPyClass nqclass);

  /**
   * Emit the byte code to coerce this NQPyClass to the
   * specifed NQPyClass.
   */
  public void coerce(NQPyBlockCompiler compiler, NQPyClass toCoerce);

  /**
   * Returns true if this NQPyClass can be coerced to the specified
   * NQPYClass.
   */
  public boolean isCoercePossible(NQPyClass toCoerce);


  /**
   * Gets the Emitter appropriate for this NQPyClass.
   */
  public Emitter getEmitter();

  /**
   * Visits the equals method of the passed in NQPyClass.
   */
  public void eq(NQPyClass nqclass);

}
