package uchicago.src.nqpy.compiler;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Map;
import java.util.Iterator;

public class NQPyClassFactory {

  private static final String STRING = "java.lang.String";
  private static final String LONG = "long";
  private static final String DOUBLE = "double";
  private static final String INT = "int";
  private static final String FLOAT = "float";
  private static final String BOOLEAN = "boolean";
  private static final String VOID = "void";

  private static HashMap cache = new HashMap();
  private static Map dynCache = new HashMap();
  private static NQPyClass voidClass = new NQPyVoid();

  // key is name, val is the NQPyDynamicClass
  //private static Hashmap dynTypes = new Hashmap();

  private static ArrayList imports = new ArrayList();

  private static ClassLoader clazzLoader = NQPyClassFactory.class.getClassLoader();

  static {
    imports.add("uchicago.src.nqpy.compiler.");
    imports.add("uchicago.src.sim.util.");
    imports.add("uchicago.src.sim.engine.");
    imports.add("uchicago.src.sim.network.");
    imports.add("uchicago.src.sim.analysis.");
    imports.add("uchicago.src.sim.games.");
    imports.add("uchicago.src.sim.gui.");
    imports.add("uchicago.src.simbuilder.beans.");
    imports.add("uchicago.src.simbuilder.beans.network.");
    imports.add("uchicago.src.simbuilder.beans.gis.");
    imports.add("uchicago.src.simbuilder.beans.grid.");
    imports.add("uchicago.src.sim.space.");
    imports.add("java.lang.");
    imports.add("java.util.");
    imports.add("uchicago.src.nqpy.builtin.");
    imports.add("");
  }

  private NQPyClassFactory() {
  }

  public static void setClassLoader(ClassLoader loader) {
    NQPyClassFactory.clazzLoader = loader;
  }

  public static void addDynamicType(String name, NQPyDynamicClass type) {
    cache.put(name, type);
    dynCache.put(name, type);
  }
  
  public static NQPyDynamicClass getDynamicType(String name) {
    return (NQPyDynamicClass) dynCache.get(name);
  }

  public static void removeDynamicType(String name) {
    cache.remove(name);
    if (name.lastIndexOf(".") != -1) {
      String key = name.substring(name.lastIndexOf(".") + 1, name.length());
      cache.remove(key);
      dynCache.remove(key);
    }
  }
  
  public static void removeAllDynamicTypes() {
    for (Iterator iter = dynCache.keySet().iterator(); iter.hasNext(); ) {
      cache.remove(iter.next());
    }
    dynCache.clear();
  }
  
  private static void doAddImport(String name) {
    if (!imports.contains(name)) imports.add(name);
  }

  public static void addImport(String name) {
    if (name.endsWith("*"))
      doAddImport(name.substring(0, name.length() - 1));
    else {
      makeJavaClass(name);
      if (name.indexOf(".") != -1) {
        doAddImport(name.substring(0, name.lastIndexOf(".") + 1));
      } else
        doAddImport(name);
    }
  }

  public static NQPyClass makeStringClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(STRING);
    if (nqclass == null) {
      nqclass = new NQPyStringClass();
      cache.put(STRING, nqclass);
    }

    return nqclass;
  }

  public static NQPyClass makeBooleanClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(BOOLEAN);
    if (nqclass == null) {
      nqclass = new NQPyBooleanClass();
      cache.put(BOOLEAN, nqclass);
    }
    return nqclass;
  }

  public static NQPyClass makeDoubleClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(DOUBLE);
    if (nqclass == null) {
      nqclass = new NQPyDoubleClass();
      cache.put(DOUBLE, nqclass);
    }
    return nqclass;
  }

  public static NQPyClass makeFloatClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(FLOAT);
    if (nqclass == null) {
      nqclass = new NQPyFloatClass();
      cache.put(FLOAT, nqclass);
    }
    return nqclass;
  }

  public static NQPyClass makeLongClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(LONG);
    if (nqclass == null) {
      nqclass = new NQPyLongClass();
      cache.put(LONG, nqclass);
    }
    return nqclass;
  }

  public static NQPyClass makeIntegerClass() {
    NQPyClass nqclass = (NQPyClass) cache.get(INT);
    if (nqclass == null) {
      nqclass = new NQPyIntClass();
      cache.put(INT, nqclass);
    }
    return nqclass;
  }

  public static NQPyClass makeVoidClass() {
    return voidClass;
  }


  public static NQPyClass makeNQPyClass(Class c) {
    String name = c.getName();

    if (name.equals(STRING)) return makeStringClass();
    if (name.equals(LONG)) return makeLongClass();
    if (name.equals(DOUBLE)) return makeDoubleClass();
    if (name.equals(FLOAT)) return makeFloatClass();
    if (name.equals(INT)) return makeIntegerClass();
    if (name.equals(BOOLEAN)) return makeBooleanClass();
    if (name.equals(VOID)) return voidClass;

    NQPyClass nqclass = (NQPyClass) cache.get(name);
    if (nqclass == null) {
      NQPyJavaClass jclass = new NQPyJavaClass(c);
      cache.put(name, jclass);
      nqclass = jclass;
    }
    return nqclass;
  }

  public static NQPyClass makeNQPyClass(String name) {
    if (name.equals(STRING)) return makeStringClass();
    if (name.equals(LONG)) return makeLongClass();
    if (name.equals(DOUBLE)) return makeDoubleClass();
    if (name.equals(FLOAT)) return makeFloatClass();
    if (name.equals(INT)) return makeIntegerClass();
    if (name.equals(BOOLEAN)) return makeBooleanClass();
    if (name.equals(VOID)) return voidClass;

    NQPyClass nqclass = (NQPyClass) cache.get(name);
    if (nqclass == null) {
      //if (name.equals("String")) throw new RuntimeException();

      nqclass = makeJavaClass(name);

    }

    return nqclass;
  }


  /**
   * Creates an NQPyClass object for the specifed name. This will
   * try to qualify the name.
   */
  public static NQPyClass makeJavaClass(String name) {
    NQPyClass nqclass = searchCacheForName(name);
    if (nqclass == null) {
      if (name.indexOf(".") != -1) {
        // assume qualified
        try {
          Class clazz = Class.forName(name, true, clazzLoader);
          NQPyJavaClass jclass = new NQPyJavaClass(clazz);
          cache.put(name, jclass);
          nqclass = jclass;
          return nqclass;
        } catch (ClassNotFoundException ex) {
        }
      }


      for (int i = 0; i < imports.size(); i++) {

        String fullName = (String) imports.get(i) + name;

        try {
          Class clazz = Class.forName(fullName, true, clazzLoader);
          NQPyJavaClass jclass = new NQPyJavaClass(clazz);
          cache.put(fullName, jclass);
          nqclass = jclass;
          break;
        } catch (ClassNotFoundException ex) {
        }
      }
    }

    return nqclass;
  }

  private static NQPyClass searchCacheForName(String name) {
    // perhaps name is already qualifed
    if (name.indexOf(".") != -1) {
      // assume already qualifed
      NQPyClass nqpyclass = (NQPyClass) cache.get(name);
      if (nqpyclass != null) return nqpyclass;
    }

    int size = imports.size();
    for (int i = 0; i < size; i++) {
      String fullName = imports.get(i) + name;
      NQPyClass nqpyclass = (NQPyClass) cache.get(fullName);
      if (nqpyclass != null) return nqpyclass;
    }

    return null;
  }
}
