package uchicago.src.nqpy.compiler;

import java.util.*;

//import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.TypeDescriptor;

import java.lang.reflect.*;

public class NQPyDynamicClass extends AbstractNQPyClass {
  
  public static final int PUBLIC = 0;
  public static final int PRIVATE = 1;
  public static final int PROTECTED = 2;
  
  private static int pubMod = 0;
  private static int privateMod = 0;
  private static int protectedMod = 0;

  protected ArrayList dynMethods = new ArrayList();
  protected HashMap methods = new HashMap();
  protected HashMap dynFields = new HashMap();
  protected HashMap fields = new HashMap();
  protected Class superClass = null;
  protected String superClassName;
  protected NQPyDynamicClass dynamicSuperClass;
  protected boolean initialized = false;
  protected String pkg = "";
  protected String uqJClassName;
  
  static {
    try {
      Method toString = String.class.getMethod("toString", null);
      pubMod = toString.getModifiers();
      
      Method[] methods = NQPyDynamicClass.class.getDeclaredMethods();
      for (int i = 0; i < methods.length; i++) {
        Method m = methods[i];
        if (m.getName().equals("initMethodsFields")) {
          privateMod = m.getModifiers();
          break;
        }
      }
      
      Field field = NQPyDynamicClass.class.getDeclaredField("pkg");
      protectedMod = field.getModifiers();
      
    } catch (NoSuchMethodException ex) {
      ex.printStackTrace();
    } catch (NoSuchFieldException e) {
      e.printStackTrace();  
    }
  }

  class DynMethod {
    String name, retType;
    String[] args;

    public DynMethod(String n, String rt, String[] args) {
      name = n;
      retType = rt;
      this.args = args;
    }
  }
  
  class DynField {
    String name, type;
    int mod;

    public DynField(int mod, String name, String type) {
      this.mod = mod;
      this.name = name;
      this.type = type;
    }
  }
  
  public NQPyDynamicClass(String name, String superClassName) {
    this(name);
    this.superClassName = superClassName;
    
  }

  public NQPyDynamicClass(String name, Class superClass) {
    this(name);
    this.superClass = superClass;
  }
  
  private NQPyDynamicClass(String name) {
    jclass = null;
    jclassName = name;
    id = DYNAMIC_CLASS;

    int index = name.lastIndexOf(".");
    if (index != -1) {
      pkg = name.substring(0, index);
      uqJClassName = name.substring(index + 1, name.length());
    } else {
      uqJClassName = jclassName;
    }
  }

  public String getPackage() {
    return pkg;
  }

  public void init() {
    if (!initialized) {
      if (superClassName != null) dynamicSuperClass = NQPyClassFactory.getDynamicType(superClassName);
      initMethodsFields();
      initialized = true;
    }
  }

  private void initMethodsFields() {
    if (superClass != null) processJavaSuperClass();
    else processDynamicSuperClass();

    for (int i = 0; i < dynMethods.size(); i++) {
      DynMethod dm = (DynMethod)dynMethods.get(i);
      JavaMethod jm  = (JavaMethod)methods.get(dm.name);
      if (jm == null) {
        jm = new JavaMethod(dm.name);
        jm.setRetType(dm.retType);
        methods.put(dm.name, jm);
      }

      jm.addArgs(dm.args, jclassName, pubMod, dm.retType);
    }

    // by the time initFields is called NQPyClassFactory should now about
    // all the DynamicTypes so it should be okay to reference them in
    // DynFields.
    Iterator i = dynFields.keySet().iterator();
    while (i.hasNext()) {
      String name = (String)i.next();
      DynField field  = (DynField) dynFields.get(name);
      fields.put(name, new JavaField(name, field.type, getMod(field.mod) , jclassName));
    }
  }
  
  private void processDynamicSuperClass() {
    dynamicSuperClass.init();
    methods.putAll(dynamicSuperClass.methods);
    fields.putAll(dynamicSuperClass.fields);
  }

  private void processJavaSuperClass() {
    // put in list so can iterate backwards through the list
    // in order that child methods with same signature as parent
    // will replace parent method.
    ArrayList classList = new ArrayList();
    Class cl = superClass;
    while (cl != null) {
      classList.add(cl);
      cl = cl.getSuperclass();
    }

    for (int i = classList.size() - 1; i > -1; i--) {
      Class c = (Class)classList.get(i);
      Method[] ms = c.getDeclaredMethods();
      for (int j = 0; j < ms.length; j++) {
        Method m = ms[j];
        int mods = m.getModifiers();
        if (Modifier.isPublic(mods) || Modifier.isProtected(mods)) {

          String methName = m.getName();
          JavaMethod jm = (JavaMethod)methods.get(methName);
          if (jm == null) {
            jm = new JavaMethod(methName);
            methods.put(methName, jm);
            jm.setRetType(m.getReturnType());
          }
          jm.addArgs(m.getParameterTypes(), m.getDeclaringClass(), mods, m.getReturnType().getName());
        }
      }

      Field[] fs = c.getDeclaredFields();
      for (int j = 0; j < fs.length; j++) {
        Field f = fs[j];
        int mods = f.getModifiers();
        fields.put(f.getName(),
                   new JavaField(f.getName(), f.getType(), mods,
                                 f.getDeclaringClass().getName()));
      }
    }
  }

  private int getMod(int mod) {
    if (mod == PRIVATE) return privateMod;
    if (mod == PUBLIC) return pubMod;
    else return protectedMod;
  }

  public void addMethod(String name, String retType, String[] args) {
    initialized = false;
    DynMethod m = new DynMethod(name, retType, args);
    dynMethods.add(m);
  }

  public void addMethod(String name, Class retType, Class[] args) {
    initialized = false;
    DynMethod m = new DynMethod(name, retType.getName(), classToStrings(args));
    dynMethods.add(m);
  }
  
  public boolean hasDynField(String name) {
    return dynFields.containsKey(name);
  }

  private String[] classToStrings(Class[] args) {
    String[] s = new String[args.length];
    for (int i = 0; i < args.length; i++) {
      s[i] = args[i].getName();
    }
    return s;
  }
  
   public void addField(String name, String typeName, int mod) {
    initialized = false;
    dynFields.put(name, new DynField(mod, name, typeName));
  }

  public void addField(String name, Class type, int mod) {
    initialized = false;
    dynFields.put(name, new DynField(mod, name, type.getName()));
  }
  
  public void addProtectedField(String name, String typeName) {
    initialized = false;
    addField(name, typeName, PROTECTED);
  }

  public void addField(String name, String typeName) {
    initialized = false;
    addField(name, typeName, PRIVATE);
  }

  public void addField(String name, Class type) {
    initialized = false;
    addField(name, type, PRIVATE);
  }

  /**
   * Makes a TypeDescriptor for the java primitive or class that this
   * NQPyClass represents. This will use the Class of the java primitive
   * or class that this represents to create the descriptor.
   */
  public TypeDescriptor makeTypeDescriptor() {
    return new TypeDescriptor(jclassName);
  }

  /**
   * Returns whether the java primitive or class represented by this
   * NQPyClass is assignable from that of the specified NQPyClass.
   */
  public boolean isAssignableFrom(NQPyClass clazz) {
    if (equals(clazz)) return true;
    if (id() == DYNAMIC_CLASS && clazz.id() == DYNAMIC_CLASS) {
      NQPyDynamicClass arg = (NQPyDynamicClass) clazz;
      arg.init();
      // both are dynamic classes so we need to see if is this the superclass of the arg
      // if arg super class is a Class then return false as this is not a Class so 
      // can't be the superclass of arg.
      if (arg.superClass != null) return false;
      
      // arg super class must be a dynamic class so through the parents to
      // see if any one is this
      NQPyDynamicClass argSuper = arg.dynamicSuperClass;
      while (argSuper != null) {
        if (this.equals(argSuper)) return true;
        if (argSuper.superClass != null) return false;
        argSuper = argSuper.dynamicSuperClass;
      }
      // we should always exit from the loop with one of the returns,
      // but just to be sure.
      return false;
    }
    return superClass.isAssignableFrom(clazz.getJClass());
  }

  /**
   * Returns true if this NQPyClass is the same as the specified NQPyClass
   * <b>or</b> if the java Class represented by the specified NQPyClass
   * is a parent of the Java Class represented by this one. In short,
   * if the specified nqclass can be considered a superclass of this one, return true.<p>
   */
  public boolean inObjHeirarchy(NQPyClass nqclass) {
    //System.out.println("this.name: " + jclassName);
    //System.out.println("arg.name: " + nqclass.getJClassName());

    if (equals(nqclass)) return true;
    if (nqclass.id() == DYNAMIC_CLASS) {
      return nqclass.isAssignableFrom(this);
    } else
      return nqclass.getJClass().isAssignableFrom(superClass);
  }

  /**
   * Gets the MethodCall for the constructor.
   */
  public MethodCall getConstructorCall(NQPyClass[] args) {
    init();
    JavaMethod ctor = (JavaMethod)methods.get(uqJClassName);
    if (ctor == null) return new ErrorMethodCall();
    else {
      ctor.setRetType(jclassName);
      return ctor.getMethodCall(args);
    }
  }

  /**
   * Returns the MethodCall object for the specified method of the
   * java object represented by the NQPyClass. If no such method exists,
   * then this returns ErrorMethodCall.
   */
  public MethodCall getMethodCall(String name, NQPyClass[] args) {
    init();
    JavaMethod meth = (JavaMethod)methods.get(name);
    //System.out.println("methName: " +  name);
    //System.out.println("methods.size(): " + methods.size());
    //System.out.println(meth);

    if (meth == null) return new ErrorMethodCall();
    return meth.getMethodCall(args);
  }

  public FieldCall getFieldCall(String name, String callerClassName) {
    init();
    //uchicago.src.debug.DebugMethods.printHashMap(fields);
    JavaField f = (JavaField)fields.get(name);
    if (f  == null) return null;
    if (f.getFieldCall().isPrivate() || f.getFieldCall().isProtected()) {
      // make sure that callerClass is of the same class as 
      // this one
      if (!jclassName.equals(callerClassName)) return null;
    }
    
    return f.getFieldCall();
  }

  public boolean equals(Object o) {
    if (o instanceof NQPyDynamicClass) {
      NQPyDynamicClass nq = (NQPyDynamicClass)o;
      return nq.jclassName.equals(jclassName);
    }
    return false;
  }

  public Emitter getEmitter() {
    return new JavaEmitter(this);
  }
}
