package uchicago.src.nqpy.compiler;

//import uchicago.src.nqpy.parser.SimpleNode;

public class NQPyIntClass extends NQPyNumberClass {

  public NQPyIntClass() {
    jclass = int.class;
    jclassName = jclass.getName();
    id = INT_CLASS;
  }

  /**
   * Returns true if this NQPyIntClass can be coerced to the specified
   * NQPyClass.
   */
  public boolean isCoercePossible(NQPyClass toCoerce) {
    switch (toCoerce.id()) {
    case BOOLEAN_CLASS:
      return true;
    case DOUBLE_CLASS:
      return true;
    case FLOAT_CLASS:
      return true;
    case LONG_CLASS:
      return true;
    default:
        return false;
    }

  }

  /**
   * Emit the byte code to coerce this NQPyClass to the
   * specifed NQPyClass. Assumes the int is on the stack already.
   */
  public void coerce(NQPyBlockCompiler compiler, NQPyClass toCoerce) {
    switch (toCoerce.id()) {
    case BOOLEAN_CLASS:
      compiler.emitIntToBoolean();
      break;
    case DOUBLE_CLASS:
      compiler.emitPrimitiveConversion(int.class, double.class);
      break;
    case FLOAT_CLASS:
      compiler.emitPrimitiveConversion(int.class, float.class);
      break;
    case LONG_CLASS:
      compiler.emitPrimitiveConversion(int.class, long.class);
      break;
    default:
      String message = "Unable to coerce " + this + " to " + toCoerce;
      throw ErrorFactory.createCompilerException(message);
    }
  }

  /**
   * Performs a match of this NQPyClass to the specified NQPyClass. If there
   * is an exact match then this returns this NQPyClass. If there
   * is a match, but requires conversion of the java primitive or class
   * represented by this NQPyClass then the passed in NQPyClass is returned.
   * If there is no match at all, then null is returned.<p>
   *
   * This indicates that numbers can convert to booleans.
   */
  public NQPyClass match(NQPyClass nqclass) {
    if (nqclass.id() == LONG_CLASS || nqclass.id() == DOUBLE_CLASS ||
	nqclass.id() == FLOAT_CLASS) return nqclass;
    return super.match(nqclass);
  }


  /**
   * Gets the Emitter appropriate for this NQPyIntClass.
   */
  public Emitter getEmitter() {
    return new IntEmitter(this);
  }

  /**
   * Visits the equals method of the passed in NQPyClass.
   */
  public void eq(NQPyClass nqclass) {

  }
}
