package uchicago.src.nqpy.compiler;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;

public class NQPyJavaClass extends AbstractNQPyClass {

  protected HashMap methods = new HashMap();
  protected HashMap fields = new HashMap();
  protected boolean initialized = false;

  public NQPyJavaClass(Class clazz) {
    jclass = clazz;
    jclassName = clazz.getName();
    id = JAVA_CLASS;
    canBeBool = true;
  }

  public void init() {
    if (!initialized) {
      initMethods();
      initFields();
      initialized = true;
    }
  }

  private void initMethods() {

    Method[] ms = jclass.getMethods();
    for (int i = 0; i < ms.length; i++) {
      Method m = ms[i];
      String methName = m.getName();
      JavaMethod jm = (JavaMethod) methods.get(methName);
      int mods = m.getModifiers();
      if (jm == null) {
        jm = new JavaMethod(methName);
        methods.put(methName, jm);
      }

      jm.addArgs(m.getParameterTypes(), m.getDeclaringClass(), mods, m.getReturnType().getName());
    }

    if (jclass.isInterface()) {
      // we need to add the Object methods so that we can call those from an Interface instance
      ms = Object.class.getMethods();
      for (int i = 0; i < ms.length; i++) {
        Method m = ms[i];
        String methName = m.getName();
        JavaMethod jm = (JavaMethod) methods.get(methName);
        int mods = m.getModifiers();
        if (jm == null) {
          jm = new JavaMethod(methName);
          methods.put(methName, jm);
        }

        jm.addArgs(m.getParameterTypes(), m.getDeclaringClass(), mods, m.getReturnType().getName());
      }
    }

    Constructor[] ctors = jclass.getConstructors();
    for (int i = 0; i < ctors.length; i++) {
      Constructor ctor = ctors[i];
      int mods = ctor.getModifiers();
      String ctorName = ctor.getName();
      JavaMethod jm = (JavaMethod) methods.get(ctorName);
      if (jm == null) {
        jm = new JavaMethod(ctorName);
        methods.put(ctorName, jm);
      }
      jm.addArgs(ctor.getParameterTypes(), jclass, mods, jclass.getName());
    }
  }

  private void initFields() {
    Field[] fs = jclass.getFields();
    for (int i = 0; i < fs.length; i++) {
      Field f = fs[i];
      fields.put(f.getName(),
              new JavaField(f.getName(), f.getType(), f.getModifiers(),
                      f.getDeclaringClass().getName()));
    }
  }

  /**
   * Returns the MethodCall object for the constructor of the
   * java object represented by the NQPyClass. If no such constructor
   * exists, then this returns ErrorMethodCall.
   */
  public MethodCall getConstructorCall(NQPyClass[] args) {
    init();
    JavaMethod ctor = (JavaMethod) methods.get(jclass.getName());
    if (ctor == null)
      return new ErrorMethodCall();
    else {
      return ctor.getMethodCall(args);
    }
  }

  /**
   * Returns the MethodCall object for the specified method of the
   * java object represented by the NQPyClass. If no such method exists,
   * then this returns ErrorMethodCall.
   */
  public MethodCall getMethodCall(String name, NQPyClass[] args) {
    init();
    JavaMethod meth = (JavaMethod) methods.get(name);
    //System.out.println("methods = " + methods);
    //System.out.println("methName: " +  name);
    //System.out.println("methods.size(): " + methods.size());
    //System.out.println(meth);
    if (meth == null) return new ErrorMethodCall();
    return meth.getMethodCall(args);
  }

  /**
   * Return the FieldCall object for the specified field of the java
   * object represented by this NQPyClass. If no such field exists,
   * this returns null.
   */
  public FieldCall getFieldCall(String name, String callerClassName) {
    init();
    JavaField field = (JavaField) fields.get(name);
    if (field == null) return null;
    return field.getFieldCall();

  }

  public boolean equals(Object o) {
    if (o instanceof NQPyJavaClass) {
      NQPyJavaClass nq = (NQPyJavaClass) o;
      return nq.jclass == jclass;
    }
    return false;
  }

  public Emitter getEmitter() {
    return new JavaEmitter(this);
  }
}
