package uchicago.src.nqpy.compiler;

import uchicago.src.nqpy.parser.SimpleNode;
import com.go.trove.classfile.Label;

public class StringEmitter extends JavaEmitter {
  
  public StringEmitter(NQPyClass nqclass) {
    super(nqclass);
  }

  /**
   * Gets the result type (as an NQPyClass) of arithmetic operations
   * on this Emitter.
   */
  public NQPyClass getArithResType(NQPyClass rhs) { 
    switch (rhs.id()) {
    case NQPyClass.INT_CLASS:
      return NQPyClassFactory.makeStringClass();
    case NQPyClass.DOUBLE_CLASS:
      return NQPyClassFactory.makeStringClass();
    case NQPyClass.LONG_CLASS:
      return NQPyClassFactory.makeStringClass();
    case NQPyClass.FLOAT_CLASS:
      return NQPyClassFactory.makeStringClass();
    case NQPyClass.STRING_CLASS:
      return NQPyClassFactory.makeStringClass();
    case NQPyClass.BOOLEAN_CLASS:
      return NQPyClassFactory.makeStringClass();
    default:
      String message = "Arithmetic operations not supported between " +
	nqPyClass.getJClassName() + " and " + rhs.getJClassName();
      throw ErrorFactory.createCompilerException(message);
    }
  }
  
  /**
   * Emits the appropriatxe byte code to go to the specified branch
   * if the specified node compiles to true. This tests whether
   * the result of node.compile() is null and if so then don't branch. 
   * If not null then check if equal to zero, and branches if the
   * result is not equal to zero.
   */
  public void branchOnTrue(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitDup();
    Label lengthTest = compiler.createLabel();
    Label end = compiler.createLabel();
    compiler.emitIfNotNull(lengthTest);
    // if this code is executed then we know it was null
    // which is equivalent to false, so we don't go to the branch we just goto the end
    compiler.emitPop();
    compiler.emitGoto(end);
    lengthTest.setLocation();
    compiler.emitInvokeVirtual("java.lang.String", "length", int.class,
			       new Class[]{});
    compiler.emitNotEqZero(branch);
    end.setLocation();
  }

  /**
   * Emits the appropriate byte code to go to the specified branch
   * if the single node in nodes compiles to false. This tests whether
   * the result of node.compile() is null and if so then branch. If it
   * it is not null then check if length not equal to zero, and branches if so.
   */
  public void branchOnFalse(SimpleNode node, NQPyBlockCompiler compiler,
		   Label branch)
  {
    node.compile(compiler);
    compiler.emitDup();
    Label lengthTest = compiler.createLabel();
    // check if not null
    compiler.emitIfNotNull(lengthTest);
    // if this code is executed then we know the value was
    // null which is equal to false, so we want to branch to the branch
    compiler.emitPop();
    compiler.emitGoto(branch);
    compiler.emitInvokeVirtual("java.lang.String", "length", int.class,
			       new Class[]{});
    compiler.emitEqZero(branch);
  }

  public void eqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeVirtual("java.lang.Object", "equals",
			       boolean.class, new Class[] {Object.class});
    compiler.emitNotEqZero(branch);
  }

  public void notEqCmpString(SimpleNode lhsn, SimpleNode rhsn, Label branch,
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeVirtual("java.lang.Object", "equals",
			       boolean.class, new Class[] {Object.class});
    compiler.emitEqZero(branch);
  }

  public void addInt(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {int.class});
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class});
  }
    

  public void addDouble(SimpleNode lhsn, SimpleNode rhsn, 
			  NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {double.class});
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class});
  }

  public void addLong(SimpleNode lhsn, SimpleNode rhsn, 
			NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {long.class});
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class}); 
  }

  public void addFloat(SimpleNode lhsn, SimpleNode rhsn, 
			 NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {float.class});
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class}); 
  }

  public void addBoolean(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeStatic("java.lang.String", "valueOf",
			      String.class, new Class[] {boolean.class});
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class}); 
  }
  
  public void addString(SimpleNode lhsn, SimpleNode rhsn, 
		       NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    rhsn.compile(compiler);
    compiler.emitInvokeVirtual("java.lang.String", "concat", String.class,
			       new Class[] {String.class});
  }

  public void multInt(SimpleNode lhsn, SimpleNode rhsn,
			NQPyBlockCompiler compiler)
  {
    lhsn.compile(compiler);
    String stringName = compiler.createTmp(String.class);
    compiler.emitStoreTmp(stringName);
    
    compiler.emitInt(0);
    String iName = compiler.createTmp(int.class);
    compiler.emitStoreTmp(iName);
    rhsn.compile(compiler);
    String sizeName = compiler.createTmp(int.class);
    compiler.emitStoreTmp(sizeName);
    
    compiler.emitNew(StringBuffer.class);
    compiler.emitDup();
    compiler.emitConstructor("java.lang.StringBuffer", new Class[] {});
    String bufName = compiler.createTmp(StringBuffer.class);
    compiler.emitStoreTmp(bufName);
    
    Label check = compiler.createLabel();
    Label end = compiler.createLabel();
    Label start = compiler.createLabel();
    compiler.emitGoto(check);
    start.setLocation();
    compiler.emitLoadTmp(bufName);
    compiler.emitLoadTmp(stringName);
    compiler.emitInvokeVirtual("java.lang.StringBuffer", "append",
			       StringBuffer.class, new Class[] {String.class});
    compiler.emitPop();
    compiler.emitIncTmp(iName, 1);
    check.setLocation();
    compiler.emitLoadTmp(iName);
    compiler.emitLoadTmp(sizeName);
    compiler.emitIntComp(start, "<");
    end.setLocation();
    compiler.emitLoadTmp(bufName);
    compiler.emitInvokeVirtual("java.lang.StringBuffer", "toString",
    			       String.class, new Class[] {});
  }
}

