package uchicago.src.nqpy.parser;

import uchicago.src.nqpy.compiler.NQPyClass;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyClassFactory;
import uchicago.src.nqpy.compiler.ErrorFactory;

import java.util.List;

/**
 * Emits code for index subscripting on a List object.
 * 
 * $Revision: 1.1 $ $Date: 2004/10/12 13:20:19 $
 */
public class ListIndexer implements Indexer {
  
  private static final Class[] ARGS = {List.class, int.class};
  private static final Class[] SET_ARGS = {List.class, int.class, Object.class};
  private static final Class[] SUB_ARGS = {List.class, int.class, int.class};
  private static final NQPyClass OBJECT_CLASS = NQPyClassFactory.makeNQPyClass(Object.class);
  
  public NQPyClass getNQPyClass(NQPyBlockCompiler compiler) {
    return NQPyClassFactory.makeNQPyClass(Object.class);
  }
  
  public NQPyClass getNQPyClassForSlice(NQPyBlockCompiler compiler) {
    return NQPyClassFactory.makeNQPyClass(List.class);
  }

  public void compileSliceIndex(NQPyBlockCompiler compiler, SimpleNode listNode, ASTSlice sliceIndex) {
    listNode.compile(compiler);
    sliceIndex.compileForList(compiler);
    // now should be 2 ints on stack
    compiler.emitInvokeStatic("uchicago.src.nqpy.util.NQPyUtil", "sublist", List.class, ListIndexer.SUB_ARGS);
    //compiler.emitInvokeInterface("java.util.List", "subList", List.class, new Class[]{int.class, int.class});
  }

  public void compileIntIndex(NQPyBlockCompiler compiler, SimpleNode listNode, SimpleNode intIndex) {
    listNode.compile(compiler);
    intIndex.compile(compiler);
    compiler.emitInvokeStatic("uchicago.src.nqpy.util.NQPyUtil", "get", Object.class, ListIndexer.ARGS);
  }

  public void assign(NQPyBlockCompiler compiler, SimpleNode listNode, SimpleNode indexNode, SimpleNode rhs) {
    NQPyClass rhsClass = rhs.getNQPyClass(compiler);
    if (!OBJECT_CLASS.isAssignableFrom(rhsClass)) {
      ErrorFactory.setCurNode(listNode);
      String message = "List does not support item assignment of non-Object classes";
      throw ErrorFactory.createCompilerException(message);
    }
    
    listNode.compile(compiler);
    indexNode.compile(compiler);
    rhs.compile(compiler);
    compiler.emitInvokeStatic("uchicago.src.nqpy.util.NQPyUtil", "set", void.class, ListIndexer.SET_ARGS);
  }
}
