package uchicago.src.nqpy.parser;

import uchicago.src.nqpy.compiler.NQPyClass;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyClassFactory;
import uchicago.src.nqpy.compiler.ErrorFactory;
import uchicago.src.nqpy.builtin.range;

/**
 * Emits code for index subscripting on a range object.
 * 
 * $Revision: 1.1 $ $Date: 2004/10/12 13:20:19 $
 */
public class RangeIndexer implements Indexer {
  
  private static final Class[] ARGS = {int.class};
  private static final NQPyClass INT_CLASS = NQPyClassFactory.makeIntegerClass();
  private static final Class[] SET_ARGS = {int.class, int.class};

  public NQPyClass getNQPyClass(NQPyBlockCompiler compiler) {
    return INT_CLASS;
  }
  
  public NQPyClass getNQPyClassForSlice(NQPyBlockCompiler compiler) {
    return NQPyClassFactory.makeNQPyClass(uchicago.src.nqpy.builtin.range.class);
  }

  public void compileSliceIndex(NQPyBlockCompiler compiler, SimpleNode rangeNode, ASTSlice sliceIndex) {
    rangeNode.compile(compiler);
    sliceIndex.compileForRange(compiler, rangeNode);
    // now should be 2 ints on stack
    compiler.emitInvokeVirtual("uchicago.src.nqpy.builtin.range", "subrange", range.class, SET_ARGS);
  }
    
 public void compileIntIndex(NQPyBlockCompiler compiler, SimpleNode listNode, SimpleNode index) {
    listNode.compile(compiler);
    index.compile(compiler);
    compiler.emitInvokeVirtual("uchicago.src.nqpy.builtin.range", "get", int.class, ARGS);
  }

  public void assign(NQPyBlockCompiler compiler, SimpleNode listNode, SimpleNode indexNode, SimpleNode rhs) {
    NQPyClass rhsClass = rhs.getNQPyClass(compiler);
    if (!INT_CLASS.isAssignableFrom(rhsClass)) {
      ErrorFactory.setCurNode(listNode);
      String message = "range does not support item assignment of non-int objects";
      throw ErrorFactory.createCompilerException(message);
    }
    
    listNode.compile(compiler);
    indexNode.compile(compiler);
    rhs.compile(compiler);
    compiler.emitInvokeVirtual("uchicago.src.nqpy.builtin.range", "set", void.class, RangeIndexer.SET_ARGS);
  }
}
