package uchicago.src.nqpy.parser;

import uchicago.src.nqpy.compiler.NQPyClass;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyClassFactory;
import uchicago.src.nqpy.compiler.ErrorFactory;

import java.util.List;

/**
 * Emits code for index subscripting on a String.
 * 
 * $Revision: 1.1 $ $Date: 2004/10/12 13:20:05 $
 */
public class StringIndexer implements Indexer {
  
  private static final Class[] ARGS = {String.class, int.class};
  private static final Class[] SUB_ARGS = {String.class, int.class, int.class};
  
  public NQPyClass getNQPyClass(NQPyBlockCompiler compiler) {
    return NQPyClassFactory.makeNQPyClass(String.class);
  }

  public NQPyClass getNQPyClassForSlice(NQPyBlockCompiler compiler) {
    return getNQPyClass(compiler);
  }

  public void compileSliceIndex(NQPyBlockCompiler compiler, SimpleNode stringNode, ASTSlice sliceIndex) {
    stringNode.compile(compiler);
    sliceIndex.compileForString(compiler, stringNode);
    // now should be 2 ints on stack
    compiler.emitInvokeStatic("uchicago.src.nqpy.util.NQPyUtil", "substring", String.class, SUB_ARGS);
  }

  public void compileIntIndex(NQPyBlockCompiler compiler, SimpleNode stringNode, SimpleNode intIndex) {
    stringNode.compile(compiler);
    intIndex.compile(compiler);
    compiler.emitInvokeStatic("uchicago.src.nqpy.util.NQPyUtil", "get", String.class, StringIndexer.ARGS);
  }

  public void assign(NQPyBlockCompiler compiler, SimpleNode listNode, SimpleNode indexNode, SimpleNode rhs) {
    ErrorFactory.setCurNode(listNode);
    String message = "String does not support item assignment";
    throw ErrorFactory.createCompilerException(message);
  }
}
