package uchicago.src.nqpy.util;

import uchicago.src.nqpy.compiler.NQPyClass;

/**
 * Determines the component type and dimensions of an Array when that array is encapsulated by 
 * a NQPyClass.
 * 
 * @author Nick Collier
 * @version $Revision: 1.1 $ $Date: 2005/03/21 17:17:56 $
 */
public class ArrayData {
  
  private int dimensions;
  private Class componentType;
  
  /**
   * Creates an ArrayData from the specfied NQPyClass.
   * 
   * @param clazz an NQPyClass representing a Java array class.
   * @throws IllegalArgumentException if the clazz parameter does not represent a Java
   * array class.
   */ 
  public ArrayData(NQPyClass clazz) {
    if (!ArrayData.isArray(clazz)) throw new IllegalArgumentException("Parameter must be an array");
    String name = clazz.getJClass().getName();
    dimensions = name.lastIndexOf('[') + 1;
    if (name.endsWith(";")) {
      // its an Object
      try {
        componentType = Class.forName(name.substring(dimensions, name.length() - 1));
        System.out.println("componentType = " + componentType);
      } catch (ClassNotFoundException ex) {
        throw new IllegalArgumentException("Unable to determine array component type");
      }
    } else {
      if (name.equals("[B")) componentType = byte.class;
      else if (name.equals("[C")) componentType = char.class;
      else if (name.equals("[D")) componentType = double.class;
      else if (name.equals("[F")) componentType = float.class;
      else if (name.equals("[I")) componentType = int.class;
      else if (name.equals("[J")) componentType = long.class;
      else if (name.equals("[S")) componentType = short.class;
      else if (name.equals("[Z")) componentType = boolean.class;
    }
  }

  /**
   * Returns the component type of the array.
   * @return
   */ 
  public Class getComponentType() {
    return componentType;
  }

  /**
   * Returns the dimensions of the array.
   * @return
   */ 
  public int getDimensions() {
    return dimensions;
  }

  /**
   * Returns true if the parameter represents a Java array, otherwise false.
   * @param clazz
   * @return
   */ 
  public static boolean isArray(NQPyClass clazz) {
    if (clazz.getJClass() != null) {
      return clazz.getJClass().getName().startsWith("[");
    }
    
    return false;
  }
}
