package uchicago.src.nqpy.util;

import java.util.HashMap;
import java.io.ByteArrayOutputStream;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;

import com.go.trove.classfile.ClassFile;

public class NQPyClassLoader extends ClassLoader {

  private HashMap classDefs = new HashMap();
  private HashMap classes = new HashMap();

  class ClassData {
    byte[] bytecode;
    String name;

    public ClassData(String name, byte[] bytecode) {
      this.name = name;
      this.bytecode = bytecode;
    }
  }

  public NQPyClassLoader(ClassLoader parent) {
    super(parent);
  }

  public void addClass(String name, ClassFile cf) {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      cf.writeTo(out);
      out.close();
    } catch (java.io.IOException ex) {
      ex.printStackTrace();
    }

    classDefs.put(name, new ClassData(name, out.toByteArray()));
  }

  public void writeClass(String name, String dir) {
    ClassData cd = (ClassData)classDefs.get(name);
    if (cd == null) throw new RuntimeException("Class " + name + " not found");
    try {
      BufferedOutputStream out =
        new BufferedOutputStream(
          new FileOutputStream(dir + java.io.File.separator + name + ".class"));
      out.write(cd.bytecode);
      out.flush();
      out.close();
    } catch (java.io.IOException ex) {
      ex.printStackTrace();
    }
  }

  protected Class findClass(String name) throws ClassNotFoundException {
    //System.out.println("looking for class " + name);
    Class c = (Class)classes.get(name);
    if (c == null) {
      ClassData cd = (ClassData)classDefs.get(name);
      if (cd == null) throw new ClassNotFoundException(name);
      c = this.defineClass(name, cd.bytecode, 0, cd.bytecode.length);
    }

    return c;
  }
}
