package uchicago.src.simbuilder;

import uchicago.src.nqpy.compiler.AbstractCompilerException;
import uchicago.src.simbuilder.beans.ActionCodeProducer;
import uchicago.src.simbuilder.beans.EvolverProject;
import uchicago.src.simbuilder.property.PropertyTablePanel;
import uchicago.src.simbuilder.util.JarClassLoader;

import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.jar.JarFile;

/**
 * The GUI for Evolver. EvolverGui sets up and lays out the Evolver's
 * gui. Event handling etc. is handled by the components of the gui,
 * or forwarded to the EvolverController.
 * <p/>
 * $Revision: 1.4 $ $Date: 2004/12/10 22:07:34 $
 */


public class EvolverGui extends JPanel {

  private static final String OPEN_ICON = "/uchicago/src/simbuilder/images/Open.gif";
  private static final String NEW_ICON = "/uchicago/src/simbuilder/images/New.gif";
  private static final String SAVE_ICON = "/uchicago/src/simbuilder/images/Save.gif";
  private static final String HELP_ICON = "/uchicago/src/simbuilder/images/Help.gif";
  private static final String RUN_ICON = "/uchicago/src/simbuilder/images/GreenFlag.gif";
  private static final String COMPILE_ICON = "/uchicago/src/simbuilder/images/BlueFlag.gif";
  private static final String EXPORT_ICON = "/uchicago/src/simbuilder/images/DocumentOut.gif";

  private Palette palette;
  private JFrame frame;
  private String title = "Repast Py";
  EvolverBeanTree tree;
  private JScrollPane treePane;
  private PropertyTablePanel props;
  private JMenuBar menuBar;
  private Action exit;
  private JTextArea xmlArea = new JTextArea();

  public static final int YES = JOptionPane.YES_OPTION;
  public static final int NO = JOptionPane.NO_OPTION;
  public static final int CANCEL = JOptionPane.CANCEL_OPTION;

  HelpSet mainHS = null;
  HelpBroker mainHB;
  private JToolBar toolBar;


  public EvolverGui(EvolverController control) {
    try {
      ClassLoader cl = control.getClass().getClassLoader();
      URL url = HelpSet.findHelpSet(cl, "uchicago/src/simbuilder/help/SBHelp.hs");
      mainHS = new HelpSet(cl, url);
      mainHB = mainHS.createHelpBroker();
    } catch (Exception ee) {
      ee.printStackTrace();
      System.out.println("Help Set not found");
    } catch (ExceptionInInitializerError ex) {
      System.err.println("initialization error:");
      ex.getException().printStackTrace();
    }

    tree = new EvolverBeanTree();
    tree.addMouseListener(control.getTreeListener());
    tree.addKeyListener(control.getTreeKeyListener());
    palette = new Palette(control);
    exit = control.exit;
    guiInit(control);
    getProject().addPropertyChangeListener(control);
  }

  public static void showError(String message, Exception ex) {
    JFrame aFrame = new JFrame();
    String exMessage = ex.getMessage();
    if (message != null) exMessage = "No Exception Message";
    JOptionPane.showMessageDialog(aFrame, message + ": " + exMessage, "Repast Py Error",
            JOptionPane.ERROR_MESSAGE);
    aFrame.dispose();
  }

  private void guiInit(EvolverController control) {
    setLayout(new BorderLayout());

    toolBar = makeToolBar(control);
    menuBar = makeMenuBar(control);

    treePane = new JScrollPane(tree);
    JTabbedPane tabs = new JTabbedPane(JTabbedPane.BOTTOM);
    props = new PropertyTablePanel();
    tabs.addTab("Properties", props);

    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true,
            treePane, tabs);

    JPanel xML = new JPanel(new BorderLayout());
    JScrollPane sp = new JScrollPane(xmlArea);
    xML.add(sp, BorderLayout.CENTER);
    tabs.addTab("XML", xML);

    tabs.addChangeListener(control.tabChange);

    JPanel top = new JPanel(new BorderLayout());
    //top.add(palette, BorderLayout.CENTER);
    top.add(toolBar, BorderLayout.NORTH);

    add(top, BorderLayout.NORTH);
    add(split, BorderLayout.CENTER);

    split.setDividerLocation(200);
  }

  private JToolBar makeToolBar(EvolverController control) {
    JToolBar bar = new JToolBar(JToolBar.HORIZONTAL);

    JButton bNew = new JButton(new ImageIcon(getClass().
            getResource(NEW_ICON)));
    bNew.setToolTipText("New Model");
    bNew.addActionListener(control.actNew);
    setButtonSize(bNew);
    bar.add(bNew);

    JButton bOpen = new JButton(new ImageIcon(getClass().
            getResource(OPEN_ICON)));
    bOpen.setToolTipText("Open Model");
    bOpen.addActionListener(control.open);
    setButtonSize(bOpen);
    bar.add(bOpen);

    JButton bSave = new JButton(new ImageIcon(getClass().
            getResource(SAVE_ICON)));
    bSave.setToolTipText("Save");
    bSave.addActionListener(control.save);
    setButtonSize(bSave);
    bar.add(bSave);

    JButton bExport = new JButton(new ImageIcon(getClass().
            getResource(EXPORT_ICON)));
    bExport.setToolTipText("Export To Java");
    bExport.addActionListener(control.exportToJava);
    setButtonSize(bExport);
    bar.add(bExport);

    bar.addSeparator();
    JButton bHelp = new JButton(new ImageIcon(getClass().
            getResource(HELP_ICON)));
    bHelp.setToolTipText("Help");
    bHelp.addActionListener(new CSH.DisplayHelpFromSource(mainHB));
    bar.add(bHelp);
    setButtonSize(bHelp);
    bHelp.setEnabled(true);
    bar.addSeparator();

    bar.add(palette);
    bar.addSeparator();

    JButton bComp = new JButton(new ImageIcon(getClass().
            getResource(COMPILE_ICON)));
    bComp.setToolTipText("Compile");
    bComp.addActionListener(control.compile);
    setButtonSize(bComp);
    bar.add(bComp);

    JButton bRun = new JButton(new ImageIcon(getClass().
            getResource(RUN_ICON)));
    bRun.setToolTipText("Run");
    bRun.addActionListener(control.run);
    setButtonSize(bRun);
    bar.add(bRun);

    return bar;
  }

  private void setButtonSize(JButton button) {
    Dimension d = new Dimension(40, 40);
    button.setMinimumSize(d);
    button.setMaximumSize(d);
    button.setPreferredSize(d);
  }

  public void loadPalette(String name) {
    palette.loadPalette(name);
    toolBar.updateUI();
  }

  public PaletteItem getPaletteItemByName(String tabName, String itemName) {
    return palette.getPaletteItemByName(tabName, itemName);
  }

  private JMenuBar makeMenuBar(EvolverController control) {
    JMenuBar bar = new JMenuBar();
    JMenu file = new JMenu("File");
    file.setMnemonic('f');

    JMenuItem i = file.add(control.actNew);
    i.setMnemonic('n');

    i = file.add(control.open);
    i.setMnemonic('o');
    if (control.isArcGIS()) {
      i.setText("Import ...");
    }

    i = file.add(control.save);
    i.setMnemonic('s');

    i = file.add(control.saveAs);
    i.setMnemonic('a');
    if (control.isArcGIS()) {
      i.setText("Export ...");
    }

    /*
    file.addSeparator();

    i = file.add(control.addJar);
    i.setMnemonic('j');
    */

    file.addSeparator();

    i = file.add(control.exit);
    i.setMnemonic('x');
    bar.add(file);

    JMenu mProj = new JMenu("Model");
    mProj.setMnemonic('m');
    i = mProj.add(control.exportToJava);
    i.setMnemonic('e');
    mProj.addSeparator();
    i = mProj.add(control.compile);
    i.setMnemonic('c');
    i = mProj.add(control.run);
    i.setMnemonic('r');
    bar.add(mProj);

    JMenu mHelp = new JMenu("Help");
    mHelp.setMnemonic('h');
    //i = mHelp.add(control.help);
    i = new JMenuItem("Help Topics");
    if (mainHB != null)
      i.addActionListener(new CSH.DisplayHelpFromSource(mainHB));
    i.setMnemonic('h');
    i.setEnabled(true);
    mHelp.add(i);

    i = mHelp.add(about);
    i.setMnemonic('a');

    bar.add(mHelp);

    return bar;
  }

  private Action about = new AbstractAction("About") {
    public void actionPerformed(ActionEvent evt) {
      String version = "Repast Py v. " + EvolverController.VERSION;
      JOptionPane.showMessageDialog(frame, version);
    }
  };

  public void display() {
    if (frame == null) {
      frame = new JFrame(title);
      frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      frame.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent evt) {
          exit.actionPerformed(new ActionEvent(EvolverGui.this,
                  ActionEvent.ACTION_PERFORMED,
                  "exit"));
        }
      });

      frame.setJMenuBar(menuBar);

      Container c = frame.getContentPane();
      c.setLayout(new BorderLayout());
      c.add(this, BorderLayout.CENTER);
    }

    frame.setSize(650, 350);
    uchicago.src.guiUtils.GuiUtilities.centerComponentOnScreen(frame);
    frame.setVisible(true);
  }

  public void setTitle(String title) {
    this.title = title;
    if (frame != null) frame.setTitle(title);
  }

  public void setObject(Object bean) {
    props.setObject(bean);
  }

  public EvolverProject getProject() {
    return tree.getProjectBean();
  }

  public void renameTreeNode(Object bean, String oldName, String newName) {
    tree.renameNode(bean, oldName, newName);
  }

  public TreePath selectNodeByBean(Object bean) {
    return tree.selectNodeByBean(bean);
  }

  public TreePath selectNodeForObject(Object bean) {
    return tree.selectNodeForObject(bean);
  }

  public void setProject(EvolverProject project) {
    tree.setProjectBean(project);
  }

  public void showError(String message) {
    JOptionPane.showMessageDialog(frame, message, "Repast Py Error",
            JOptionPane.ERROR_MESSAGE);
  }

  public void showSourceEditor(ActionCodeProducer bean,
                               AbstractCompilerException ex) {
    bean.displayEditor(frame, ex);
  }

  public void setXML(String val) {
    xmlArea.setText(val);
  }

  public boolean askYesNo(String message) {
    int response = JOptionPane.showConfirmDialog(frame, message, message,
            JOptionPane.YES_NO_OPTION);
    return response == JOptionPane.YES_OPTION;
  }

  public int askYesNoCancel(String message) {
    return JOptionPane.showConfirmDialog(frame, message);
  }

  /**
   * Displays a save dialog for sim builder project files.
   * Returns the selected path. If the dialog is canceled then returns null.
   *
   * @param dir  default directory
   * @param file default file
   * @return the choosen path, or null if no path is selected
   */
  public String getSavePath(String dir, String file) {
    JFileChooser dialog = new JFileChooser(dir + File.separator + file);
    dialog.setFileFilter(new FileFilter() {
      public boolean accept(File f) {
        return (f.getName().endsWith(".sbp") || f.getName().endsWith(".xml") || f.isDirectory());
      }

      public String getDescription() {
        return "SimBuilder Model Files (.sbp / .xml)";
      }
    });

    dialog.setDialogTitle("Save Model");
    int result = dialog.showSaveDialog(frame);
    if (result == JFileChooser.APPROVE_OPTION) {
      String name = dialog.getSelectedFile().getAbsolutePath();
      if (!name.endsWith(".sbp") && !name.endsWith(".xml")) name = name + ".sbp";
      return name;
    }
    return null;
  }

  /**
   * Displays a open dialog for simbuilder project files. Returns the selected
   * path. If the dialog is canceled then returns null.
   *
   * @param dir  default directory
   * @param file default file
   * @return the choosen path, or null if no path is selected
   */
  public String getOpenPath(String dir, String file) {
    return getOpenPath(dir, file, "Open Model", new FileFilter() {
      public boolean accept(File f) {
        return (f.getName().endsWith(".sbp") || f.getName().endsWith(".xml") || f.isDirectory());
      }

      public String getDescription() {
        return "SimBuilder Model Files (.sbp / .xml)";
      }
    });
  }

  private String getOpenPath(String dir, String file, String title,
                             FileFilter filter) {
    JFileChooser dialog = new JFileChooser(dir + File.separator + file);
    dialog.setFileFilter(filter);

    dialog.setDialogTitle(title);
    int result = dialog.showOpenDialog(frame);
    if (result == JFileChooser.APPROVE_OPTION) {
      return dialog.getSelectedFile().getAbsolutePath();
    }
    return null;
  }

  /**
   * Displays a open dialog for jar files. Returns the selected path.
   * If the dialog is canceled then returns null.
   *
   * @param dir  default directory
   * @param file default file
   * @return the name of the new palette and the choosen path, or null
   *         if no path is selected
   */
  public String[] getJarPath(String dir, String file) {
    AddJarDialog dialog = new AddJarDialog(dir + File.separator + file);
    dialog.setFileFilter(new FileFilter() {
      public boolean accept(File f) {
        return (f.getName().endsWith(".jar") || f.isDirectory());
      }

      public String getDescription() {
        return "SimBuilder Bean Jar File (.jar)";
      }
    });

    dialog.setDialogTitle(title);

    int result = dialog.showOpenDialog(frame);
    if (result == JFileChooser.APPROVE_OPTION) {
      String[] ret = new String[]{dialog.getPaletteName(),
                                  dialog.getSelectedFile().getAbsolutePath()};
      return ret;
    }
    return null;
  }

  void addJarToPalette(String paletteName, JarClassLoader loader, JarFile jar) throws IOException {
    palette.addJar(loader, jar, paletteName);
  }
}

class AddJarDialog extends JFileChooser {

  JTextField txtName = new JTextField("New Palette", 20);


  public AddJarDialog() {
  }

  public AddJarDialog(String currentDirectoryPath) {
    super(currentDirectoryPath);
  }

  public AddJarDialog(File currentDirectory) {
    super(currentDirectory);
  }

  public AddJarDialog(FileSystemView fsv) {
    super(fsv);
  }

  public AddJarDialog(File currentDirectory, FileSystemView fsv) {
    super(currentDirectory, fsv);
  }

  public AddJarDialog(String currentDirectoryPath, FileSystemView fsv) {
    super(currentDirectoryPath, fsv);
  }

  public String getPaletteName() {
    return txtName.getText().trim();
  }

  protected JDialog createDialog(Component parent) throws HeadlessException {
    JDialog dialog = super.createDialog(parent);
    JPanel p = new JPanel(new FlowLayout(FlowLayout.LEFT, 10, 16));
    p.add(new JLabel("Palette Name: "));
    p.add(txtName);


    dialog.getContentPane().add(p, BorderLayout.NORTH);
    dialog.pack();
    return dialog;
  }
}

