package uchicago.src.simbuilder;

import org.apache.velocity.app.Velocity;
import uchicago.src.simbuilder.beans.EvolverProject;
import uchicago.src.simbuilder.beans.ConstrainedProperty;
import uchicago.src.simbuilder.beans.gis.VectorAgentProducer;
import uchicago.src.simbuilder.beans.gis.VectorDataSource;
import uchicago.src.simbuilder.beans.gis.DefaultGISModelProducer;
import uchicago.src.simbuilder.beans.gis.GISPackage;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Simple main class to start RepastPy. Handles command line arguments.
 * These are: TODO
 * <p/>
 * $Revision: 1.3 $ $Date: 2004/12/10 22:10:49 $
 */

public class Main {

  static boolean choose = false;

  public static void main(String[] args) {
    boolean arc = false;
    try {
      // initialize Velocity template engine
      // init velocity with resource loader config settings
      Properties props = new Properties();
      props.put("resource.loader", "class");
      props.put("class.resource.loader.description", "Velocity Classpath Resource Loader");
      props.put("class.resource.loader.class", "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
      Velocity.init(props);
      
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      EvolverController c = new EvolverController();
      
      if (args.length > 0 && args[0].equals("-ArcGIS")) {
        c.setArcGIS(true);
        arc = true;
      }
      EvolverGui gui = new EvolverGui(c);
      c.setGui(gui);

      boolean run = false;
      if (args.length > 0) {
        if (arc) {
          //c.setArcGIS(true);
          if (args.length > 1) {
            int argsAdjust = 0;
            if (args[1].equals("-run")) {
              run = true;
              argsAdjust = 1;
            }
            handleArcLoad(c, args, argsAdjust);
          } else {
            choose = true;
          }
        } else {
          // assume its a model file and try to load it
          c.load(args[0]);
        }
      } else {
        choose = true;
      }
      gui.display();
      if (choose) c.chooseNewModel(false);
      if (run) c.run();
    } catch (Exception ex) {
      String name = arc ? "Agent Analyst" : "Repast Py";
      EvolverGui.showError("Error Starting " + name, ex);
      ex.printStackTrace();
      System.exit(0);
    }
  }

  private static void handleArcLoad(EvolverController controller, String[] args, int argsAdjust) throws IOException {
    if (1 + argsAdjust >= args.length) {
      EvolverGui.showError("No model file specified", new IllegalArgumentException(""));
      choose = true;
      return;
    }
    String filename = args[1 + argsAdjust];
    File file = new File(filename);
    if (!file.exists()) {
      // create a model and save it to that file
      // including creating the directory
      file.createNewFile();
      controller.startNewProject();
      controller.addBeanToTree("GIS", "uchicago.src.simbuilder.beans.gis.DefaultGISModelProducer", 0);
      controller.addBeanToTree("GIS", "uchicago.src.simbuilder.beans.gis.VectorAgentProducer", 1);
      controller.setFilename(file);
      controller.saveAs(file.getCanonicalPath());
    } else
      controller.load(filename);
    
    EvolverProject project = controller.getProjectBean();
    DefaultGISModelProducer model = (DefaultGISModelProducer)project.getModel();
    ConstrainedProperty prop = model.getGisPackage();
    prop.setSelectedView(GISPackage.ARCGIS.toString());
    model.setGisPackage(prop);

    if (args.length > 2 + argsAdjust) {
      // add the shapefiles as options that user can choose from 
      // in the model's datasource
     
      boolean found = false;
      for (Iterator iter = model.compileables().iterator(); iter.hasNext();) {
        Object obj = iter.next();
        if (obj instanceof VectorAgentProducer) {
          found = true;
          // add the shapefiles to this producers datasource
          VectorAgentProducer producer = (VectorAgentProducer) obj;
          VectorDataSource dataSource = producer.getDatasource();
          for (int i = 2 + argsAdjust; i < args.length; i++) {
            dataSource.addShapefile(args[i]);
          }
        }
      }

      if (!found) {
        // add a VectorAgentProducer with the shapefiles
        VectorAgentProducer producer = (VectorAgentProducer) controller.addBeanToTree("GIS",
                "uchicago.src.simbuilder.beans.gis.VectorAgentProducer", 1);
        VectorDataSource dataSource = producer.getDatasource();
        for (int i = 2 + argsAdjust; i < args.length; i++) {
          dataSource.addShapefile(args[i]);
        }
      }
    }
  }
}
