package uchicago.src.simbuilder;

import layout.TableLayout;
import uchicago.src.guiUtils.GuiUtilities;
import uchicago.src.simbuilder.beans.ModelProducerBeanInfo;
import uchicago.src.simbuilder.beans.gis.DefaultGISModelProducerBeanInfo;
import uchicago.src.simbuilder.beans.grid.GridModelProducerBeanInfo;
import uchicago.src.simbuilder.beans.network.NetworkModelProducerBeanInfo;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

/**
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:24 $
 */
public class NewModelDialog extends JPanel {

  private JList modelList;
  private JDialog dialog;
  private NewModelItem selectedModel;
  private JButton okBtn;
  private JButton cancelBtn;

  private JTextArea description = new JTextArea();

  public NewModelDialog(boolean showCancel) {
    description.setWrapStyleWord(true);
    description.setLineWrap(true);

    setLayout(new BorderLayout());
    JPanel center = new JPanel();
    double[][] sizes = {{.5, 6, .5}, {TableLayout.FILL}};
    center.setLayout(new TableLayout(sizes));
    JPanel leftPanel = new JPanel(new BorderLayout());
    leftPanel.setBorder(BorderFactory.createTitledBorder("Select Model Type"));

    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());
    panel.setBorder(BorderFactory.createLoweredBevelBorder());
    leftPanel.add(panel);

    Vector items = new Vector();
    ImageIcon icon = new ImageIcon(getClass().getResource(DefaultGISModelProducerBeanInfo.ICON_IMAGE));
    NewModelItem item = new NewModelItem(true, icon, "GIS Model", new DefaultGISModelProducerBeanInfo());
    item.setDescription("GIS type model in which agents can operate in GIS defined spaces. Agents can be created from GIS data.");
    items.add(item);

    icon = new ImageIcon(getClass().getResource(NetworkModelProducerBeanInfo.IMAGE));
    item = new NewModelItem(true, icon, "Network Model", new NetworkModelProducerBeanInfo());
    item.setDescription("Network type model in which agents are nodes in a network. Agents can be created in a variety of" +
            " network topologies.");
    items.add(item);

    icon = new ImageIcon(getClass().getResource(uchicago.src.simbuilder.beans.grid.GridModelProducerBeanInfo.IMAGE));
    item = new NewModelItem(true, icon, "Grid Model", new GridModelProducerBeanInfo());
    item.setDescription("Grid type model in which agents operate on a two dimensional matrix grid or torus.");
    items.add(item);
    
    icon = new ImageIcon(getClass().getResource("/uchicago/src/simbuilder/images/Open.gif"));
    item = new NewModelItem(true, icon, "A Saved Model", new LoadModelBeanInfo());
    item.setDescription("Load a previously saved model.");
    items.add(item);
    
    modelList = new JList(items);
    modelList.setCellRenderer(new ModelItemRenderer());

    panel.add(modelList, BorderLayout.CENTER);
    panel.setBackground(Color.WHITE);

    JPanel rightPanel = new JPanel(new BorderLayout());
    rightPanel.setBorder(BorderFactory.createTitledBorder("Description"));
    description.setBackground(rightPanel.getBackground());
    rightPanel.add(description, BorderLayout.CENTER);

    center.add(leftPanel, "0, 0");
    center.add(rightPanel, "2, 0");
    add(center, BorderLayout.CENTER);

    JPanel bottom = new JPanel();
    okBtn = new JButton("OK");
    bottom.add(okBtn);
    cancelBtn = new JButton("Cancel");
    if (showCancel) bottom.add(cancelBtn);
    add(bottom, BorderLayout.SOUTH);

    addListeners();
  }

  public void display(JFrame parent) {
    dialog = new JDialog(parent, "New Model", true);
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.setSize(400, 200);
    dialog.setLocation(GuiUtilities.getPointToCenterOnComponent(dialog, parent));
    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    dialog.show();
  }

  private void addListeners() {
    modelList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        if (!e.getValueIsAdjusting()) {
          NewModelItem item = (NewModelItem) modelList.getSelectedValue();
          if (item != null) {
            description.setText(item.getDescription());
          } else {
            description.setText("");
          }
        }
      }
    });

    cancelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        selectedModel = null;
        dialog.dispose();
      }
    });

    okBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        NewModelItem item = (NewModelItem) modelList.getSelectedValue();
        if (item == null) {
          JOptionPane.showMessageDialog(dialog, "Please select a model type.");
        } else {
          setChosenModel(item);
        }
      }
    });


    modelList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          NewModelItem item = (NewModelItem) modelList.getSelectedValue();
          if (item != null)
            setChosenModel(item);
        }
      }
    });
  }

  public ModelProducerBeanInfo getSelectedModelInfo() {
    return selectedModel == null ? null : selectedModel.getModelProducerBeanInfo();
  }

  private void setChosenModel(NewModelItem item) {
    if (item.isEnabled()) {
      selectedModel = item;
      dialog.dispose();
    } else {
      JOptionPane.showMessageDialog(dialog, "Selected model type is currently disabled");
    }
  }

  class ModelItemRenderer extends DefaultListCellRenderer {

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
      JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
      NewModelItem item = (NewModelItem) value;
      label.setEnabled(item.isEnabled());
      label.setIcon(item.getIcon());
      label.setText(item.getName());
      label.setHorizontalTextPosition(JLabel.RIGHT);
      label.setVerticalTextPosition(JLabel.CENTER);
      return label;
    }
  }

}
