package uchicago.src.simbuilder;

import uchicago.src.simbuilder.util.JarClassLoader;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.beans.IntrospectionException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.jar.JarFile;

/**
 * Panel which contains the PalettePanels.
 *
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:24 $
 */

public class Palette extends JPanel {

  //private static final String SB_BEAN_TAG = "SimBuilder-Bean";
  private static final String BEAN_TAG_NAME = "RePastSimBuilder:Bean";

  PaletteButton selector;
  //private JTabbedPane tPanel;
  private PalettePanel palettePanel;
  private ActionListener paletteItemListener;
  private Map palettes = new HashMap();


  public Palette(ActionListener listener) {
    super();
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

    selector = new PaletteButton(new PaletteSelectorItem(), null);
    selector.addActionListener(listener);
    paletteItemListener = listener;

    //tPanel = new JTabbedPane();

    addCoreJar();
    add(selector);
    add(palettePanel);
  }

  private void addCoreJar() {
    makeNetworkPanel();
    makeGridPanel();
    makeGISPanel();
  }

  private void makeGridPanel() {
    String[] items = {"uchicago.src.simbuilder.beans.grid.GridAgentProducer",
                      "uchicago.src.simbuilder.beans.grid.GridDisplayProducer"};
    palettePanel = new PalettePanel(items, "Grid", paletteItemListener,
            selector);
    palettePanel.addHiddenModel("uchicago.src.simbuilder.beans.grid.GridModelProducer");
    palettePanel.add(Box.createRigidArea(new Dimension(10, 0)));
    try {
      palettePanel.addButton("uchicago.src.simbuilder.beans.SequenceGraph", paletteItemListener.getClass().getClassLoader());
      palettePanel.addButton("uchicago.src.simbuilder.beans.DataRecorder", paletteItemListener.getClass().getClassLoader());
    } catch (ClassNotFoundException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    } catch (IntrospectionException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    palettes.put(palettePanel.getName(), palettePanel);
  }

  private void makeNetworkPanel() {
    String[] items = {"uchicago.src.simbuilder.beans.network.NetworkNodeProducer",
                      "uchicago.src.simbuilder.beans.network.NetworkDisplayProducer",
                      "uchicago.src.simbuilder.beans.network.NetworkPlotProducer",
                      "uchicago.src.simbuilder.beans.network.NetworkRecorderProducer"};
    palettePanel = new PalettePanel(items, "Network", paletteItemListener,
            selector);
    palettePanel.addHiddenModel("uchicago.src.simbuilder.beans.network.NetworkModelProducer");
    palettePanel.add(Box.createRigidArea(new Dimension(10, 0)));
    try {
      palettePanel.addButton("uchicago.src.simbuilder.beans.SequenceGraph", paletteItemListener.getClass().getClassLoader());
      palettePanel.addButton("uchicago.src.simbuilder.beans.DataRecorder", paletteItemListener.getClass().getClassLoader());
    } catch (ClassNotFoundException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    } catch (IntrospectionException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    palettes.put(palettePanel.getName(), palettePanel);
  }

  private void makeGISPanel() {
    String[] items = {"uchicago.src.simbuilder.beans.gis.VectorAgentProducer",
                      "uchicago.src.simbuilder.beans.gis.GenericAgentProducer"};
    palettePanel = new PalettePanel(items, "GIS", paletteItemListener,
            selector);
    palettePanel.addHiddenModel("uchicago.src.simbuilder.beans.gis.DefaultGISModelProducer");
    palettePanel.add(Box.createRigidArea(new Dimension(10, 0)));
    try {
      palettePanel.addButton("uchicago.src.simbuilder.beans.SequenceGraph", paletteItemListener.getClass().getClassLoader());
      palettePanel.addButton("uchicago.src.simbuilder.beans.DataRecorder", paletteItemListener.getClass().getClassLoader());
    } catch (ClassNotFoundException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    } catch (IntrospectionException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    palettes.put(palettePanel.getName(), palettePanel);
  }


  void addJar(ClassLoader loader, JarFile jar, String tabName) throws IOException {
    /*
          
ZipEntry ze = jar.getEntry("beans.xml");
if (ze == null) throw new IOException(jar.getName() + " must contain a 'beans.xml' file");
DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
dbf.setValidating(false);

DocumentBuilder parser = null;
Document doc = null;
try {
parser = dbf.newDocumentBuilder();
doc = parser.parse(jar.getInputStream(ze));
} catch (ParserConfigurationException e) {
throw new IOException("Invalid beans.xml file: " + e.getMessage());
} catch (SAXException e) {
throw new IOException("Invalid beans.xml file: " + e.getMessage());
}

Element root = doc.getDocumentElement();
NodeList beanList = root.getElementsByTagName(BEAN_TAG_NAME);
ArrayList beansToAdd = new ArrayList();
for (int i = 0, n = beanList.getLength(); i < n; i++) {
Element bean = (Element) beanList.item(i);
String beanName = bean.getAttribute("class");
int index = beanName.indexOf(".class");
beanName = beanName.substring(0, index);
beansToAdd.add(beanName);
}

String[] toAdd = new String[beansToAdd.size()];
beansToAdd.toArray(toAdd);
PalettePanel p = new PalettePanel(toAdd, tabName, paletteItemListener,
    selector, loader);
tPanel.add(tabName, p);
*/



    /*
    Iterator iter = mf.getEntries().keySet().iterator();
    ArrayList beansToAdd = new ArrayList();
    while (iter.hasNext()) {
      String beanName = (String) iter.next();
      Attributes attribs = mf.getAttributes(beanName);
      if (attribs != null) {
        String isBean = attribs.getValue(SB_BEAN_TAG);
        if (isBean != null && isBean.equalsIgnoreCase("true")) {
          beanName = beanName.replace('/', '.');
          int index = beanName.indexOf(".class");
          beanName = beanName.substring(0, index);
          //System.out.println(beanName);
          beansToAdd.add(beanName);
        }
      }
    }

    String[] toAdd = new String[beansToAdd.size()];
    for (int i = 0; i < beansToAdd.size(); i++) {
      toAdd[i] = (String) beansToAdd.get(i);
    }

    PalettePanel p = new PalettePanel(toAdd, tabName, paletteItemListener,
                                      selector, loader);
    tPanel.add(tabName, p);
    */
  }

  public PaletteItem getPaletteItemByName(String tabName, String itemName) {
    PalettePanel panel = (PalettePanel) palettes.get(tabName);
    if (panel != null) {
      return panel.getItemByName(itemName);
    }

    return null;
  }

  public void loadPalette(String name) {
    PalettePanel panel = (PalettePanel) palettes.get(name);
    if (panel != null) {
      remove(palettePanel);
      palettePanel = panel;
      add(palettePanel);
      palettePanel.addSelectorToGroup();
      invalidate();
      validate();
      repaint();
    } else {
      throw new IllegalArgumentException("Panel '" + name + "' not found");
    }
  }

  void addJar(JarClassLoader loader, JarFile jar) throws IOException {
    addJar(loader, jar, "User");
  }
}





