package uchicago.src.simbuilder;

import java.awt.*;
import java.beans.BeanInfo;
import java.beans.Beans;
import java.io.IOException;

/**
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:24 $
 */

/**
 * A wrapper around EvolverBeans allowing them to be displayed
 * in PalettePanels. A PaletteBeanItem is also responsible for
 * instantiating the EvolverBean as an Object.
 */

public class PaletteBeanItem implements PaletteItem {

  private Class beanClass;
  private String beanName;
  private BeanInfo info;

  public PaletteBeanItem(String name) throws ClassNotFoundException {
    beanName = name;
    beanClass = Class.forName(name);
    info = BeanIntrospector.getBeanInfo(beanClass);
  }

  public PaletteBeanItem(String name, Class clazz) {
    beanClass = clazz;
    beanName = name;
    info = BeanIntrospector.getBeanInfo(beanClass);
  }

  public String getName() {
    return info.getBeanDescriptor().getName();
  }

  public String getBeanName() {
    return beanName;
  }

  public PaletteItem getItem() {
    return this;
  }

  public String getToolTipText() {
    return info.getBeanDescriptor().getDisplayName();
  }

  public Object makeBean() throws IOException, ClassNotFoundException {
    return Beans.instantiate(beanClass.getClassLoader(), beanName);
  }

  public Image getIcon(int iconType) {
    return info.getIcon(iconType);
  }

  public int getCursorType() {
    return Cursor.HAND_CURSOR;
  }
}

