package uchicago.src.simbuilder;

import javax.swing.*;
import java.awt.*;
import java.beans.BeanInfo;

/**
 * A Button wrapper around PaletteItems allowing PaletteItems
 * to function like JToggleButtons.
 *
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:24 $
 */

public class PaletteButton extends JToggleButton {

  private PaletteItem item;
  private PaletteButton selector;

  public PaletteButton(PaletteItem item, PaletteButton selector) {
    this.item = item;
    Image icon = item.getIcon(BeanInfo.ICON_COLOR_32x32);
    if (icon != null) {
      this.setIcon(new ImageIcon(icon));
    } else {
      this.setText(item.getName());
    }

    this.setToolTipText(item.getToolTipText());
    this.selector = selector;
  }

  public void unselect() {
    if (selector != null) {
      selector.setSelected(true);
    }
  }


  public Dimension getMinimumSize() {
    return new Dimension(40, 40);
  }

  public Dimension getMaximumSize() {
    return getMinimumSize();
  }

  public Dimension getPreferredSize() {
    return getMinimumSize();
  }

  public void setRootCursor() {
    Component root = SwingUtilities.getRoot(this);
    root.setCursor(Cursor.getPredefinedCursor(item.getCursorType()));
  }

  public PaletteItem getItem() {
    return item;
  }
}
