package uchicago.src.simbuilder.base;

import cern.colt.function.IntDoubleProcedure;
import cern.colt.map.OpenIntDoubleHashMap;
import uchicago.src.collection.RangeMap;
import uchicago.src.sim.network.AbstractProbabilityRule;

import java.util.List;

/**
 * Base class for dynamically compiled ProbabiltyRules created
 * within Evolver.
 *
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:24 $
 */
public abstract class AbstractEvProbRule extends AbstractProbabilityRule {

  protected OpenIntDoubleHashMap curWeights = new OpenIntDoubleHashMap();
  protected float discount = 1f;

  public AbstractEvProbRule(List objs, float startWeight, Object exclude) {
    super(objs, startWeight, exclude);
  }

  public void setPastDiscount(float val) {
    if (val < 0 || val > 1) {
      throw new IllegalArgumentException("Discount must be between 0 and 1");
    }

    discount = val;
  }

  public void setCurNodeWeight(Object o, float amt) {
    int index = ((Integer) weightMap.get(o)).intValue();
    curWeights.put(index, amt);
  }

  public RangeMap makeProbabilityMap(RangeMap map) {
    if (discount != 1) {

      for (int i = 0; i < weights.size(); i++) {
        float f = weights.getQuick(i) * discount;
        weights.setQuick(i, f);
      }

      curWeights.forEachPair(new IntDoubleProcedure() {
        public boolean apply(int index, double val) {
          float f = weights.getQuick(index);
          weights.setQuick(index, (float) (f + val));
          return true;
        }
      });
      curWeights.clear();
    }

    return super.makeProbabilityMap(map);
  }

  public void update(Object o) {
    this.update(o, 1f);
  }
}

