package uchicago.src.simbuilder.base;

import uchicago.src.sim.engine.CustomProbeable;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.gui.Drawable;
import uchicago.src.sim.space.Object2DGrid;
import uchicago.src.sim.space.Object2DTorus;

import java.awt.*;
import java.util.Vector;

public abstract class DefaultGridAgent implements CustomProbeable, Drawable {

  protected static final int RECT = 0;
  protected static final int CIRCLE = 1;

  protected Object2DGrid space;
  protected int x, y;
  protected Color color = Color.red;
  protected int shape = RECT;
  private boolean isTorus = false;

  public void setSpace(Object2DGrid space) {
    this.space = space;
    isTorus = (space instanceof Object2DTorus);
  }

  public Vector getVonNeumannNeighbors() {
    return space.getVonNeumannNeighbors(x, y, false);
  }

  public Vector getVonNeumannNeighbors(int extent) {
    return space.getVonNeumannNeighbors(x, y, extent, extent, false);
  }

  public Vector getMooreNeighbors() {
    return space.getMooreNeighbors(x, y, false);
  }

  public Vector getMooreNeighbors(int extent) {
    return space.getMooreNeighbors(x, y, extent, extent, false);
  }

  /**
   * Moves this DefaultGridAgent to the new coordinates, if
   * the cell at those coordinates is empty. Returns true if
   * this DefaultGridAgent has successfully moved and false if
   * not.
   */
  public boolean moveIfEmpty(int newX, int newY) {
    if (space.getObjectAt(newX, newY) == null) {
      move(newX, newY);
      return true;
    }

    return false;
  }

  public void move(int newX, int newY) {
    space.putObjectAt(x, y, null);
    space.putObjectAt(newX, newY, this);
    if (isTorus) {
      newX = ((Object2DTorus)space).xnorm(newX);
      newY = ((Object2DTorus)space).ynorm(newY);
    }
    x = newX;
    y = newY;
  }

  public void setDrawAsRect() {
    shape = RECT;
  }

  public void setDrawAsCircle() {
    shape = CIRCLE;
  }

  public void setColor(Color c) {
    color = c;
  }

  public int getShape() {
    return shape;
  }

  public void setShape(int shape) {
    this.shape = shape;
  }


  /**
   * Invoked when the object should draw itself
   */
  public void draw(SimGraphics g) {
    if (shape == RECT)
      g.drawFastRect(color);
    else
      g.drawFastRoundRect(color);
  }


  /**
   * Gets the x coordinate of this drawable. Note that this should return the
   * x coordinate in some space, not necessarily a screen coordinate.
   */
  public int getX() {
    return x;
  }

  public void setX(int val) {
    x = val;
  }

  /**
   * Gets the y coordinate of the this drawable. Note that this should return
   * the y coordinate in some space, not necessarily a screen coordinate.
   */
  public int getY() {
    return y;
  }

  public void setY(int val) {
    y = val;
  }
}
