/*
 *
 */
package uchicago.src.simbuilder.base;

import uchicago.src.sim.games.GameChoice;
import uchicago.src.sim.games.PayoffMatrix;
import uchicago.src.sim.games.Strategy;

public abstract class GridGameAgent extends DefaultGridAgent {

  protected Strategy strategy;
  protected GameChoice nextMove, lastMove;
  protected double payoff;
  protected PayoffMatrix matrix;
  protected int numPlays = 0;

  /**
   * Sets the payoff and number of plays of this GridGameAgent to 0.
   */
  public void initialize() {
    payoff = 0;
    numPlays = 0;
  }

  public void setPayoff(double val) {
    payoff = val;
  }

  public double getPayoff() {
    return payoff;
  }

  public void setStrategy(Strategy val) {
    strategy = val;
  }

  public Strategy getStrategy() {
    return strategy;
  }

  public void setMatrix(PayoffMatrix m) {
    matrix = m;
  }

  public PayoffMatrix getMatrix() {
    return matrix;
  }

  public void calcNextMove(GameChoice oppLastMove) {
    nextMove = strategy.calculateMove(oppLastMove);
  }

  public GameChoice getLastMove() {
    return lastMove;
  }

  /**
   * Plays the other agent by getting the results from the payoff matrix
   * given the two players current strategies. This GridGameAgent is
   * the row player. If we want to update the payoff of the other agent
   * that GridGameAgent needs to play as well. This also remembers the
   * other players last play.
   */
  public void play(GridGameAgent other) {
    payoff += matrix.getRowPayoff(nextMove, other.nextMove);
    numPlays++;
    lastMove = nextMove;
  }

  /**
   * Returns the current payoff total for this GridGameAgent divided by
   * the number of times this GridGameAgent has played. If this GridGameAgent
   * has not played at all then this returns Double.NEGATIVE_INFINITY such
   * that not playing at all is equivalent to achieving the worst possible
   * payoff.
   */
  public double getAvgPayoff() {
    if (numPlays == 0) return Double.NEGATIVE_INFINITY;
    return payoff / numPlays;
  }
}
