package uchicago.src.simbuilder.base.gis;

import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.util.Random;

import java.util.*;

import anl.repast.gis.data.GeotoolsData;
import anl.repast.gis.data.OpenMapData;
import anl.repast.gis.display.ESRIDisplay;
import anl.repast.gis.display.OpenMapDisplay;

public class GISSimModelImpl extends SimModelImpl {

  protected Schedule schedule;
  protected String[] params;
  //protected DisplaySurface surface;
  protected String name;
  protected OpenMapDisplay omDisplay;
  
  public interface DisplayUpdater {
    public void updateDisplay();
    public void updateDisplay(String path);
    public void addLayer(String title, Collection agentGroup);
    public void reset();
  }
  
  protected DisplayUpdater esriDisplayUpdater = new DisplayUpdater() {
    public void addLayer(String title, Collection agentGroup) {
      
    }

    public void reset() {}
    
    public void updateDisplay() {
      ESRIDisplay.getInstance().updateDisplay();
    }
    
    public void updateDisplay(String path) {
      ESRIDisplay.getInstance().updateDisplay(path);
    }
  };
  
  protected DisplayUpdater omDisplayUpdater = new DisplayUpdater() {
    
    private Map map = new HashMap();

    public void addLayer(String title, Collection agentGroup) {
      map.put(title, agentGroup);
    }
    
    public void reset() {
      map.clear();
    }
    
    public void updateDisplay(String path) {
      updateDisplay();
    }

    public void updateDisplay() {
      for (Iterator iter = map.entrySet().iterator(); iter.hasNext(); ) {
        Map.Entry entry = (Map.Entry) iter.next();
        omDisplay.updateLayer((List)entry.getValue(), (String)entry.getKey());
      }
      omDisplay.updateDisplay();
    }
  };
  
  protected DisplayUpdater updater = esriDisplayUpdater;

  protected void buildDisplay() {
    _display();
  }

  protected void buildSchedule() {
    //schedule.scheduleActionBeginning(1, agentList, "step");
    _schedule();
  }

  /**
   * This should maybe implemented by dynamic subclass..
   */
  public void _display() {
  }

  /**
   * This should be implemented by dynamic subclass.
   */
  public void _build() {
  }

  /**
   * This should be implemented by dynamic subclass.
   */
  public void _setup() {
  }

  public void _schedule() {
  }

  public void begin() {
    _build();
    buildDisplay();
    buildSchedule();

    //if (surface != null) surface.display();
  }

  public void updateGISDisplay() {
    updater.updateDisplay();
  }
  
  public void updateGISDisplay(String path) {
    updater.updateDisplay(path);
  }

  public void writeAgents(Collection agents, String shapefile) {
    if (updater == esriDisplayUpdater) GeotoolsData.getInstance().writeAgents(agents, shapefile);
    else OpenMapData.getInstance().writeAgents(agents, shapefile);
  }
  
  public void writeAgents(Collection agents, String origShapefile, String newShapefile) {
    if (updater == esriDisplayUpdater) GeotoolsData.getInstance().writeAgents(agents, origShapefile, newShapefile);
    else OpenMapData.getInstance().writeAgents(agents, origShapefile, newShapefile);
  }

  public void setup() {
    Random.createUniform();
    if (omDisplay != null) omDisplay.dispose();
    schedule = null;

    schedule = new Schedule();
    _setup();

    System.gc();
  }

  public String[] getInitParam() {
    return params;
  }

  public Schedule getSchedule() {
    return schedule;
  }

  public String getName() {
    return name;
  }

  public Class getClassByName(String name) {
    //System.out.println(name);
    try {
      return Class.forName(name, true, this.getClass().getClassLoader());
    } catch (Exception ex) {
      ex.printStackTrace();
    }

    return null;
  }
}
