package uchicago.src.simbuilder.beans;

import ViolinStrings.Strings;

import java.util.ArrayList;

/**
 * Stores rscript for later compilation. The code is compiled to a method
 * in a dynamically compiled class (model or agent).
 *
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:25 $
 */

public class ActionCode {

  // name is the name of the method the code in this ActionCode object
  // will become.
  protected String name;
  protected String retType = "void";
  protected String access = "public";
  protected String code = "";
  protected ArrayList imports = new ArrayList();
  protected ActionCodeProducer definingBean;


  protected ArrayList paramNames = new ArrayList();

  public ActionCode(String name) {
    this.name = name;
  }

  public ActionCodeProducer getDefiningProducer() {
    return definingBean;
  }

  public void setDefiningProducer(ActionCodeProducer definingBean) {
    this.definingBean = definingBean;
  }

  public String getRetType() {
    return retType;
  }

  public void setRetType(String retType) {
    this.retType = retType;
  }

  public String getAccess() {
    return access;
  }

  public void setAccess(String access) {
    this.access = access;
  }

  /**
   * Returns a deep copy of this ActionCode.
   */
  public Object copy() {
    ActionCode newCode = new ActionCode(name);
    newCode.retType = retType;
    newCode.access = access;
    newCode.code = code;
    newCode.imports = imports;
    newCode.paramNames = paramNames;
    newCode.definingBean = definingBean;

    return newCode;
  }

  /**
   * Sets the code (the RScript statements) for this ActionCode object.
   */
  public void setCode(String code) {
    this.code = code;
  }

  /**
   * Gets the code (the RScript statements) for this ActionCode object.
   */
  public String getCode() {
    return code;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getName() {
    return name;
  }

  public String toString() {
    return name;
  }

  public void addParameterName(String name) {
    paramNames.add(name);
  }

  public ArrayList getParameterNames() {
    return paramNames;
  }

  public void setParameterNames(ArrayList val) {
    paramNames = val;
  }

  /**
   * Removes simbuilder generated lines of code from this action code.
   */
  public void removeEvCode(String tag, int numLines) {
    int index = code.indexOf(tag);
    if (index != -1) {
      String substring = code.substring(index, code.length());
      int snipStart, lineCount, snipEnd;
      snipStart = lineCount = snipEnd = 0;
      snipEnd = lineCount = 0;
      char prevChar = 'a';
      numLines++;
      for (int i = 0; i < substring.length(); i++) {
        char c = substring.charAt(i);
        if (c == '\n' || c == '\r') {
          if (prevChar != '\n' && prevChar != '\r') {
            lineCount++;
            if (lineCount == numLines) {
              snipEnd = i;
              break;
            }
          }
        }
        prevChar = c;
      }

      StringBuffer b = new StringBuffer(code);
      ////System.out.println("part to snip: " +
      //		   code.substring(index + snipStart, index + snipEnd));
      code = b.delete(index, index + snipEnd).toString();
    }
  }


  /**
   * Replaces the AGENT_NAME_TEMPLATE place holder in the rscript code with
   * the actual agent class name.
   */
  public void setAgentName(String agentName) {
    code = Strings.change(code, EvolverBean.AGENT_NAME_TEMPLATE, agentName);
  }

  public void setUnqualifedAgentName(String name) {
    code = Strings.change(code, EvolverBean.AGENT_UNQ_NAME_TEMPLATE, name);
  }


  /**
   * Replaces the MODEL_NAME_TEMPLATE place holder in the rscript code with
   * the actual model class name.
   */
  public void setModelName(String modelName) {
    code = Strings.change(code, EvolverBean.MODEL_NAME_TEMPLATE, modelName);
  }

  public void setImports(ArrayList val) {
    imports = val;
  }

  public ArrayList getImports() {
    return imports;
  }

  public String toXML() {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XMLNS);
    b.append(":action name=\"");
    b.append(name);
    b.append("\" retType=\"");
    b.append(retType);
    b.append("\" imports=\"");
    for (int i = 0; i < imports.size(); i++) {
      if (i != 0) b.append(" ");
      b.append(imports.get(i));
    }

    b.append("\" paramNames=\"");
    for (int i = 0; i < paramNames.size(); i++) {
      if (i != 0) b.append(" ");
      b.append(paramNames.get(i));
    }

    b.append("\">");
    b.append("\n<![CDATA[\n");
    b.append(code);
    b.append("\n]]>\n</");
    b.append(EvolverConstants.XMLNS);
    b.append(":action>\n");
    return b.toString();
  }

  public static void main(String[] args) {
    ActionCode c = new ActionCode("test");
    StringBuffer b = new StringBuffer("rule = Foo");
    b.append("(self.model.getAgentList(), self.startWeight, self)\n\r");
    b.append("self._linker = Linker(rule)\n\r");
    String scode = b.toString();
    scode = "### generated code\n\r" + scode;
    scode += "\n\r\n\r### user code\n\rfoo bar";
    c.setCode(scode);
    //System.out.println("old code:\n" + scode);
    c.removeEvCode("### generated code", 2);
    //System.out.println("new code:\n" + c.getCode());

  }

}
