package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.*;
import uchicago.src.nqpy.parser.TokenMgrError;
import uchicago.src.simbuilder.EvolverModelCompiler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * A Container for ActionCode objects. While this is primarily a wrapper
 * around an ArrayList, it has to be its own class in order to register
 * the appropriate property editor for editing the contained ActionCode.
 *
 * @author Nick Collier
 * @version $Revision: 1.3 $ $Date: 2004/10/12 14:50:37 $
 */

public class Actions {

  private ArrayList actions = new ArrayList();
  private ArrayList varNames = new ArrayList();

  public Actions() {
  }

  public Actions(Actions a) {
    for (int i = 0; i < a.actions.size(); i++) {
      actions.add(((ActionCode) a.actions.get(i)).copy());
    }
    varNames.addAll(a.varNames);
  }

  public void addAction(ActionCode code) {
    actions.add(code);
  }

  public Object remove(int index) {
    return actions.remove(index);
  }


  public Object remove(String name) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode c = (ActionCode) actions.get(i);
      if (c.getName().equals(name)) {
        actions.remove(i);
        return c;
      }
    }

    return null;
  }

  public void addParameterName(String actionName, String parameterName) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode c = (ActionCode) actions.get(i);
      if (c.getName().equals(actionName)) {
        c.addParameterName(parameterName);
        break;
      }
    }
  }

  public void removeEvCode(String actionName, String tag, int lines) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode c = (ActionCode) actions.get(i);
      if (c.getName().equals(actionName)) {
        c.removeEvCode(tag, lines);
        break;
      }
    }
  }

  public int size() {
    return actions.size();
  }

  public Iterator iterator() {
    return actions.iterator();
  }

  public void clearVarNames() {
    varNames.clear();
  }

  public void addVarName(String name) {
    varNames.add(name);
  }

  public ArrayList getVarNames() {
    Collections.sort(varNames);
    return varNames;
  }

  public void clear() {
    actions.clear();
  }

  /**
   * Replaces the agent name and model name placeholders with the
   * actual model class name and agent class name.
   
  public voidsetDynTypeNames(String modelName, String agentName) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode code = (ActionCode) actions.get(i);
      if (modelName != null) code.setModelName(modelName);
      if (agentName != null) {
        code.setAgentName(agentName);
        int index = agentName.lastIndexOf('.');
        if (index != -1) code.setUnqualifedAgentName(agentName.substring(index + 1, agentName.length()));
      }
    }
  }
   */

  public boolean hasAction(String name) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode c = (ActionCode) actions.get(i);
      if (c.getName().equals(name)) return true;
    }

    return false;
  }

  public void addToAction(String name, ArrayList stmts) {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < stmts.size(); i++) {
      b.append(stmts.get(i));
      b.append("\n");
    }

    addToAction(name, b.toString());
  }

  public void prependToAction(String name, String stmts) {
    ActionCode c = null;
    for (int i = 0; i < actions.size(); i++) {
      if (((ActionCode) actions.get(i)).getName().equals(name)) {
        c = (ActionCode) actions.get(i);
        break;
      }
    }

    if (c == null) {
      c = new ActionCode(name);
      actions.add(c);
    }


    StringBuffer b = new StringBuffer(stmts);
    b.append(c.getCode());
    c.setCode(b.toString());
  }

  public void addToAction(String name, String stmts) {
    ActionCode c = null;
    for (int i = 0; i < actions.size(); i++) {
      if (((ActionCode) actions.get(i)).getName().equals(name)) {
        c = (ActionCode) actions.get(i);
        break;
      }
    }

    if (c == null) {
      c = new ActionCode(name);
      actions.add(c);
    }

    /*
    if (c == null) {
      System.out.println("should remove: " + name);

      c = new ActionCode(name);
      actions.add(c);
      actionsToRemove.add(c);
    } else {
      ActionCode orig = c.copy();
      actionsToAdd.add(orig);
      actionsToRemove.add(c);
    }
    */

    StringBuffer b = new StringBuffer(c.getCode());
    b.append(stmts);

    //System.out.println(c.getName());
    //System.out.println(b.toString());

    c.setCode(b.toString());
  }

  public void createDefaultAction(String name, String stmts) {
    ActionCode c = new ActionCode(name);
    c.setCode(stmts);
    actions.add(c);
  }

  /**
   * Does post compile clean up.
   */
  public void postCompile() {
    varNames.clear();
    ArrayList remove = new ArrayList();
    for (int i = 0; i < actions.size(); i++) {
      ActionCode c = (ActionCode) actions.get(i);
      if (c.getName().startsWith("_")) {
        remove.add(c);
      }
    }

    actions.removeAll(remove);

    /*
    actions.removeAll(actionsToRemove);
    actions.addAll(actionsToAdd);

    actionsToAdd.clear();
    actionsToRemove.clear();

    System.out.println("actions size post clean: " + actions.size());
    */


  }

  /**
   * Build a dynamic type from this Actions collection.
   */
  public NQPyDynamicClass buildType(NQPyDynamicClass dt) {
    for (int i = 0; i < actions.size(); i++) {
      ActionCode code = (ActionCode) actions.get(i);
      String retType = code.getRetType();
      if (retType == null || retType.trim().length() == 0 || retType.equals("void")) {
        dt.addMethod(code.getName(), void.class, new Class[]{});
      } else {
        dt.addMethod(code.getName(), code.getRetType(), new String[]{});
      }
    }

    return dt;
  }
  
  // checks if a name is a dynamic type and returns the fully qualified 
  // name of the type.
  private String getFQNDynamicType(Hashtable dynTypes, String name) {
    if (name.trim().length() == 0) return null;
    if (dynTypes.containsKey(name)) return name;
    for (Iterator iter = dynTypes.keySet().iterator(); iter.hasNext(); ) {
      String dynName = (String) iter.next();
      if (dynName.endsWith(name)) return dynName;
    }
    
    return null;
  }

  /**
   * Compile the action code into bytecode methods. selfName is used
   * as the object type for the self variable.
   *
   * @param selfName the name of the object whose methods these are.
   */
  public void compile(ByteCodeGenerator generator, String selfName) {
    Hashtable dynTypes = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) dynTypes.get(selfName);
    
    for (int i = 0; i < actions.size(); i++) {
      ActionCode code = (ActionCode) actions.get(i);
      String blockName = selfName + "." + code.getName();
      EvolverModelCompiler.putInBeanTable(blockName, code.getDefiningProducer());
      
      ArrayList imports = code.getImports();
      for (int j = 0; j < imports.size(); j++) {
        NQPyClassFactory.addImport((String) imports.get(j));
      }
      NQPyClassFactory.addImport(dt.getPackage() + ".*");
      
      String retType = code.getRetType();
      NQPyBlockCompiler compiler;
      String fqnDynType = getFQNDynamicType(dynTypes, retType);
      if (fqnDynType != null) {
        compiler = new NQPyBlockCompiler(code.getName(), fqnDynType, generator.getClassFile());
      } else {
        Class retClass;
        if (retType == null || retType.trim().length() == 0 || retType.equals("void"))
          retClass = void.class;
        else {
          NQPyClass clazz = NQPyClassFactory.makeNQPyClass(retType);
          if (clazz == null || clazz.getJClass() == null) {
            CompilerException ex = new CompilerException("Return type '" + retType + "' not found", 
                    new ClassNotFoundException("Class '" + retType + "' not found."));
            ex.setBlockName(blockName);
            ex.setLineNum(-1);
            throw ex;
          }
          
          retClass = clazz.getJClass();
        }

        compiler = new NQPyBlockCompiler(code.getName(), retClass, generator.getClassFile());
      }
      
      compiler.setSelf(dt);
      compiler.setBlockName(blockName);
      try {
        compiler.compile(code.getCode());
      } catch (TokenMgrError er) {
        CompilerException ex = new CompilerException(er.getMessage());
        ex.setBlockName(blockName);
        ex.setLineNum(er.errorLine);
        throw ex;
      }
    }
  }

  public String toXML() {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < actions.size(); i++) {
      ActionCode code = (ActionCode) actions.get(i);
      b.append("  ");
      b.append(code.toXML());
      b.append("\n");
    }
    return b.toString();
  }
}
