package uchicago.src.simbuilder.beans;

import org.w3c.dom.Element;
import uchicago.src.sim.util.RepastException;

import java.util.ArrayList;

/**
 * Abstract base class for components that add charting capability to
 * a model.
 *
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:25 $
 */

public abstract class ChartComponent extends AbstractEvolverComponent implements Schedulable {
  
  protected ScheduleDetails details = new ScheduleDetails();
  protected String name = "chart";
  protected String title = "A Chart";
  protected FileProperty fileProperty = new FileProperty(false, "");
  protected double yAxis = 10;
  protected double xAxis = 100;
  public String xAxisTitle = "Time";
  public String yAxisTitle = "yAxis";
  protected AbstractEvolverModel model;

  public void addToParent(EvolverParent parent) throws RepastException {
    parent.addComponent(this);
    if (parent instanceof AbstractEvolverModel) model = (AbstractEvolverModel) parent;
  }

  public void removeFromParent(EvolverParent parent) {
    parent.removeComponent(this);
  }

  public String getName() {
    return name;
  }

  public void setName(String val) {
    name = val;
  }

  public String getTitle() {
    return title;
  }

  public void setTitle(String val) {
    title = val;
  }

  public String getXAxisTitle() {
    return xAxisTitle;
  }

  public void setXAxisTitle(String xAxisTitle) {
    this.xAxisTitle = xAxisTitle;
  }

  public String getYAxisTitle() {
    return yAxisTitle;
  }

  public void setYAxisTitle(String yAxisTitle) {
    this.yAxisTitle = yAxisTitle;
  }

  public void setFileName(FileProperty val) {
    fileProperty = val;

    ArrayList a = details.getActionNames();
    if (fileProperty.doWrite()) {
      if (!a.contains("writeToFile")) a.add("writeToFile");
    } else {
      if (a.contains("writeToFile")) a.remove("writeToFile");
    }
  }

  public FileProperty getFileName() {
    return fileProperty;
  }

  public void setInitialYAxisSize(double val) {
    yAxis = val;
  }

  public double getInitialYAxisSize() {
    return yAxis;
  }

  public void setInitialXAxisSize(double val) {
    xAxis = val;
  }

  public double getInitialXAxisSize() {
    return xAxis;
  }

  /**
   * Gets the ScheduleDetails representing the schedule property for
   * this ChartComponent. These details are for display in an editor
   * and not for compilation. schedule() returns ScheduleDetails for
   * compilation.
   */
  public ScheduleDetails getSchedule() {
    return details;
  }

  /**
   * Sets the ScheduleDetails representing the schedule property for
   * this ChartComponent.
   */
  public void setSchedule(ScheduleDetails val) {
    details = val;
  }

  /**
   * Returns the XML representation of this ChartComponent's properties.
   * <b>Note</b> the FileProperty is excluded as that produces its own
   * XML via the <code>FileProperty.toXML()</code> call. The returned XML
   * begins with name=" and ends with a ".
   */
  protected String getCommonXML() {
    StringBuffer b = new StringBuffer("name=\"");
    b.append(name);

    b.append("\" title=\"");
    b.append(title);

    b.append("\" initialYAxisSize=\"");
    b.append(yAxis);

    b.append("\" initialXAxisSize=\"");
    b.append(xAxis);
    b.append("\"");

    return b.toString();
  }

  /**
   * Loads the properties described in the specified Element into this class.
   * This includes the name, title, initial axes sizes, schedule details, and
   * the FileProperty.
   *
   * @param elem the Element containing the property values
   */
  protected void loadCommonXML(Element elem) {
    setName(elem.getAttribute("name"));
    setTitle(elem.getAttribute("title"));
    setInitialYAxisSize(XMLLoader.getDoubleVal(elem, "initialYAxisSize"));
    setInitialXAxisSize(XMLLoader.getDoubleVal(elem, "initialXAxisSize"));
    setSchedule(XMLLoader.getScheduleDetails(elem));

    Element prop = (Element) XMLLoader.getChildElementsByName(elem,
            EvolverConstants.XMLNS +
            ":file_property").get(0);
    fileProperty.setDoWrite(XMLLoader.getBooleanVal(prop, "doWrite"));
    String txt = prop.getAttribute("fileName");
    if (txt == null) txt = "";
    fileProperty.setFileName(txt);
    setFileName(fileProperty);
  }
}
