package uchicago.src.simbuilder.beans;

import ViolinStrings.Strings;
import org.w3c.dom.Element;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.simbuilder.util.BooleanMessage;
import uchicago.src.simbuilder.util.RepastPyUtilities;
import uchicago.src.nqpy.compiler.AbstractCompilerException;
import uchicago.src.sim.util.RepastException;

import javax.swing.*;
import java.beans.PropertyEditorManager;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:25 $
 */
public class DataRecorder extends AbstractEvolverComponent implements ActionCodeProducer, Schedulable {

  private static final String RECORD = "record";
  private static final String WRITE = "write";

  private String name = "dataRecorder";
  private String headerComment = "";
  private String fileName = "data.txt";
  private char delimiter = ',';
  private ScheduleDetails details = new ScheduleDetails();
  private AbstractEvolverModel model;

  private DataSourceActions dataSources = new DataSourceActions();

  static {
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.DataSourceActions.class,
            uchicago.src.simbuilder.beans.DSActionsEditor.class);
  }

  public DataRecorder() {
    // the record and write actionNames to details
    ArrayList a = new ArrayList();
    a.add(RECORD);
    a.add(WRITE);
    details.setActionNames(a);
  }

  public String getHeaderComment() {
    return headerComment;
  }

  public void setHeaderComment(String headerComment) {
    this.headerComment = headerComment;
  }

  /**
   * Performs a precompile check making sure that the component is ready to compile.
   *
   * @return
   */
  public BooleanMessage sanityCheck() {
    if (!RepastPyUtilities.isValidJavaIdentifier(name)) {
      return new BooleanMessage("Invalid name '" + name + "'" +
              ". Valid names contain only letters, numbers, or '_'",
                false);
    }
    
    return BooleanMessage.TRUE;
  }

  public void displayEditor(JFrame frame, AbstractCompilerException ex) {
    DSActionsEditor editor = new DSActionsEditor();
    editor.setValue(getDataSources());
    editor.addPropertyChangeListener(this);
    DSActionsEditorPanel panel = new DSActionsEditorPanel(dataSources);
    panel.addError(ex);
    panel.setVarNames(dataSources.getVarNames());
    panel.display(frame, editor);

    setDataSources((DataSourceActions) editor.getValue());
  }

  public DataSourceActions getDataSources() {
    if (model != null) {
      this.dataSources.clearVarNames();
      Actions actions = model.getActions();
      java.util.List varNames = actions.getVarNames();
      for (int i = 0, n = varNames.size(); i < n; i++) {
        this.dataSources.addVarName((String)varNames.get(i));
      }
    }
    
    return dataSources;
  }

  public void setDataSources(DataSourceActions dataSources) {
    this.dataSources = dataSources;
    
    for (Iterator iter = this.dataSources.iterator(); iter.hasNext();) {
      DataSourceActionCode ds = (DataSourceActionCode) iter.next();
      ds.setDefiningProducer(this);
    }
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getFileName() {
    return fileName;
  }

  public void setFileName(String fileName) {
    this.fileName = fileName;
  }

  public char getDelimiter() {
    return delimiter;
  }

  public void setDelimiter(char delimiter) {
    this.delimiter = delimiter;
  }

  public ScheduleDetails getSchedule() {
    return details;
  }

  public void setSchedule(ScheduleDetails details) {
    this.details = details;
  }

  public String buildStmts() {
    StringBuffer b = new StringBuffer("self.");
    b.append(name);
    b.append(" = DataRecorder(\"");
    b.append(fileName);
    b.append("\", self, \"");
    b.append(headerComment);
    b.append("\")\n");

    for (Iterator iter = dataSources.iterator(); iter.hasNext();) {
      DataSourceActionCode ds = (DataSourceActionCode) iter.next();
      b.append("self.");
      b.append(name);
      b.append(".");
      b.append(ds.getBuildStatement());
    }

    return b.toString();
  }

  public void compile(String dir, ByteCodeGenerator generator) {

  }

  public String setupStmts() {
    return null;
  }

  public ScheduleDetails schedule() {
    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail d = details.getDetail(i);
      d.setTarget("Data Recorder: " + name);
      StringBuffer b = new StringBuffer("self.");
      b.append(name);
      if (d.getActionName().equals(RECORD)) {
        b.append(".record()\n");
      } else {
        b.append(".write()\n");
      }

      d.setNQPy(b.toString());
    }

    return details;
  }

  public String displayStmts() {
    return null;
  }

  public ArrayList methods() {
    ArrayList list = new ArrayList();
    for (Iterator iter = dataSources.iterator(); iter.hasNext();) {
      DataSourceActionCode code = (DataSourceActionCode) iter.next();
      DataSourceActionCode copy = (DataSourceActionCode) code.copy();
      list.add(copy);
    }

    return list;
  }

  public void addToParent(EvolverParent parent) throws RepastException {
    parent.addComponent(this);
    if (parent instanceof AbstractEvolverModel) {
      model = (AbstractEvolverModel) parent;
    }
  }

  public void removeFromParent(EvolverParent parent) {
    parent.removeComponent(this);
  }


  public Hashtable fields() {
    Hashtable h = new Hashtable();
    h.put(name, "uchicago.src.sim.analysis.DataRecorder");
    return h;
  }


  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"Component\" class=\"");
    b.append(getClass());
    b.append("\" name=\"");
    b.append(name);
    b.append("\" fileName=\"");
    b.append(fileName);
    b.append("\" headerComment=\"");
    b.append(headerComment);
    b.append("\" delimiter=\"");
    b.append(this.delimiter);
    b.append("\" >");
    out.write(b.toString());
    out.newLine();
    out.write(dataSources.toXML());
    out.newLine();
    getSchedule();
    out.write(details.toXML());
    out.newLine();
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
    out.newLine();

  }

  public void loadFromXML(Element elem) {
    setName(elem.getAttribute("name"));
    setFileName(elem.getAttribute("fileName"));
    setHeaderComment(elem.getAttribute("headerComment"));
    setDelimiter(elem.getAttribute("delimiter").charAt(0));
    setDataSources(XMLLoader.getDSActions(elem));
    setSchedule(XMLLoader.getScheduleDetails(elem));

    // the record and write actionNames to details
    ArrayList a = new ArrayList();
    a.add(RECORD);
    a.add(WRITE);
    details.setActionNames(a);
    dataSources.addVarName("model");
  }
}
