package uchicago.src.simbuilder.beans;

/**
 * ActionCode class for DataSources.
 *
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:25 $
 */
public class DataSourceActionCode extends ActionCode {

  public static final int NUMERIC = 0;
  public static final int OBJECT = 1;

  private int type = 0;
  private int maxIntDigits = -1;
  private int maxFracDigits = -1;

  public DataSourceActionCode(String name) {
    super(name);
  }

  public Object copy() {
    DataSourceActionCode newCode = new DataSourceActionCode(name);
    newCode.retType = retType;
    newCode.access = access;
    newCode.code = code;
    newCode.imports = imports;
    newCode.paramNames = paramNames;
    newCode.type = type;
    newCode.maxFracDigits = maxFracDigits;
    newCode.maxIntDigits = maxIntDigits;
    newCode.definingBean = definingBean;

    return newCode;
  }

  public boolean isNumeric() {
    return type == NUMERIC;
  }

  public void setType(int val) {
    type = val;
    if (isNumeric())
      setRetType("double");
    else
      setRetType("java.lang.Object");
  }

  public boolean isObject() {
    return type == OBJECT;
  }

  public int getMaxIntDigits() {
    return maxIntDigits;
  }

  public void setMaxIntDigits(int maxIntDigits) {
    this.maxIntDigits = maxIntDigits;
  }

  public int getMaxFracDigits() {
    return maxFracDigits;
  }

  public void setMaxFracDigits(int maxFracDigits) {
    this.maxFracDigits = maxFracDigits;
  }

  public String getBuildStatement() {
    StringBuffer b;
    if (type == NUMERIC) {
      b = new StringBuffer("createNumericDataSource(\"");
      b.append(name);
      b.append("\", self, \"");
      b.append(name);
      b.append("\", ");
      b.append(maxIntDigits);
      b.append(", ");
      b.append(maxFracDigits);
      b.append(")\n");
    } else {
      b = new StringBuffer("createObjectDataSource(\"");
      b.append(name);
      b.append("\", self, \"");
      b.append(name);
      b.append("\")\n");
    }

    return b.toString();
  }

  public String toXML() {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XMLNS);
    b.append(":dsAction name=\"");
    b.append(name);
    b.append("\" imports=\"");
    for (int i = 0; i < imports.size(); i++) {
      if (i != 0) b.append(" ");
      b.append(imports.get(i));
    }

    b.append("\" retVal=\"");
    b.append(retType);

    b.append("\" maxFracDigits=\"");
    b.append(maxFracDigits);
    b.append("\" maxIntDigits=\"");
    b.append(maxIntDigits);

    b.append("\" type=\"");
    b.append(type);

    b.append("\" paramNames=\"");
    for (int i = 0; i < paramNames.size(); i++) {
      if (i != 0) b.append(" ");
      b.append(paramNames.get(i));
    }

    b.append("\">");
    b.append("\n<![CDATA[\n");
    b.append(code);
    b.append("\n]]>\n</");
    b.append(EvolverConstants.XMLNS);
    b.append(":dsAction>\n");
    return b.toString();
  }
}
