package uchicago.src.simbuilder.beans;

import org.w3c.dom.Element;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.sim.util.RepastException;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Serializable;
import java.beans.PropertyChangeListener;

/**
 * The common interface for all beans in the EvolverTool.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public interface EvolverBean extends PropertyChangeListener {

  public static final String MODEL_NAME = "model.name";
  public static final String AGENT_NAME = "agent.name";
  public static final String PROJ_NAME = "proj.name";


  // these templates can be used in place of the
  // actual agent and model class names when these are not
  // knowN beforehand. The actual class names will be substituted
  // for these placeholders during compilation.
  public static final String AGENT_NAME_TEMPLATE = "__AGENT_NAME__";
  public static final String MODEL_NAME_TEMPLATE = "__MODEL_NAME__";
  public static final String AGENT_UNQ_NAME_TEMPLATE = "__AGENT_UNQ_NAME__";


  // this is a key value for the "details" hashtable in the
  // ByteCodeGenerator. This key will return a hashable whoses keys
  // are the class names of the dynamic types in this model, and whose
  // values are the Type objects themselves. The BlockCompiler needs this
  // information to do dynamic compilation of rscript when that rscript
  // refers to the dynamic types.
  public static final Integer DYN_TYPE_LIST = new Integer(1);


  /**
   * Compile this EvolverBean.
   */
  public void compile(String dir, ByteCodeGenerator generator);

  /**
   * Returns ScheduleDetails that are used as the basis of the model's
   * _schedule() method.
   */
  public ScheduleDetails schedule();

  /**
   * Adds this EvolverBean to the specified EvolverParent. This method is
   * corresponds to the accept method in the visitor pattern. Classes which
   * implement this method should call the appropriate method of the
   * EvolverParent with themselves as the parameter.
   *
   * @throws uchicago.src.sim.util.RepastException
   *          if the
   *          parent cannot successfully add the child
   */
  public void addToParent(EvolverParent parent) throws RepastException;

  /**
   * Removes this EvolverBean from the specified EvolverParent. This method is
   * corresponds to the accept method in the visitor pattern. Classes which
   * implement this method should call the appropriate method of the
   * EvolverParent with themselves as the parameter.
   */
  public void removeFromParent(EvolverParent parent);

  /**
   * Saves an xml representation of this EvolverBean to the specified
   * output stream.
   *
   * @param out the BufferedWriter to write the xml to.
   */
  public void saveXML(BufferedWriter out) throws IOException;

  /**
   * Loads the beans properties from the specified xml element.
   */
  public void loadFromXML(Element elem);

}
