package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.codegen.ByteCodeGenerator;

import java.beans.PropertyChangeListener;
import java.util.Hashtable;

/**
 * An EvolverBean that is dynamically compiled to ByteCode.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public interface EvolverCompileable extends EvolverBean {

  /**
   * Builds a Type object corresponding to this EvolverCompileable and
   * places it in the specified Hashtable. The key of the Hashtable is
   * the Class name of this Type and the value is the Type object itself.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator);

  /**
   * Sets the package in which the class produced by this EvolverCompileable
   * will reside.
   *
   * @param pkg the name of the package
   */

  public void setPackage(String pkg);

  /**
   * Gets the class name for this EvolverCompileable.
   */
  public String className();

  /**
   * Gets the Actions defined by this EvolverCompileable. Actions are
   * compiled into java methods during compilation.
   */
  public Actions getActions();

  /**
   * Sets the Actions associated with this EvolverCompileable.
   *
   * @param val the Actions
   */
  public void setActions(Actions val);

  public Fields getParameters();

  public void setParameters(Fields val);

  /**
   * Perform any post-compile clean up. Removing non-user defined
   * internally created actions, for example.
   */
  public void postCompile();

  /**
   * Adds a listener to listen for changes to the bean properties.
   */
  public void addPropertyChangeListener(PropertyChangeListener l);

  /**
   * Removes a listener.
   */
  public void removePropertyChangeListener(PropertyChangeListener l);
}
