package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.util.BooleanMessage;

import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Interface for Evolver components. Components encapsulate information
 * about how a class should be instantiated and used by a dynamically
 * compiled class.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public interface EvolverComponent extends EvolverBean {
  
  /**
   * Performs a precompile check making sure that the component is ready to compile.
   * @return
   */ 
  public BooleanMessage sanityCheck();

  /**
   * Returns a String of rscript statements to be added to an
   * EvolverModel's _build method. Can return null if nothing is to be added.
   */
  public String buildStmts();

  /**
   * Returns a String of rscript statements to be added to an
   * EvolverModel's _setup method. Can return null if nothing is to be
   * added.
   */
  public String setupStmts();

  /**
   * Returns a String of rscript statements to be added to an
   * EvolverModel's _display method. Can return null if nothing is to be
   * added.
   */
  public String displayStmts();

  /**
   * Returns a Hashtable describing the fields in the model required by this
   * EvolverComponent. The key is the fieldName and the value is the
   * class of the field.
   */
  public Hashtable fields();

  /**
   * Returns a list of ActionCode objects that become methods in the model.
   */
  public ArrayList methods();

}
