package uchicago.src.simbuilder.beans;

import java.util.List;

/**
 * An interface for simbuilder models. All model classes must
 * implement this interface.
 * <p/>
 * $Revision: 1.2 $ $Date: 2004/11/08 15:22:29 $
 */
public interface EvolverModel extends EvolverCompileable, EvolverParent, Schedulable {

  /**
   * Returns the ModelParameters associated with this model.
   */
  public Fields parameters();

  /**
   * Returns the Actions associated with this model.
   */
  public Actions actions();

  /**
   * Gets the List of EvolverCompileables associated with this model.
   */
  public List compileables();

  /**
   * Gets the List of EvolverComponents associated with this model.
   */
  public List components();

  /**
   * Gets the EvSchedule (the master schedule) associated with this model.
   */
  public EvSchedule masterSchedule();

  /**
   * Gets the display name for this model.
   */
  public String displayName();

  /**
   * Gets nqpy statements that become part of the compiled class's
   * _build() method.
   */
  public String buildStmts();
  
  /**
   * Gets nqpy statements that become the first part of the compiled class's
   * _buildStatement. These statements are guaranteed to execute before any
   * agent added code executed in _build.
   */ 
  public String preBuildStmts();
  
  /**
   * Gets nqpy statements that become the first part of the compiled class's 
   * _displayStatement. These statements are guaranteed to execute before any
   * agent or component added code executed in _display.
   * 
   * @return
   */ 
  public String preDisplayStmts();
  
   /**
   * Gets nqpy statements that become the first part of the compiled class's 
   * _setup statements. These statements are guaranteed to execute before any
   * agent or component added code executed in _setup.
   * 
   * @return
   */ 
  public String preSetupStmts();

  /**
   * Gets the superClass of the class created by the EvolverModel.
   */
  public Class getDynamicSuperclass();

}
