package uchicago.src.simbuilder.beans;

import uchicago.src.sim.util.RepastException;

import java.util.ArrayList;

/**
 * An EvolverBean that can itself contain other EvolverBeans. Classes
 * that implement this interface function as Vistor objects according
 * to the Visitor pattern. The client will call
 * <code>child.addToParent(EvolverParent p) {
 * p.addChild(this);
 * }
 * </code>
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public interface EvolverParent extends EvolverBean {

  /**
   * Adds the specified component to this EvolverParent
   */
  public void addComponent(EvolverComponent child) throws RepastException;

  /**
   * Adds the specified compileable to this EvolverParent
   */
  public void addCompileable(EvolverCompileable child) throws RepastException;

  /**
   * Removes the specified component to this EvolverParent
   */
  public void removeComponent(EvolverComponent child);

  /**
   * Removes the specified compileable to this EvolverParent
   */
  public void removeCompileable(EvolverCompileable child);

  /**
   * Gets a the children of this EvolverParent.
   */
  public ArrayList children();
}
