package uchicago.src.simbuilder.beans;

import ViolinStrings.Strings;

/**
 * Represents a model or agent field. This can be a simple variable, a
 * variable with accessor methods, or a variable with accesor methods and
 * probeable (a parameter).
 *
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class FieldData {

  public static final String xmlName = EvolverConstants.XMLNS + ":parameter";

  private String type, defaultValue, name;
  private boolean probe = false;
  private boolean isImplicit = false;

  // legacy
  private boolean accessible = true;

  public FieldData(String name, String type, String defVal, boolean probeable) {
    this(name, type, defVal, probeable, false);
  }

  public FieldData(String name, String type, String defVal, boolean probeable,
                   boolean accessible) {
    this.type = type;
    defaultValue = defVal;
    this.name = name;
    probe = probeable;
    this.accessible = accessible;
    if (probe) this.accessible = true;

  }

  public String getType() {
    return type;
  }

  public void setType(String type) {
    this.type = type;
  }

  public String getDefaultValue() {
    return defaultValue;
  }

  public void setDefaultValue(String defaultValue) {
    this.defaultValue = defaultValue;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public boolean isProbeable() {
    return probe;
  }

  public void setProbeable(boolean probe) {
    this.probe = probe;
    if (probe) accessible = true;
  }

  public boolean isAccessible() {
    return accessible;
  }

  public void setAccessible(boolean accessible) {
    this.accessible = accessible;
  }

  public boolean isImplicit() {
    return isImplicit;
  }

  public void setImplicit(boolean implicit) {
    isImplicit = implicit;
  }

  public String toXML() {
    StringBuffer b = new StringBuffer("<");
    b.append(xmlName);
    b.append(" name=\"");
    b.append(name);
    b.append("\" type=\"");
    b.append(type);
    b.append("\" defaultValue=\"");
    if (defaultValue != null) {
      String val = Strings.change(defaultValue, "\"", "&quot;");
      val = Strings.change(val, "'", "&apos;");
      val = Strings.change(val, ">", "&gt;");
      val = Strings.change(val, "<", "&lt;");
      b.append(val);
    }
    b.append("\" probe=\"");
    b.append(probe);
    b.append("\" accessible=\"");
    b.append(accessible);
    b.append("\" />");

    return b.toString();
  }

  public String toString() {
    StringBuffer b = new StringBuffer("[");
    b.append(name);
    b.append(", ");
    b.append(type);
    b.append(", ");
    b.append(defaultValue);
    b.append(", ");
    b.append(probe);
    b.append("]");
    return b.toString();
  }
}
