package uchicago.src.simbuilder.beans;

import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.simbuilder.codegen.Modifier;
import uchicago.src.simbuilder.util.BooleanMessage;
import uchicago.src.simbuilder.util.RepastPyUtilities;

import java.io.PrintWriter;
import java.util.*;

/**
 * Stores the user-defineable fields of an EvolverCompileable. Fields becomes
 * ivar in EvolverCompileables and may have accessor methods.
 * <p/>
 * $Revision: 1.2 $ $Date: 2004/12/10 22:00:22 $
 */
public class Fields {

  private List fields = new ArrayList();
  private Map implicitParams = new HashMap();
  private FieldData[] tmpFields;

  class NamesIterator implements Iterator {

    int index = 0;
    String name = null;

    public NamesIterator() {
      if (fields.size() > 0) {
        name = ((FieldData) fields.get(index)).getName();
      }
    }

    public boolean hasNext() {
      return name != null;
    }

    public Object next() {
      index++;
      String tmp = name;
      if (tmp == null) throw new NoSuchElementException();
      name = null;
      if (index < fields.size()) {
        name = ((FieldData) fields.get(index)).getName();
      }

      return tmp;
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }
  }

  public void addField(String name, String type, String defVal,
                       boolean probe) {
    fields.add(new FieldData(name, type, defVal, probe));
  }

  public void addField(String name, String type, String defVal,
                       boolean probe, boolean accessible) {
    fields.add(new FieldData(name, type, defVal, probe, accessible));
  }

  public void addImplicitField(Object tag, String name, String type,
                               String defVal, boolean probe) {
    addImplicitField(tag, name, type, defVal, probe, false);
  }

  public void addImplicitField(Object tag, String name, String type,
                               String defVal, boolean probe, boolean accessible) {
    FieldData field = new FieldData(name, type, defVal, probe, accessible);
    field.setImplicit(true);
    fields.add(field);
    ArrayList l = (ArrayList) implicitParams.get(tag);
    if (l == null) {
      l = new ArrayList();
      implicitParams.put(tag, l);
    }

    l.add(field);
  }

  public void removeImplicitFields(Object tag) {
    ArrayList list = (ArrayList) implicitParams.remove(tag);
    if (list != null) {
      for (int i = 0; i < list.size(); i++) {
        fields.remove(list.get(i));
      }
    }
  }

  public void addField(FieldData data) {
    fields.add(data);
  }

  public void addFields(Fields fs) {
    for (Iterator iter = fs.fields.iterator(); iter.hasNext();) {
      FieldData data = (FieldData) iter.next();
      addField(data.getName(), data.getType(), data.getDefaultValue(), data.isProbeable(),
              data.isAccessible());
    }
  }

  /**
   * Returns an iterator over the list of all parameter names.
   */
  public Iterator namesIter() {
    return new NamesIterator();
  }

  public void remove(FieldData data) {
    fields.remove(data);
  }

  /**
   * Returns a collection of all the parameters.
   */
  public Collection fields() {
    return new ArrayList(fields);
  }

  public int size() {
    return fields.size();
  }

  public Fields copy() {
    Map oldNewMap = new HashMap();
    Fields fs = new Fields();
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData fd = (FieldData) iter.next();
      FieldData copy = new FieldData(fd.getName(), fd.getType(), fd.getDefaultValue(),
              fd.isProbeable(), fd.isAccessible());
      copy.setImplicit(fd.isImplicit());
      fs.addField(copy);
      oldNewMap.put(fd, copy);
    }

    Map newIP = new HashMap();
    for (Iterator tags = implicitParams.keySet().iterator(); tags.hasNext();) {
      Object tag = tags.next();
      List pList = (List) implicitParams.get(tag);
      List newList = new ArrayList();
      newIP.put(tag, newList);
      for (int i = 0, n = pList.size(); i < n; i++) {
        newList.add(oldNewMap.get(pList.get(i)));
      }
    }

    fs.implicitParams = newIP;
    return fs;
  }


  public String toString() {
    return fields.toString();
  }
  
  public NQPyDynamicClass buildType(NQPyDynamicClass dt) {
    return buildType(dt, new FieldData[0], false);
  }

  public NQPyDynamicClass buildType(NQPyDynamicClass dt, boolean isParent) {
    return buildType(dt, new FieldData[0], isParent);
  }
  
   /**
   * Build a dynamic from this ModelParameters collection.
   */
  public NQPyDynamicClass buildType(NQPyDynamicClass dt, FieldData[] tmpFields) {
     return buildType(dt, tmpFields, false);
  }

  /**
   * Build a dynamic from this Parameters collection.
   */
  public NQPyDynamicClass buildType(NQPyDynamicClass dt, FieldData[] tmpFields, boolean isParent) {
    this.tmpFields = new FieldData[tmpFields.length];
    for (int i = 0; i < tmpFields.length; i++) {
      fields.add(tmpFields[i]);
      this.tmpFields[i] = tmpFields[i];
    }
    
    //Enumeration keys = fields.keys();
    //while (keys.hasMoreElements()) {
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData fd = (FieldData) iter.next();
      if (isParent) dt.addProtectedField(fd.getName(), fd.getType()); 
      else dt.addField(fd.getName(), fd.getType());

      if (fd.isAccessible()) {
        String methodName = fd.getName().substring(0, 1).toUpperCase() +
                fd.getName().substring(1, fd.getName().length());
        dt.addMethod("get" + methodName, fd.getType(), new String[]{});
        dt.addMethod("set" + methodName, "void", new String[]{fd.getType()});
      }
    }

    for (int i = 0; i < tmpFields.length; i++) {
      fields.remove(tmpFields[i]);
    }

    return dt;
  }

  /**
   * Checks if two fields have the same name.
   *
   * @return
   */
  public BooleanMessage sanityCheck(String ownerName) {
    Set names = new HashSet();
    String ownerPhrase = ownerName.length() == 0 ? "" : " in " + ownerName;
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData fd = (FieldData) iter.next();
      String name = fd.getName();
      if (names.contains(name))
        return new BooleanMessage("Duplicate field name '" + name +
                "'" + ownerPhrase, false);
      names.add(name);

      if (!RepastPyUtilities.isValidJavaIdentifier(name)) {
        return new BooleanMessage("Invalid field name '" + name + "'" + ownerPhrase +
                ". Field names may only contain letters, numbers, or '_'",
                false);
      }

      if (fd.getDefaultValue() != null && fd.getDefaultValue().length() > 0) {
        // check if the default value is valid for that type
        String type = fd.getType();
        if (RepastPyUtilities.isNumericType(type)) {
          if (!RepastPyUtilities.isValidValueForNumericType(fd.getDefaultValue(),
                  fd.getType())) {
            return new BooleanMessage("Invalid default value for field '" + name + "'" + ownerPhrase + ".",
                    false);

          } else {
            continue;
          }
        } else if (type.equals("boolean")) {
          if (type.equals("boolean") || type.equals("java.lang.boolean")) {
            if (!RepastPyUtilities.isValidBooleanValue(fd.getDefaultValue())) {
              return new BooleanMessage("Invalid default value for field '" + name + "'" + ownerPhrase + ".",
                    false);
            }
            continue;
          }
        } else if (type.equals("String") || type.equals("java.lang.String")) {
          continue;
        } else {
          return new BooleanMessage("Default field value not supported for '" + name + "'" + ownerPhrase + ".",
                  false);
        }

      }
    }
    return BooleanMessage.TRUE;
  }
  
  /**
   * Compiles the FieldData contains by this Fields into actual fields
   * in a java class. If arrayName is not null then that an array with that name
   * will be created in the compiled class and any fields marked
   * as parameters will have their name placed in this array.
   *
   * @param generator
   * @param arrayName the name of the parameter array
   */

  public void compile(ByteCodeGenerator generator, String arrayName) {
    compile(generator, arrayName, false);
  }

  /**
   * Compiles the FieldData contains by this Fields into actual fields
   * in a java class. If arrayName is not null then that an array with that name
   * will be created in the compiled class and any fields marked
   * as parameters will have their name placed in this array.
   *
   * @param generator
   * @param arrayName the name of the parameter array
   */

  public void compile(ByteCodeGenerator generator, String arrayName, boolean isProtected) {
    Modifier varMod = isProtected ? Modifier.PROTECTED : Modifier.PRIVATE;
    if (tmpFields != null) {
      for (int i = 0; i < tmpFields.length; i++) {
        fields.add(tmpFields[i]);
      }
    }
    //Enumeration keys = fields.keys();
    ArrayList probed = new ArrayList();
    //while (keys.hasMoreElements()) {
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData fd = (FieldData) iter.next();
      String pName = fd.getName();
      if (fd.isAccessible()) {
        if (fd.getDefaultValue() == null || fd.getDefaultValue().trim().length() == 0) {
          if (fd.isProbeable() && fd.getType().equals("java.lang.String")) {
            // probeable but with no default value and is string so set to empty string
            generator.createAccessor(pName, fd.getType(), "", varMod);
          } else {
            generator.createAccessor(pName, fd.getType(), varMod);
          }
        } else {
          generator.createAccessor(pName, fd.getType(), fd.getDefaultValue(), varMod);
        }
      } else {
        if (fd.getDefaultValue() == null || fd.getDefaultValue().trim().length() == 0) {
          if (isProtected) generator.createProtectedField(pName, fd.getType());
          else generator.createPrivateField(pName, fd.getType());
        } else {
          generator.setInitialFieldValue(pName, fd.getType(), fd.getDefaultValue(), varMod);
        }
      }

      if (fd.isProbeable()) {
        probed.add(fd);
      }
    }

    if (arrayName != null) generator.emitInitStringtArrayInCtor(probed.size(), arrayName);
    for (int i = 0; i < probed.size(); i++) {
      FieldData p = (FieldData) probed.get(i);
      if (arrayName != null) generator.emitAddStringToCtorArray(p.getName(), i, arrayName);
    }


    if (tmpFields != null) {
      for (int i = 0; i < tmpFields.length; i++) {
        fields.remove(tmpFields[i]);
      }
    }
  }

  /**
   * Returns a list of nqpy statements to be inserted into a _setup
   * action. This _setup method is called by setup(). E.g.
   * self.numNodes = 100;
   */
  public ArrayList getSetupStmts() {
    ArrayList list = new ArrayList();
    //Enumeration keys = fields.keys();
    //while (keys.hasMoreElements()) {
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData data = (FieldData) iter.next();
      if (data.getDefaultValue() != null && data.getDefaultValue().trim().length() > 0) {
        StringBuffer stmt = new StringBuffer("self.");
        stmt.append(data.getName());
        stmt.append(" = ");
        if (data.getType().equals("String") || data.getType().equals("java.lang.String")) {
          String defVal = data.getDefaultValue();
          if (!defVal.startsWith("\"")) data.setDefaultValue("\"" + data.getDefaultValue());
          if (!defVal.endsWith("\"")) data.setDefaultValue(data.getDefaultValue() + "\"");
        }

        if (data.getType().equals("double")) {
          stmt.append(Double.parseDouble(data.getDefaultValue()));
        } else if (data.getType().equals("int")) {
          stmt.append(Integer.parseInt(data.getDefaultValue()));
        } else if (data.getType().equals("long")) {
          stmt.append(Long.parseLong(data.getDefaultValue()));
          stmt.append("L");
        } else if (data.getType().equals("float")) {
          stmt.append("Float.parseFloat(\"" + data.getDefaultValue() + "\")");
        } else {
          stmt.append(data.getDefaultValue());
        }
        //stmt.append(";");
        list.add(stmt);
      }
    }

    if (tmpFields != null) {
      for (int i = 0; i < tmpFields.length; i++) {
        FieldData data = tmpFields[i];
        StringBuffer stmt = new StringBuffer("self.");
        stmt.append(data.getName());
        stmt.append(" = ");
        if (data.getType().equals("String") || data.getType().equals("java.lang.String")) {
          String defVal = data.getDefaultValue();
          if (!defVal.startsWith("\"")) data.setDefaultValue("\"" + data.getDefaultValue());
          if (!defVal.endsWith("\"")) data.setDefaultValue(data.getDefaultValue() + "\"");
        }

        stmt.append(data.getDefaultValue());
        stmt.append(";");
        list.add(stmt);
      }
    }

    return list;
  }

  public String toXML() {

    StringBuffer b = new StringBuffer();
    //Enumeration keys = fields.keys();
    //while (keys.hasMoreElements()) {
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData data = (FieldData) iter.next();
      if (!data.isImplicit()) {
        b.append(data.toXML());
        b.append("\n");
      }
    }

    return b.toString();
  }

  public void writePropertiesFile(PrintWriter writer) {
    Set numberTypes = new HashSet();
    numberTypes.add("int");
    numberTypes.add("long");
    numberTypes.add("double");
    numberTypes.add("float");

    writer.println("runs: 1");
    for (Iterator iter = fields.iterator(); iter.hasNext();) {
      FieldData data = (FieldData) iter.next();
      if (data.isProbeable()) {
        writer.print(data.getName());
        writer.println(" {");
        String type = data.getType();
        String setString = "  set: ";
        if (type.equals("boolean")) {
          setString = "  set_boolean: ";
        } else if (type.equals("java.lang.String")) {
          setString = "  set_string: ";
        }
        writer.print(setString);
        String defaultValue = data.getDefaultValue();
        if (defaultValue.startsWith("\"")) {
          defaultValue = defaultValue.substring(1);
        }

        if (defaultValue.endsWith("\"")) {
          defaultValue = defaultValue.substring(0, defaultValue.length() - 1);
        }


        if (defaultValue == null || defaultValue.trim().length() == 0) {
          if (numberTypes.contains(type)) {
            defaultValue = "0";
          } else if (type.equals("boolean")) {
            defaultValue = "false";
          } else {
            defaultValue = "";
          }
        }

        writer.println(defaultValue);
        writer.println("}");
        writer.println();
      }
    }
  }
}


