package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.util.BooleanMessage;
import uchicago.src.simbuilder.util.RepastPyUtilities;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * GUI for displaying and manipulating ModelParameter properties.
 * <p/>
 * $Revision: 1.3 $ $Date: 2004/11/03 22:28:44 $
 */

public class FieldsPropertyPanel extends JPanel {
  
  private static final String FIELDS_FRAME = "FIELDS_FRAME";

  BorderLayout borderLayout1 = new BorderLayout();
  JPanel tablePanel = new JPanel();
  JPanel editPanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  JLabel lblName = new JLabel();
  JTextField txtName = new JTextField();
  JLabel lblType = new JLabel();
  Object[] types = {"int", "long", "float", "double", "boolean", "java.lang.String"};
  JComboBox cmbType = new JComboBox(types);
  JLabel lblDefVal = new JLabel();
  JTextField txtDefVal = new JTextField();
  JPanel topPanel = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JScrollPane sp = new JScrollPane();
  JTable table = new JTable();
  JPanel buttonPanel = new JPanel();
  JButton btnAdd = new JButton();
  JButton btnDelete = new JButton();
  JPanel bottomPanel = new JPanel();
  JButton btnOk = new JButton();
  JButton btnCancel = new JButton();

  private JDialog dialog;
  JLabel lblProbe = new JLabel();
  JCheckBox chkProbe = new JCheckBox();
  JCheckBox chkAccess = new JCheckBox();

  private FieldsPropertyEditor editor;
  

  public FieldsPropertyPanel(Fields params) {
    try {
      jbInit();
    } catch (Exception e) {
      e.printStackTrace();
    }

    table.setModel(new FieldsTableModel(params));
    table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
    table.getColumnModel().getColumn(1).setCellEditor(new TypeEditor(new JComboBox(types)));
    table.setRowHeight(table.getRowHeight() + 4);
    addListeners();
  }

  private void jbInit() throws Exception {
    this.setLayout(borderLayout1);
    editPanel.setLayout(gridBagLayout1);
    tablePanel.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(142, 142, 142)), "Fields"), BorderFactory.createEmptyBorder(2, 2, 2, 2)));
    tablePanel.setLayout(borderLayout2);
    lblName.setHorizontalAlignment(SwingConstants.LEFT);
    lblName.setText("Name:");
    lblType.setText("Type:");
    lblDefVal.setText("Default Value:");
    cmbType.setEditable(true);
    topPanel.setLayout(borderLayout3);
    topPanel.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(142, 142, 142)), "Add Field"), BorderFactory.createEmptyBorder(2, 2, 2, 2)));
    btnAdd.setMnemonic('A');
    btnAdd.setText("Add");
    btnDelete.setMnemonic('D');
    btnDelete.setText("Delete");
    btnOk.setText("OK");
    btnCancel.setText("Cancel");
    lblProbe.setToolTipText("");
    lblProbe.setText("Parameter:");
    chkProbe.setToolTipText("");
    this.add(tablePanel, BorderLayout.CENTER);
    tablePanel.add(sp, BorderLayout.CENTER);
    sp.getViewport().add(table, null);
    this.add(topPanel, BorderLayout.NORTH);
    topPanel.add(editPanel, BorderLayout.CENTER);
    editPanel.add(lblName, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(txtName, new GridBagConstraints(1, 0, GridBagConstraints.REMAINDER, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(lblType, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(cmbType, new GridBagConstraints(1, 1, GridBagConstraints.REMAINDER, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(lblDefVal, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 0, 2), 0, 0));
    editPanel.add(txtDefVal, new GridBagConstraints(1, 2, GridBagConstraints.REMAINDER, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(new JLabel("Accessible:"), new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(chkAccess, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(lblProbe, new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(chkProbe, new GridBagConstraints(3, 3, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    topPanel.add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(btnAdd, null);
    buttonPanel.add(btnDelete, null);
    this.add(bottomPanel, BorderLayout.SOUTH);
    bottomPanel.add(btnOk, null);
    bottomPanel.add(btnCancel, null);
  }

  private void addListeners() {
    btnOk.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Fields f = getParameters();
        BooleanMessage result = f.sanityCheck("");
        if (!result.isValid()) {
          JOptionPane.showMessageDialog(FieldsPropertyPanel.this,  result.getMessage(), "Fields Error", 
                  JOptionPane.OK_OPTION);
          return;
        }
        
        RepastPyUtilities.savePreferredWindowBounds(FIELDS_FRAME, dialog.getBounds());
        if (dialog != null) dialog.dispose();
        editor.setValue(getParameters());
        editor.firePropertyChange();
      }
    });

    btnCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        RepastPyUtilities.savePreferredWindowBounds(FIELDS_FRAME, dialog.getBounds());
        if (dialog != null) dialog.dispose();
      }
    });

    chkProbe.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        if (chkProbe.isSelected()) {
          chkAccess.setSelected(true);
          chkAccess.setEnabled(false);
        } else {
          chkAccess.setEnabled(true);
        }
      }
    });

    btnAdd.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (txtName.getText().trim().length() == 0) {
          JOptionPane.showMessageDialog(FieldsPropertyPanel.this, "Field must have a name!", "Invalid Field",
                  JOptionPane.WARNING_MESSAGE);
          txtName.requestFocus();
          return;
        }
        FieldsTableModel model = (FieldsTableModel) table.getModel();
        FieldData fd = new FieldData(txtName.getText().trim(),
                cmbType.getSelectedItem().toString().trim(),
                txtDefVal.getText().toString().trim(),
                chkProbe.isSelected(),
                chkAccess.isSelected());
        model.addRow(fd);
      }
    });

    btnDelete.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (table.getSelectedRow() != -1) {
          FieldsTableModel model = (FieldsTableModel) table.getModel();
          if (model.canRemoveRow(table.getSelectedRow())) {
            model.removeRow(table.getSelectedRow());
          } else {
            JOptionPane.showMessageDialog(FieldsPropertyPanel.this, "Field is required by model and cannot be deleted",
                    "Unable to Delete", JOptionPane.OK_OPTION);
          }
        }
      }
    });
  }

  public Fields getParameters() {
    FieldsTableModel model = (FieldsTableModel) table.getModel();
    return model.getParameters();
  }

  public void display(JFrame f, FieldsPropertyEditor editor) {
    this.editor = editor;
    dialog = new JDialog(f, false);
    dialog.setTitle("Fields Editor");
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        RepastPyUtilities.savePreferredWindowBounds(FIELDS_FRAME, dialog.getBounds());
        dialog.dispose();
      }
    });

    dialog.pack();
    Rectangle bounds = RepastPyUtilities.getPreferredWindowBounds(FieldsPropertyPanel.FIELDS_FRAME, 350, 350);
    dialog.setBounds(bounds);
    uchicago.src.guiUtils.GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
  }
  
  class TypeEditor extends DefaultCellEditor {

    private JComboBox box;
  

    public TypeEditor(JComboBox box) {
      super(box);
      box.setEditable(true);
      this.box = box;
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
      if (value != null) {
        box.setSelectedItem(value);
      } else {
        box.setSelectedIndex(-1);
      }

      return box;
    }
  }


}
