package uchicago.src.simbuilder.beans;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;

/**
 * TableModel for the JTable used in FieldsPropertyPanel.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class FieldsTableModel extends AbstractTableModel {

  private static final int NUM_COLS = 5;
  private static final String[] COL_NAMES = {"Name", "Type", "Default Value",
                                             "Accessible", "Parameter"};

  private static final int NAME = 0;
  private static final int TYPE = 1;
  private static final int DEF_VAL = 2;
  private static final int ACCESSIBLE = 3;
  private static final int PROBE = 4;


  private ArrayList rows;
  private Fields params;

  public FieldsTableModel(Fields params) {
    rows = new ArrayList(params.fields());
    this.params = params;
  }

  public int getRowCount() {
    return rows.size();
  }

  public int getColumnCount() {
    return NUM_COLS;
  }

  public String getColumnName(int col) {
    return COL_NAMES[col];
  }

  public Class getColumnClass(int col) {
    return getValueAt(0, col).getClass();
  }

  public boolean isCellEditable(int row, int col) {
    FieldData p = (FieldData) rows.get(row);
    if (p.isImplicit() && (col == NAME || col == TYPE || col == ACCESSIBLE || col == PROBE)) return false;
    if (col == ACCESSIBLE) {
      // only editable if field is also not probeable.
      return !(p.isProbeable());
    }

    return true;
  }

  public Object getValueAt(int row, int col) {
    if (row < rows.size()) {
      FieldData p = (FieldData) rows.get(row);
      switch (col) {
        case NAME:
          return p.getName();
        case TYPE:
          return p.getType();
        case DEF_VAL:
          return p.getDefaultValue();
        case ACCESSIBLE:
          return p.isAccessible() ? Boolean.TRUE : Boolean.FALSE;
        case PROBE:
          return p.isProbeable() ? Boolean.TRUE : Boolean.FALSE;
        default:
          return "";
      }
    }
    return null;
  }

  public void setValueAt(Object value, int row, int col) {
    FieldData p = (FieldData) rows.get(row);
    switch (col) {
      case NAME:
        p.setName(value.toString());
        break;
      case TYPE:
        p.setType(value.toString());
        break;
      case DEF_VAL:
        p.setDefaultValue(value.toString());
        break;
      case ACCESSIBLE:
        p.setAccessible(((Boolean) value).booleanValue());
        break;
      case PROBE:
        boolean probeable = ((Boolean) value).booleanValue();
        p.setProbeable(probeable);
        if (probeable) p.setAccessible(true);
        break;
    }

    fireTableDataChanged();
  }

  public void addRow(FieldData param) {
    rows.add(param);
    params.addField(param);
    this.fireTableRowsInserted(rows.size() - 1, rows.size());
  }

  public void removeRow(int row) {
    FieldData data = (FieldData) rows.remove(row);
    params.remove(data);
    this.fireTableRowsDeleted(row, row);
  }

  public Fields getParameters() {
    return params;
  }

  public boolean canRemoveRow(int row) {
    FieldData p = (FieldData) rows.get(row);
    return !p.isImplicit();
  }
}
