package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.property.EvolverPropertyEditor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;


public class FilePropertyEditor extends EvolverPropertyEditor implements
        FocusListener {

  private JCheckBox box = new JCheckBox("Write to File");
  private JTextField field = new JTextField();
  private FileProperty fileProp;
  private JButton browse = EvolverPropertyEditor.getEditButton();

  public FilePropertyEditor() {
    panel = new JPanel();
    browse.setText("Browse");
    box.setBackground(Color.WHITE);
    panel.setBackground(Color.WHITE);
    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(box);
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(field);
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(browse);

    box.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        enable(box.isSelected());
      }
    });

    box.addFocusListener(this);
    field.addFocusListener(this);

    browse.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        FileDialog dialog =
                new FileDialog((JFrame) panel.getTopLevelAncestor(),
                        "File to Save Network Stats to",
                        FileDialog.SAVE);
        String txt = field.getText().trim();
        if (txt.length() > 0) {
          File file = new File(txt);
          if (file.getParentFile().isDirectory())
            dialog.setDirectory(file.getParent());
        }

        dialog.show();
        String file = dialog.getFile();
        if (file != null) {
          String dir = dialog.getDirectory();
          field.setText(dir + file);
          fileProp.setFileName(field.getText());
          fileProp.setDoWrite(box.isSelected());
          FilePropertyEditor.super.setValue(new FileProperty(fileProp));
          field.requestFocus();
        }
      }
    });
  }

  public void setValue(Object val) {
    fileProp = new FileProperty((FileProperty) val);
    boolean write = fileProp.doWrite();
    box.setSelected(write);
    field.setText(fileProp.getFileName());
    enable(write);
  }

  private void enable(boolean val) {
    field.setEnabled(val);
    browse.setEnabled(val);
    fileProp.setDoWrite(val);
  }

  public Object getValue() {
    if (fileProp.doWrite()) {
      if (field.getText().trim().length() == 0) fileProp.setDoWrite(false);
    }
    fileProp.setFileName(field.getText());

    return new FileProperty(fileProp);
  }

  public void focusLost(FocusEvent evt) {
    Component comp = evt.getOppositeComponent();
    if (comp != field && comp != box && comp != browse) {
      // box, field and browse have not lost focus to each otherj
      String txt = field.getText().trim();
      if (txt.length() == 0) box.setSelected(false);
      fileProp.setDoWrite(box.isSelected());
      fileProp.setFileName(field.getText());
      super.setValue(fileProp);
    }
  }

  public void focusGained(FocusEvent evt) {
  }
}
