package uchicago.src.simbuilder.beans;

import java.util.ArrayList;

/**
 * Store ScheduleDetail in a list. Suitable for PAUSE and END actions
 * where there is no need to map the actions to a particular tick.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */

// most of the methods below have been documented in AbstractScheduleUnit
// see there for more information.

public class ListScheduleUnit extends AbstractScheduleUnit {

  private ArrayList details = new ArrayList();

  public ListScheduleUnit(String name, Integer id) {
    super(name, id);
  }

  public ListScheduleUnit copy(EvSchedule schedule) {
    ListScheduleUnit unit = new ListScheduleUnit(name, id);
    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail copy = ((ScheduleDetail) details.get(i)).copy();
      copy.setEvSchedule(schedule);
      unit.addScheduleDetail(copy);
    }

    return unit;
  }

  public boolean isEmpty() {
    return details.isEmpty();
  }

  public Actions makeActions(Actions actions) {
    return detailsToActions(details, actions);
  }

  public void addScheduleDetail(ScheduleDetail detail) {
    int index = details.indexOf(detail);
    if (index == -1)
      details.add(detail);
    else {
      ScheduleDetail d = (ScheduleDetail) details.get(index);
      d.setID(detail.getID());
    }
  }

  public SchTreeNode makeNode() {
    SchTreeNode node = new SchTreeNode(name, id, true);
    node.setScheduleUnit(this);
    if (!details.isEmpty()) {
      SchTreeNode actions = new SchTreeNode("Actions", node.FIRST, false);
      actions.setActions(details);
      node.add(actions);
    }
    return node;
  }

  public void delete(ScheduleDetail detail) {
    details.remove(detail);
  }

  /**
   * Returns an XML representation of this ListScheduleUnit.
   */
  public String toXML() {
    if (!details.isEmpty()) {
      StringBuffer b = new StringBuffer("<");
      b.append(xmlString);
      b.append(" order=\"");
      for (int i = 0; i < details.size(); i++) {
        ScheduleDetail det = (ScheduleDetail) details.get(i);
        if (i != 0) b.append(" ");
        b.append(det.getID());
      }
      b.append("\"/>\n");
      return b.toString();
    }
    return "";
  }
}
