package uchicago.src.simbuilder.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Store ScheduleDetail in a map. Maps a list of ScheduleDetail-s to the
 * execution tick. The map value tracks both ScheduleDetail should be
 * executed normally and those that should be executed last.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */

// most of the methods below are documented in AbstractScheduleUnit

public class MapScheduleUnit extends AbstractScheduleUnit {

  class Pair implements Serializable {
    ArrayList first = new ArrayList();
    ArrayList last = new ArrayList();
  }

  private HashMap map = new HashMap();

  public MapScheduleUnit(String name, Integer id) {
    super(name, id);
  }

  public boolean isEmpty() {
    return map.isEmpty();
  }

  public MapScheduleUnit copy(EvSchedule schedule) {
    MapScheduleUnit unit = new MapScheduleUnit(name, id);
    Iterator i = map.keySet().iterator();
    while (i.hasNext()) {
      Pair p = (Pair) map.get(i.next());
      for (int j = 0; j < p.first.size(); j++) {
        //System.out.println("FIRST SIZE: " + p.first.size() + ", THIS: " + this);
        ScheduleDetail d = (ScheduleDetail) p.first.get(j);
        ScheduleDetail copy = d.copy();
        copy.setEvSchedule(schedule);
        unit.addScheduleDetail(copy);
      }

      for (int j = 0; j < p.last.size(); j++) {
        ScheduleDetail d = (ScheduleDetail) p.last.get(j);
        ScheduleDetail copy = d.copy();
        copy.setEvSchedule(schedule);
        unit.addScheduleDetail(copy);
      }
    }

    return unit;
  }

  void showDetails() {
    Iterator i = map.keySet().iterator();

    while (i.hasNext()) {
      Pair p = (Pair) map.get(i.next());
      if (!p.first.isEmpty()) {
        for (int j = 0; j < p.first.size(); j++) {
          ScheduleDetail d = (ScheduleDetail) p.first.get(j);
          //System.out.println("!!!!!! details in in unit !!!!!");
          //System.out.println(d.getActionName() + ":\n" + d.getNQPy());
        }
      }
    }
  }


  public Actions makeActions(Actions actions) {
    Iterator i = map.keySet().iterator();
    while (i.hasNext()) {
      Pair p = (Pair) map.get(i.next());
      if (!p.last.isEmpty()) actions = detailsToActions(p.last, actions);
      if (!p.first.isEmpty()) actions = detailsToActions(p.first, actions);
    }

    return actions;
  }

  public void addScheduleDetail(ScheduleDetail detail) {
    Long l = new Long(detail.getTick());
    Pair p = (Pair) map.get(l);
    if (p == null) {
      p = new Pair();
      map.put(l, p);
    }

    if (detail.getLast()) {
      int index = p.last.indexOf(detail);
      if (index == -1)
        p.last.add(detail);
      else {
        ScheduleDetail d = (ScheduleDetail) p.last.get(index);
        // this is a bit of a hack. For some reason the nqpy might be null and
        // we need to make sure that is correct. A proper solution would figure
        // out why this was null in the first place
        if (d.getNQPy() == null) d.setNQPy(detail.getNQPy());
        if (detail.getID() != -1) d.setID(detail.getID());
      }
    } else {
      int index = p.first.indexOf(detail);
      if (index == -1) {
        p.first.add(detail);
      } else {
        ScheduleDetail d = (ScheduleDetail) p.first.get(index);
        // this is a bit of a hack. For some reason the nqpy might be null and
        // we need to make sure that is correct. A proper solution would figure
        // out why this was null in the first place
        if (d.getNQPy() == null) d.setNQPy(detail.getNQPy());
        if (detail.getID() != -1) d.setID(detail.getID());

      }
    }

    //showDetails();
  }

  public SchTreeNode makeNode() {
    SchTreeNode node = new SchTreeNode(name, id, true);
    node.setScheduleUnit(this);
    Iterator keys = map.keySet().iterator();
    while (keys.hasNext()) {
      Long tick = (Long) keys.next();
      Pair p = (Pair) map.get(tick);
      SchTreeNode tickNode = new SchTreeNode("Tick " + tick, node.TICK, false);
      tickNode.setActions(p.first);
      tickNode.setLastActions(p.last);
      node.add(tickNode);
    }

    return node;
  }

  public void delete(ScheduleDetail detail) {
    //System.out.println("THIS: " + this);
    Long l = new Long(detail.getTick());
    Pair p = (Pair) map.get(l);

    if (p != null) {
      if (detail.getLast()) {
        p.last.remove(detail);
      } else {
        boolean b = p.first.remove(detail);
        //System.out.println("deleting: " + detail + ", success: " + b);
      }
      if (p.last.size() == 0 && p.first.size() == 0) {
        map.remove(l);
      }
    }


  }

  /**
   * Returns an XML representation of this MapScheduleUnit.
   */
  public String toXML() {
    if (!map.isEmpty()) {
      StringBuffer b = new StringBuffer();
      Iterator iter = map.keySet().iterator();
      while (iter.hasNext()) {
        StringBuffer item = new StringBuffer("<");
        Long tick = (Long) iter.next();
        Pair p = (Pair) map.get(tick);
        item.append(xmlString);
        if (!p.first.isEmpty())
          b.append(makeOrderList(item.toString(),
                  p.first));
        if (!p.last.isEmpty())
          b.append(makeOrderList(item.toString(),
                  p.last));
        b.append("\n");
      }

      return b.toString();

    }

    return "";

  }

  private String makeOrderList(String prefix, ArrayList list) {
    StringBuffer first = new StringBuffer(prefix);
    first.append(" order=\"");
    first.append(getOrder(list));
    first.append("\" />");

    return first.toString();
  }


  private String getOrder(ArrayList list) {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < list.size(); i++) {
      ScheduleDetail det = (ScheduleDetail) list.get(i);
      if (det.getID() != -1) {
        b.append(det.getID());
        b.append(" ");
      }
    }

    return b.toString().trim();
  }
}
