package uchicago.src.simbuilder.beans;

import java.io.Serializable;

/**
 * Stores execution information for actions. Details the type of
 * execution (e.g. every tick), the execution tick, the name of the
 * action to execute and so forth. See *ScheduleUnit-s and EvSchedule
 * for more information.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class ScheduleDetail implements ScheduleDataConstants, Serializable {

  private String actionName;

  // an id unique across all the ScheduleDetails in a particular model.
  // This only used in saving and loading the model via xml.
  private int id = -1;

  // the script statement that actually gets scheduled.
  private String nqpy;
  private long tick = 1;
  private int type = 1;
  private boolean last = false;
  private String target = "";
  private EvSchedule schedule;

  public ScheduleDetail(String actionName, long tick, int type, boolean last) {
    this.actionName = actionName;
    this.tick = tick;
    this.type = type;
    this.last = last;

  }

  /**
   * Does a deep copy of this ScheduleDetail.
   */
  public ScheduleDetail copy() {
    ScheduleDetail d = new ScheduleDetail(actionName, tick, type, last);
    d.id = id;
    d.setTarget(target);
    d.setNQPy(nqpy);
    d.setEvSchedule(schedule);
    return d;
  }

  /**
   * Returns the name of the action for which this is a ScheduleDetail.
   */
  public String getActionName() {
    return actionName;
  }

  /**
   * Sets the target for this ScheduleDetail. The target is for informational
   * display purposes only, and is displayed in the EvScheduleEditorPane.
   * It is irrelevant to compilation etc.
   */
  public void setTarget(String val) {
    target = val;
  }

  public long getTick() {
    return tick;
  }

  public int getType() {
    return type;
  }

  public boolean getLast() {
    return last;
  }

  /**
   * Sets the NQPy statements that implement the action for which this
   * is a ScheduleDetail.
   */
  public void setNQPy(String stmts) {
    nqpy = stmts;
  }

  /**
   * Gets the NQPy statements that implement the action for which this
   * is a ScheduleDetail.
   */
  public String getNQPy() {
    return nqpy;
  }

  public boolean equals(Object o) {
    if (o instanceof ScheduleDetail) {
      ScheduleDetail other = (ScheduleDetail) o;
      return (tick == other.tick && type == other.type && last == other.last &&
              target.equals(other.target) && actionName.equals(other.actionName));
    }

    return false;
  }

  public void setEvSchedule(EvSchedule schedule) {
    this.schedule = schedule;
  }

  /**
   * Delete this ScheduleDetail from the EvSchedule, if that has been
   * set yet.
   */
  public void delete() {
    if (schedule != null) schedule.delete(this);
  }

  public String toString() {
    return target + ": " + actionName;
  }

  public void setID(int val) {
    id = val;
  }

  public int getID() {
    return id;
  }

  public String toXML() {
    String xmlName = EvolverConstants.XMLNS + ":schedule_item";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlName);
    b.append(" action=\"");
    b.append(actionName);
    b.append("\" tick=\"");
    b.append(tick);
    b.append("\" type=\"");
    b.append(type);
    b.append("\" last=\"");
    b.append(last);
    b.append("\" target=\"");
    b.append(target);
    b.append("\" id=\"");
    b.append(id);
    b.append("\"/>");

    return b.toString();
  }
}
