package uchicago.src.simbuilder.beans;

import uchicago.src.guiUtils.LongDocument;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

/**
 * GUI for editing ScheduleDetails properties.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class ScheduleDetailPropertyPanel extends JPanel {
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel topPanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel tablePanel = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel bottomPanel = new JPanel();
  FlowLayout flowLayout1 = new FlowLayout();
  JButton btnOk = new JButton();
  JButton btnCancel = new JButton();
  JScrollPane sp = new JScrollPane();
  JTable schTable;
  JPanel editPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton btnAdd = new JButton();
  JButton btnDel = new JButton();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  JLabel jLabel1 = new JLabel();
  JComboBox cmbAction = new JComboBox(new DefaultComboBoxModel());
  JLabel jLabel2 = new JLabel();
  JComboBox cmbExecution = new JComboBox(new DefaultComboBoxModel());
  JLabel jLabel3 = new JLabel();
  JTextField txtTick = new JTextField();
  JCheckBox chkLast = new JCheckBox();
  JLabel jLabel4 = new JLabel();

  private ArrayList actions;
  private JDialog dialog;
  private ScheduleDetailsPropertyEditor editor;

  class StringIntPair {
    String s;
    int val;

    public StringIntPair(String s, int val) {
      this.s = s;
      this.val = val;
    }

    public String toString() {
      return s;
    }
  }

  public ScheduleDetailPropertyPanel(ScheduleDetails details) {
    this.actions = details.getActionNames();
    schTable = new JTable(new ScheduleDetailTableModel(details));
    try {
      jbInit();
      addListeners();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setLayout(borderLayout1);
    topPanel.setLayout(borderLayout2);
    tablePanel.setLayout(borderLayout3);
    bottomPanel.setLayout(flowLayout1);
    btnOk.setMnemonic('O');
    btnOk.setText("OK");
    btnCancel.setMnemonic('C');
    btnCancel.setText("Cancel");
    btnAdd.setMnemonic('A');
    btnAdd.setText("Add");
    btnDel.setMnemonic('D');
    btnDel.setText("Delete");
    editPanel.setLayout(gridBagLayout1);
    jLabel1.setText("Actions:");
    jLabel2.setText("Execution:");
    jLabel3.setText("Tick:");
    txtTick.setToolTipText("");
    chkLast.setForeground(UIManager.getColor("Label.foreground"));
    chkLast.setHorizontalTextPosition(SwingConstants.LEADING);
    topPanel.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(142, 142, 142)), "Edit Action Execution"), BorderFactory.createEmptyBorder(2, 2, 2, 2)));
    tablePanel.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(142, 142, 142)), "Action Executions"), BorderFactory.createEmptyBorder(2, 2, 2, 2)));
    jLabel4.setText("Execute Last:");
    this.add(topPanel, BorderLayout.NORTH);
    topPanel.add(editPanel, BorderLayout.CENTER);
    editPanel.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(cmbAction, new GridBagConstraints(1, 0, 3, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(jLabel2, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    editPanel.add(cmbExecution, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(jLabel3, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(txtTick, new GridBagConstraints(3, 1, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(chkLast, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    editPanel.add(jLabel4, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 4, 2, 2), 0, 0));
    topPanel.add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(btnAdd, null);
    buttonPanel.add(btnDel, null);
    this.add(tablePanel, BorderLayout.CENTER);
    tablePanel.add(sp, BorderLayout.CENTER);
    sp.getViewport().add(schTable, null);
    this.add(bottomPanel, BorderLayout.SOUTH);
    bottomPanel.add(btnOk, null);
    bottomPanel.add(btnCancel, null);

    DefaultComboBoxModel model = (DefaultComboBoxModel) cmbAction.getModel();
    for (int i = 0; i < actions.size(); i++) {
      model.addElement(actions.get(i));
    }

    model = (DefaultComboBoxModel) cmbExecution.getModel();
    model.addElement(new StringIntPair("EVERY TICK", ScheduleDataConstants.EVERY_TICK));
    model.addElement(new StringIntPair("AT A SINGLE TICK", ScheduleDataConstants.AT));
    model.addElement(new StringIntPair("AT INTERVAL", ScheduleDataConstants.AT_INTERVAL));
    model.addElement(new StringIntPair("AT END", ScheduleDataConstants.END));
    model.addElement(new StringIntPair("AT PAUSE", ScheduleDataConstants.PAUSE));

    txtTick.setDocument(new LongDocument());
    txtTick.setText("1");
    chkLast.setEnabled(false);
    jLabel4.setEnabled(false);

  }

  private void addListeners() {
    btnAdd.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addScheduleDetail();
      }
    });

    btnDel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (schTable.getSelectedRow() != -1) {
          ScheduleDetailTableModel model = (ScheduleDetailTableModel) schTable.getModel();

          model.removeRow(schTable.getSelectedRow());
        }
      }
    });


    btnOk.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (dialog != null) dialog.dispose();
        ScheduleDetailTableModel model = (ScheduleDetailTableModel) schTable.getModel();
        editor.setValue(model.getDetails());
        editor.firePropertyChange();
      }
    });

    btnCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (dialog != null) dialog.dispose();
      }
    });

    cmbExecution.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int type = ((StringIntPair) cmbExecution.getSelectedItem()).val;
        if (type == ScheduleDataConstants.AT ||
                type == ScheduleDataConstants.AT_INTERVAL) {
          chkLast.setEnabled(true);
          chkLast.setSelected(false);
          jLabel4.setEnabled(true);
        } else {
          chkLast.setSelected(false);
          chkLast.setEnabled(false);
          jLabel4.setEnabled(false);

        }
      }
    });
  }

  private void addScheduleDetail() {
    long tick = Long.parseLong(txtTick.getText());
    int type = ((StringIntPair) cmbExecution.getSelectedItem()).val;
    ScheduleDetail d = new ScheduleDetail((String) cmbAction.getSelectedItem(),
            tick, type, chkLast.isSelected());
    ((ScheduleDetailTableModel) schTable.getModel()).addRow(d);
  }

  public void display(JFrame f, ScheduleDetailsPropertyEditor editor) {
    this.editor = editor;
    dialog = new JDialog(f, false);
    dialog.setTitle("Schedule Editor");
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        dialog.dispose();
      }
    });

    dialog.pack();
    dialog.setSize(450, 325);
    uchicago.src.guiUtils.GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
  }

  public static void main(String[] args) {
    final JFrame f = new JFrame();
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        //System.out.println(f.getSize());
        System.exit(0);
      }
    });

    ArrayList a = new ArrayList();
    a.add("Write");
    a.add("Record");

    ScheduleDetails d = new ScheduleDetails();
    d.addDetail(new ScheduleDetail("write", 3L, ScheduleDataConstants.EVERY_TICK, false));

    f.getContentPane().setLayout(new BorderLayout());
    f.getContentPane().add(new ScheduleDetailPropertyPanel(d), BorderLayout.CENTER);
    f.pack();
    f.setSize(450, 325);
    f.setVisible(true);
  }
}
