package uchicago.src.simbuilder.beans;

import javax.swing.table.AbstractTableModel;
import java.util.HashMap;

/**
 * TableModel representing ScheduleDetails. Each row in the
 * model represents a ScheduleDetail.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class ScheduleDetailTableModel extends AbstractTableModel implements
        ScheduleDataConstants {

  private static final int NUM_COLS = 4;
  private static final String[] COL_NAMES = {"Action", "Execution Type", "Tick",
                                             "Last"};
  private HashMap typeMap = new HashMap();
  private ScheduleDetails rows;

  public ScheduleDetailTableModel(ScheduleDetails details) {
    rows = details;
    typeMap.put(new Integer(this.EVERY_TICK), "EVERY_TICK");
    typeMap.put(new Integer(END), "END");
    typeMap.put(new Integer(PAUSE), "PAUSE");
    typeMap.put(new Integer(AT), "AT SINGLE TICK");
    typeMap.put(new Integer(AT_INTERVAL), "AT INTERVAL");
  }

  public int getRowCount() {
    return rows.size();
  }

  public int getColumnCount() {
    return NUM_COLS;
  }

  public String getColumnName(int col) {
    return COL_NAMES[col];
  }

  public Class getColumnClass(int col) {
    return getValueAt(0, col).getClass();
  }

  public boolean isCellEditable(int row, int col) {
    return false;
  }

  public Object getValueAt(int row, int col) {
    Object retVal = null;
    if (row < rows.size()) {
      ScheduleDetail d = rows.getDetail(row);
      switch (col) {
        case 0:
          retVal = d.getActionName();
          break;
        case 1:
          retVal = typeMap.get(new Integer(d.getType()));
          break;
        case 2:
          retVal = new Long(d.getTick());
          break;
        case 3:
          retVal = new Boolean(d.getLast());
          break;
        default:
          retVal = null;
      }
    }

    return retVal;
  }

  public void addRow(ScheduleDetail detail) {
    rows.addDetail(detail);
    this.fireTableRowsInserted(rows.size() - 1, rows.size());
  }

  public void removeRow(int row) {
    rows.removeDetail(row);
    this.fireTableRowsDeleted(row, row);
  }

  public ScheduleDetails getDetails() {
    return rows;
  }
}
