package uchicago.src.simbuilder.beans;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * A collection of ScheduleDetail objects. This also includes a list
 * of action names for possible scheduling.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:26 $
 */
public class ScheduleDetails implements ScheduleDataConstants {

  private ArrayList details = new ArrayList();
  private ArrayList deleted = new ArrayList();
  private ArrayList actionNames = new ArrayList();

  /**
   * Sets the names of the actions for possible scheduling. That is,
   * the names of the actions that may be the subject of a ScheduleDetail.
   */
  public void setActionNames(ArrayList actionNames) {
    this.actionNames = actionNames;
  }

  /**
   * Gets the names of the actions for possible scheduling. That is,
   * the names of the actions that may be the subject of a ScheduleDetail.
   */
  public ArrayList getActionNames() {
    return actionNames;
  }

  /**
   * Returns a copy of this ScheduleDetails object.
   */
  public ScheduleDetails copy() {
    ScheduleDetails d = new ScheduleDetails();
    for (int i = 0; i < details.size(); i++) {
      d.addDetail(((ScheduleDetail) details.get(i)).copy());
    }

    for (int i = 0; i < deleted.size(); i++) {
      d.deleted.add(deleted.get(i));
    }

    ArrayList a = new ArrayList(actionNames.size());
    for (int i = 0; i < actionNames.size(); i++) {
      a.add(actionNames.get(i));
    }

    d.setActionNames(a);

    return d;
  }

  /**
   * Adds a ScheduleDetail.
   */
  public void addDetail(ScheduleDetail detail) {
    details.add(detail);
  }

  /**
   * Removes a ScheduleDetail.
   */
  public void removeDetail(ScheduleDetail detail) {
    details.remove(detail);
    detail.delete();
    deleted.add(detail);
  }

  /**
   * Removes a ScheduleDetail by index.
   */
  public void removeDetail(int index) {
    ScheduleDetail d = (ScheduleDetail) details.remove(index);
    d.delete();
    deleted.add(d);
  }

  /**
   * Gets a ScheduleDetail by index.
   */
  public ScheduleDetail getDetail(int index) {
    return (ScheduleDetail) details.get(index);
  }

  /**
   * Removes all the ScheduleDetail-s whose actions have the
   * specified name.
   */
  public void removeDetailWithName(String actionName) {
    for (Iterator iter = details.iterator(); iter.hasNext();) {
      ScheduleDetail detail = (ScheduleDetail) iter.next();
      if (detail.getActionName().equals(actionName)) {
        iter.remove();
        detail.delete();
        deleted.add(detail);
        actionNames.remove(actionName);
      }
    }
  }

  /**
   * Returns the list of ScheduleDetail-s that have been deleted.
   */
  public ArrayList getDeletedDetails() {
    return deleted;
  }

  /**
   * Clears the list of ScheduleDetail-s that have been deleted.
   */
  public void clearDeletedDetails() {
    deleted.clear();
  }

  /**
   * Returns the number of ScheduleDetail objects contained by this
   * ScheduleDetails.
   */
  public int size() {
    return details.size();
  }

  /**
   * Assigns and integer id to all the ScheduleDetail-s contained in this
   * ScheduleDetails. The first detail with have an id equal to start.
   * This returns the next available, free id number.
   */
  public int setID(int start) {
    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail detail = (ScheduleDetail) details.get(i);
      detail.setID(start++);
    }

    return start;
  }

  public String toXML() {

    StringBuffer b = new StringBuffer();

    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail detail = (ScheduleDetail) details.get(i);
      b.append(detail.toXML());
      b.append("\n");
    }

    return b.toString();
  }
}


