package uchicago.src.simbuilder.beans;

import ViolinStrings.Strings;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.AbstractCompilerException;
import uchicago.src.simbuilder.io.BeanDataLoader;
import uchicago.src.simbuilder.util.BooleanMessage;
import uchicago.src.simbuilder.util.RepastPyUtilities;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyEditorManager;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * @version $Revision: 1.4 $ $Date: 2004/10/14 18:23:27 $
 */

public class SequenceGraph extends ChartComponent implements ActionCodeProducer {

  private String buildStmts, displayStmts, setupStmts;
  private String xmlOpen, xmlClose;

  private static final String UPDATE = "update";
  private static final String WRITE = "write to file";

  private Sequences sequences = new Sequences();

  static {
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.Sequences.class,
            uchicago.src.simbuilder.beans.SequencesEditor.class);
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.FileProperty.class,
            uchicago.src.simbuilder.beans.FilePropertyEditor.class);
  }

  public SequenceGraph() {
    name = "_sequenceGraph";
    title = "Sequence Graph";
    sequences.addVarName("model");
    ArrayList a = new ArrayList();
    a.add(UPDATE);
    a.add(WRITE);
    details.setActionNames(a);


    try {
      BeanDataLoader loader = new BeanDataLoader(getClass());
      buildStmts = loader.getBuild();
      displayStmts = loader.getDisplay();
      setupStmts = loader.getSetup();
      xmlOpen = loader.getXMLOpen();
      xmlClose = loader.getXMLClose();

      /*
      System.out.println("buildStmts = " + buildStmts);
      System.out.println("displayStmts = " + displayStmts);
      System.out.println("setupStmts = " + setupStmts);
      */
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
  
  /**
   * Performs a precompile check making sure that the component is ready to compile.
   *
   * @return
   */
  public BooleanMessage sanityCheck() {
    if (!RepastPyUtilities.isValidJavaIdentifier(name)) {
      return new BooleanMessage("Invalid name '" + name + "'" +
              ". Valid names contain only letters, numbers, or '_'",
                false);
    }
    
    return BooleanMessage.TRUE;
  }

  public Sequences getSequences() {
    if (model != null) {
      sequences.clearVarNames();
      Actions actions = model.getActions();
      java.util.List varNames = actions.getVarNames();
      for (int i = 0, n = varNames.size(); i < n; i++) {
        sequences.addVarName((String)varNames.get(i));
      }
    }
    return sequences;
  }

  public void setSequences(Sequences sequences) {
    this.sequences = sequences;
    for (Iterator iter = this.sequences.iterator(); iter.hasNext();) {
      SequenceActionCode sac = (SequenceActionCode) iter.next();
      sac.setDefiningProducer(this);
    }
  }

  public String setupStmts() {
    String setup = setupStmts;
    setup = Strings.change(setup, "__name__", name);
    setup = Strings.change(setup, "__title__", title);
    if (fileProperty.doWrite()) {
      StringBuffer b = new StringBuffer(", \"");
      b.append(fileProperty.getFileName());
      b.append("\", Statistics.CSV");
      setup = Strings.change(setup, "__file__", b.toString());
    } else {
      setup = Strings.change(setup, "__file__", "");
    }
    return setup;
  }

  public String displayStmts() {
    String display = displayStmts;
    display = Strings.change(display, "__xtitle__", xAxisTitle);
    display = Strings.change(display, "__ytitle__", yAxisTitle);
    display = Strings.change(display, "__name__", name);
    display = Strings.change(display, "__xAxis__", String.valueOf(xAxis));
    display = Strings.change(display, "__yAxis__", String.valueOf(yAxis));
    return display;

  }

  public Hashtable fields() {
    Hashtable h = new Hashtable();
    h.put(name, "uchicago.src.sim.analysis.OpenSequenceGraph");
    return h;
  }

  public ArrayList methods() {
    ArrayList list = new ArrayList();
    for (Iterator iter = sequences.iterator(); iter.hasNext();) {
      SequenceActionCode code = (SequenceActionCode) iter.next();
      SequenceActionCode copy = (SequenceActionCode) code.copy();
      copy.setCode(Strings.change(copy.getCode(), "self.model", "self"));
      list.add(copy);
    }

    return list;
  }

  public void compile(String dir, ByteCodeGenerator generator) {
  }

  public ScheduleDetails schedule() {
    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail d = details.getDetail(i);
      d.setTarget("Sequence Graph: " + name);
      StringBuffer b = new StringBuffer("self.");
      b.append(name);
      if (d.getActionName().equals(UPDATE)) {
        b.append(".step()\n");
      } else {
        b.append(".writeToFile()\n");
      }

      d.setNQPy(b.toString());
    }

    return details;
  }

  public void saveXML(BufferedWriter out) throws IOException {
    String xml = xmlOpen;
    xml = Strings.change(xml, "__class__", getClass().getName());
    xml = Strings.change(xml, "__title__", title);
    xml = Strings.change(xml, "__xtitle__", xAxisTitle);
    xml = Strings.change(xml, "__ytitle__", yAxisTitle);
    xml = Strings.change(xml, "__name__", name);
    xml = Strings.change(xml, "__xAxis__", String.valueOf(xAxis));
    xml = Strings.change(xml, "__yAxis__", String.valueOf(yAxis));

    StringBuffer b = new StringBuffer(xml);
    b.append("\n");
    b.append(fileProperty.toXML());
    b.append("\n");
    for (Iterator iter = sequences.iterator(); iter.hasNext();) {
      SequenceActionCode sequence = (SequenceActionCode) iter.next();
      b.append(sequence.toXML());
    }

    getSchedule();
    b.append(details.toXML());
    b.append("\n");
    b.append(xmlClose);

    System.out.println("b = " + b.toString());
    out.write(b.toString());
    out.newLine();
  }

  public void loadFromXML(Element elem) {
    setName(elem.getAttribute("name"));
    setTitle(elem.getAttribute("title"));
    setInitialYAxisSize(XMLLoader.getDoubleVal(elem, "yAxisStart"));
    setInitialXAxisSize(XMLLoader.getDoubleVal(elem, "xAxisStart"));
    setXAxisTitle(elem.getAttribute("xAxisTitle"));
    setYAxisTitle(elem.getAttribute("yAxisTitle"));
    loadSequencesFromXML(elem);
    setSchedule(XMLLoader.getScheduleDetails(elem));

    Element prop = (Element) XMLLoader.getChildElementsByName(elem,
            EvolverConstants.XMLNS +
            ":file_property").get(0);
    fileProperty.setDoWrite(XMLLoader.getBooleanVal(prop, "doWrite"));
    String txt = prop.getAttribute("fileName");
    if (txt == null) txt = "";
    fileProperty.setFileName(txt);
    setFileName(fileProperty);

    ArrayList a = new ArrayList();
    a.add(UPDATE);
    a.add(WRITE);
    details.setActionNames(a);
  }

  private void loadSequencesFromXML(Element elem) {
    Sequences seqs = new Sequences();
    ArrayList list = XMLLoader.getChildElementsByName(elem, EvolverConstants.XMLNS +
            ":" + "seqAction");
    for (int i = 0; i < list.size(); i++) {
      Element action = (Element) list.get(i);
      SequenceActionCode code = new SequenceActionCode(action.getAttribute("name"));
      NodeList children = action.getChildNodes();
      String rscode = children.item(1).getNodeValue();
      code.setCode(rscode.trim());
      seqs.addAction(code);

      ArrayList l = new ArrayList();
      StringTokenizer t = new StringTokenizer(action.getAttribute("imports"),
              " ");
      while (t.hasMoreTokens()) {
        l.add(t.nextToken());
      }
      code.setImports(l);

      l = new ArrayList();
      t = new StringTokenizer(action.getAttribute("paramNames"), " ");
      while (t.hasMoreTokens()) {
        l.add(t.nextToken());
      }
      code.setParameterNames(l);
      code.setRetType(action.getAttribute("retVal"));

      String colors = action.getAttribute("color");
      if (colors.trim().length() > 0) {
        colors = colors.trim();
        StringTokenizer tok = new StringTokenizer(colors, " ");
        String red = tok.nextToken();
        String green = tok.nextToken();
        String blue = tok.nextToken();
        code.setColor(new Color(Integer.parseInt(red),
                Integer.parseInt(green),
                Integer.parseInt(blue)
                ));
      }

      code.setStyle(Integer.parseInt(action.getAttribute("pointType")));
    }

    setSequences(seqs);
  }

  public String buildStmts() {
    String build = buildStmts;
    build = Strings.change(build, "__name__", name);
    build = Strings.change(build, "__title__", title);
    if (fileProperty.doWrite()) {
      StringBuffer b = new StringBuffer(", \"");
      b.append(fileProperty.getFileName());
      b.append("\", Statistics.CSV");
      build = Strings.change(build, "__file__", b.toString());
    } else {
      build = Strings.change(build, "__file__", "");
    }

    StringBuffer b = new StringBuffer("\n");
    b.append(build);
    // add the sequences to the graph
    for (Iterator iter = sequences.iterator(); iter.hasNext();) {
      SequenceActionCode sequence = (SequenceActionCode) iter.next();
      b.append("self.");
      b.append(name);
      b.append(".");
      b.append(sequence.getBuildStatement());
    }
    return b.toString();
  }

  public void displayEditor(JFrame frame, AbstractCompilerException ex) {
    SequencesEditor editor = new SequencesEditor();
    editor.setValue(getSequences());
    editor.addPropertyChangeListener(this);
    SequencePropertyPanel panel = new SequencePropertyPanel(sequences);
    panel.addError(ex);
    panel.setVarNames(sequences.getVarNames());
    panel.display(frame, editor);

    setSequences((Sequences) editor.getValue());
  }
}
