package uchicago.src.simbuilder.beans;

import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.EvolverModelCompiler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * A Container for SequenceActionCode objects. While this is primarily a wrapper
 * around an ArrayList, it has to be its own class in order to register
 * the appropriate property editor for editing the contained SequenceActionCode.
 *
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:27 $
 */

public class Sequences {

  private ArrayList sequences = new ArrayList();
  private ArrayList varNames = new ArrayList();

  public Sequences() {
  }

  public Sequences(Sequences a) {
    for (int i = 0; i < a.sequences.size(); i++) {
      sequences.add(((SequenceActionCode) a.sequences.get(i)).copy());
    }
    varNames.addAll(a.varNames);
  }

  public void addAction(SequenceActionCode code) {
    sequences.add(code);
  }

  public Object remove(int index) {
    return sequences.remove(index);
  }


  public Object remove(String name) {
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode c = (SequenceActionCode) sequences.get(i);
      if (c.getName().equals(name)) {
        sequences.remove(i);
        return c;
      }
    }

    return null;
  }

  public void addParameterName(String actionName, String parameterName) {
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode c = (SequenceActionCode) sequences.get(i);
      if (c.getName().equals(actionName)) {
        c.addParameterName(parameterName);
        break;
      }
    }
  }

  public void removeEvCode(String actionName, String tag, int lines) {
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode c = (SequenceActionCode) sequences.get(i);
      if (c.getName().equals(actionName)) {
        c.removeEvCode(tag, lines);
        break;
      }
    }
  }

  public int size() {
    return sequences.size();
  }

  public Iterator iterator() {
    return sequences.iterator();
  }

  public void clearVarNames() {
    varNames.clear();
  }

  public void addVarName(String name) {
    varNames.add(name);
  }

  public ArrayList getVarNames() {
    Collections.sort(varNames);
    return varNames;
  }

  public void clear() {
    sequences.clear();
  }

  /**
   * Replaces the agent name and model name placeholders with the
   * actual model class name and agent class name.
   */
  public void setDynTypeNames(String modelName, String agentName) {
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode code = (SequenceActionCode) sequences.get(i);
      if (modelName != null) code.setModelName(modelName);
      if (agentName != null) {
        code.setAgentName(agentName);
        int index = agentName.lastIndexOf('.');
        if (index != -1) code.setUnqualifedAgentName(agentName.substring(index + 1, agentName.length()));
      }
    }
  }

  public boolean hasAction(String name) {
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode c = (SequenceActionCode) sequences.get(i);
      if (c.getName().equals(name)) return true;
    }

    return false;
  }

  public void addToAction(String name, ArrayList stmts) {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < stmts.size(); i++) {
      b.append(stmts.get(i));
      b.append("\n");
    }

    addToAction(name, b.toString());
  }

  public void prependToAction(String name, String stmts) {
    SequenceActionCode c = null;
    for (int i = 0; i < sequences.size(); i++) {
      if (((SequenceActionCode) sequences.get(i)).getName().equals(name)) {
        c = (SequenceActionCode) sequences.get(i);
        break;
      }
    }

    if (c == null) {
      c = new SequenceActionCode(name);
      sequences.add(c);
    }


    StringBuffer b = new StringBuffer(stmts);
    b.append(c.getCode());
    c.setCode(b.toString());
  }

  public void addToAction(String name, String stmts) {
    SequenceActionCode c = null;
    for (int i = 0; i < sequences.size(); i++) {
      if (((SequenceActionCode) sequences.get(i)).getName().equals(name)) {
        c = (SequenceActionCode) sequences.get(i);
        break;
      }
    }

    if (c == null) {
      c = new SequenceActionCode(name);
      sequences.add(c);
    }

    /*
    if (c == null) {
      System.out.println("should remove: " + name);

      c = new SequenceActionCode(name);
      actions.add(c);
      actionsToRemove.add(c);
    } else {
      SequenceActionCode orig = c.copy();
      actionsToAdd.add(orig);
      actionsToRemove.add(c);
    }
    */

    StringBuffer b = new StringBuffer(c.getCode());
    b.append(stmts);

    //System.out.println(c.getName());
    //System.out.println(b.toString());

    c.setCode(b.toString());
  }

  public void createDefaultAction(String name, String stmts) {
    SequenceActionCode c = new SequenceActionCode(name);
    c.setCode(stmts);
    sequences.add(c);
  }

  /**
   * Does post compile clean up.
   */
  public void postCompile() {
    varNames.clear();
    ArrayList remove = new ArrayList();
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode c = (SequenceActionCode) sequences.get(i);
      if (c.getName().startsWith("_")) {
        remove.add(c);
      }
    }

    sequences.removeAll(remove);

    /*
    System.out.println("in: ");
    for (int i = 0; i < actions.size(); i++) {
      SequenceActionCode c = (SequenceActionCode)actions.get(i);
      if (c.getName().startsWith("_")) {
	System.out.println("\t" + c.getName());
      }
    }
    */

    /*
    actions.removeAll(actionsToRemove);
    actions.addAll(actionsToAdd);

    actionsToAdd.clear();
    actionsToRemove.clear();

    System.out.println("actions size post clean: " + actions.size());
    */


  }

  /**
   * Build a dynamic type from this Actions collection.
   */
  public NQPyDynamicClass buildType(NQPyDynamicClass dt) {
    Class cls[] = new Class[0];
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode code = (SequenceActionCode) sequences.get(i);
      dt.addMethod(code.getName(), void.class, cls);
    }

    return dt;
  }

  /**
   * Compile the action code into bytecode methods. selfName is used
   * as the object type for the self variable.
   *
   * @param selfName the name of the object whose methods these are.
   */
  public void compile(ByteCodeGenerator generator, String selfName) {
    Hashtable h = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) h.get(selfName);


    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode code = (SequenceActionCode) sequences.get(i);

      //System.out.println (code.getName ());
      //System.out.println (code.getCode ());


      NQPyBlockCompiler compiler = new NQPyBlockCompiler(code.getName(),
              generator.getClassFile());
      compiler.setSelf(dt);
      String blockName = selfName + "." + code.getName();
      compiler.setBlockName(blockName);
      EvolverModelCompiler.putInBeanTable(blockName, code.getDefiningProducer());
      ArrayList imports = code.getImports();
      for (int j = 0; j < imports.size(); j++) {
        //System.out.println("ADDING IMPORT " + (String)imports.get(j));
        compiler.addImport((String) imports.get(j));
      }

      compiler.addImport(dt.getPackage() + ".*");
      compiler.compile(code.getCode());
    }

    //postCompile();
  }

  public String toXML() {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < sequences.size(); i++) {
      SequenceActionCode code = (SequenceActionCode) sequences.get(i);
      b.append("  ");
      b.append(code.toXML());
      b.append("\n");
    }
    return b.toString();
  }
}
