package uchicago.src.simbuilder.beans.gis;

import anl.repast.gis.data.GeotoolsData;
import anl.repast.gis.data.FieldNameAndType;
import anl.repast.gis.data.OpenMapData;

import java.io.IOException;

/**
 * Encapsulates GISPackage information. 
 * 
 * @author Nick Collier
 * @version $Revision: 1.5 $ $Date: 2004/11/08 15:21:41 $
 */
public abstract class GISPackage {
  
  private String name;

  public GISPackage(String name) {
    this.name = name;
  }
  
  public String toString() {
    return name;
  }
  
  public abstract Class getAbstractBaseClassForProducer();
  
  public static GISPackage createPackage(String name) {
    if (name.equals("Open Map")) return GISPackage.OPEN_MAP;
    else if (name.equals("ArcGIS")) return GISPackage.ARCGIS;
    
    throw new IllegalArgumentException("Illegal GIS Package Name");
  }
  
  public abstract String getDisplaySetupStatements(String title); 
  public abstract FieldNameAndType[] interrogate(String shapefile)  throws IOException;
  public abstract String getDataHandlerString();
  
  public static final GISPackage OPEN_MAP = new GISPackage("Open Map") {
    public String getDisplaySetupStatements(String title) {
      // GISSimModelImpl is the "self" here. 
      return "self.updater = self.omDisplayUpdater\n" +
              "self.omDisplay = OpenMapDisplay(\"" + title + "\")\n" +
              "self.registerMediaProducer(\"" + title + "\", self.omDisplay)\n" +
              "self.omDisplay.setModel(self)";
    }

    public FieldNameAndType[] interrogate(String shapefile) throws IOException {
      return OpenMapData.getInstance().interrogate(shapefile);
    }

    public String getDataHandlerString() {
      return "OpenMapData.getInstance()";
    }

    public Class getAbstractBaseClassForProducer() {
      return uchicago.src.simbuilder.base.gis.AbstractOpenMapGISAgent.class;
    }
  };
  
  public static final GISPackage ARCGIS = new GISPackage("ArcGIS") {
    
    public String getDisplaySetupStatements(String title) {
      // GISSimModelImpl is the "self" here. 
      return "self.updater = self.esriDisplayUpdater";
    }
    
    public FieldNameAndType[] interrogate(String shapefile) throws IOException {
      return GeotoolsData.getInstance().interrogate(shapefile);
    }
    
    public String getDataHandlerString() {
      return "GeotoolsData.getInstance()";
    }
    
    public Class getAbstractBaseClassForProducer() {
      return uchicago.src.simbuilder.base.gis.AbstractGeotoolsGISAgent.class;
    }
    
  };
}
