package uchicago.src.simbuilder.beans.gis;


import org.w3c.dom.Element;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.base.gis.AbstractOpenMapGISAgent;
import uchicago.src.simbuilder.beans.*;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;


public class GenericAgentProducer extends AbstractAgentProducer {

  public GenericAgentProducer() {
    super();
    name = "GenericAgent";
    groupName = "genericAgentGroup";
  }


  public Actions getActions() {
    return super.getActions();
  }

  /**
   * Returns the nqpy that becomes part of the model's build (not the produced agent's)
   * method.
   *
   * @return
   */
  public String getNQPyInit() {
    return super.getNQPyInit();
  }

  /**
   * Returns true if this AgentProducer is the parent class of another AgentProducer.
   *
   * @return
   */
  public boolean isParent() {
    return super.isParent();    //To change body of overridden methods use File | Settings | File Templates.
  }

  /**
   * Builds a DynamicType that represents the class compiled from this
   * NetworkNodeProducer.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator) {

    for (int i = 0; i < compileables.size(); i++) {
      EvolverAgentCompileable comp =
              (EvolverAgentCompileable) compileables.get(i);
      h = comp.buildType(h, generator);
    }

    String fqn = pkg + "." + name;
    NQPyDynamicClass dynClass;
    if (getPossibleSuperClasses().getSelected().equals(NO_PARENT)) {
      dynClass = new NQPyDynamicClass(fqn, AbstractOpenMapGISAgent.class);
    } else {
      AbstractAgentProducer producer = (AbstractAgentProducer) getPossibleSuperClasses().getSelected();
      // this pkg part here is OK because for now everything in same package
      dynClass = new NQPyDynamicClass(fqn, pkg + "." + producer.className());
    }

    // add a no-op constructor
    dynClass.addMethod(name, fqn, new String[]{});
    dynClass = parameters.buildType(dynClass);
    implementAsCustomProbeable(dynClass);
    dynClass = actions.buildType(dynClass);
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    dynClass.addField("model", modelName);
    dynClass.addMethod("setModel", "void", new String[]{modelName});
    h.put(dynClass.getJClassName(), dynClass);
    generator.putDetail(EvolverBean.AGENT_NAME, fqn);

    return h;
  }

  /**
   * Compile the class described by this GridAgentProducer into
   * bytecode.
   */
  public void compile(String dir, ByteCodeGenerator generator) {
    for (int i = 0; i < compileables.size(); i++) {
      EvolverBean bean = (EvolverBean) compileables.get(i);
      bean.compile(dir, generator);
    }

    String fqn = pkg + "." + name;
    ArrayList list = new ArrayList();
    list.add("model");
    String[] params = new String[1];
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    params[0] = modelName;

    if (getPossibleSuperClasses().getSelected().equals(NO_PARENT)) {
      generator.createClass(fqn, AbstractOpenMapGISAgent.class);
    } else {
      AbstractAgentProducer producer = (AbstractAgentProducer) getPossibleSuperClasses().getSelected();
      // this pkg part here is OK because for now everything in same package
      generator.createClass(fqn, pkg + "." + producer.className());
    }
    
    generator.createFieldsConstructor(list, params);
    compileAsCustomProbeable(generator, new String[]{});

    parameters.compile(generator, null, isParent());
    actions.compile(generator, fqn);
    generator.finishCtor();
    makeModelSet(generator);
    generator.finishClass();
    generator.writeToFile(fqn, dir);
  }

  /**
   * Saves an xml representation of this GenericAgentProducer to the specified
   * output stream.
   *
   * @param out the BufferedWriter to write the xml to.
   */
  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"Compileable\" class=\"");
    b.append(getClass());
    b.append("\" name=\"");
    b.append(name);
    b.append(getParentXML());
    b.append("\" groupName=\"");
    b.append(groupName);
    b.append("\">");
    out.write(b.toString());
    out.newLine();
    super.saveXML(out);
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
    out.newLine();
  }
}
