package uchicago.src.simbuilder.beans.gis;

import layout.TableLayout;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import uchicago.src.guiUtils.GuiUtilities;

/**
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:27 $
 */
public class LineEditor extends JPanel {

  private JButton colorBtn;
  private JSpinner widthFld = new JSpinner();
  private JButton okBtn = new JButton("OK");
  private JButton cancelBtn = new JButton("Cancel");
  private Color color;
  private LineSymbolIcon previewIcon;
  private JLabel previewLabel;
  private JDialog dialog;
  private boolean isCanceled = false;

  public LineEditor(Color color, int width) {
    super(new BorderLayout());
    this.color = color;
    colorBtn = new JButton(new PolygonSymbolIcon(color));
    if (width < 1) width = 1;
    widthFld.setValue(new Integer(width));

    double[][] sizes = {{TableLayout.PREFERRED, 4, TableLayout.FILL},
                        {TableLayout.PREFERRED, 4, TableLayout.PREFERRED}};


    JPanel previewPane = new JPanel(new GridLayout(1, 1));
    previewPane.setBorder(BorderFactory.createTitledBorder("Preview"));
    previewIcon = new LineSymbolIcon(this.color, width, 30, 20);
    previewLabel = new JLabel(previewIcon, JLabel.CENTER);
    previewLabel.setPreferredSize(new Dimension(70, 100));
    previewPane.add(previewLabel);


    JPanel center = new JPanel();
    center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));
    center.add(Box.createHorizontalStrut(6));
    center.add(previewPane);
    center.add(Box.createHorizontalStrut(4));

    JPanel options = new JPanel(new TableLayout(sizes));
    options.setBorder(BorderFactory.createTitledBorder("Options"));
    options.add(new JLabel("Color:"), "0, 0");
    options.add(new JLabel("Width:"), "0, 2");
    options.add(colorBtn, "2, 0");
    options.add(widthFld, "2, 2");
    center.add(options);
    center.add(Box.createHorizontalStrut(6));
    this.add(center, BorderLayout.CENTER);

    JPanel bottom = new JPanel();
    bottom.add(okBtn);
    bottom.add(cancelBtn);
    this.add(bottom, BorderLayout.SOUTH);
    addListeners();
  }
  
  public Integer getThickness() {
    if (isCanceled) return null;
    return (Integer) widthFld.getValue();
  }
  
  public Color getColor() {
    if (isCanceled) return null;
    return color;
  }

  private void addListeners() {
    colorBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        Color newColor = JColorChooser.showDialog(LineEditor.this, "Select Color", color);
        // return null on cancel
        if (newColor != null) {
          previewIcon.setColor(newColor);
          updatePreview();
          color = newColor;
          colorBtn.setIcon(new PolygonSymbolIcon(color));
        }
      }
    });
    
    widthFld.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        previewIcon.setThickness(((Integer)widthFld.getValue()).intValue());
        updatePreview();
      }
    });
    
    okBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        isCanceled = false;
        dialog.dispose();
      }
    });
    
    cancelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        isCanceled = true;
        dialog.dispose();
      }
    });
  }
  
  public void display(JDialog dialog) {
    this.dialog = dialog;
    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.setSize(200, 140);
    GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
  }

  private void updatePreview() {
    previewLabel.invalidate();
    previewLabel.validate();
    previewLabel.repaint();
  }

  public static void main(String[] args) {
    JFrame f = new JFrame();
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.getContentPane().setLayout(new BorderLayout());
    f.getContentPane().add(new LineEditor(Color.RED, 1), BorderLayout.CENTER);
    f.setSize(200, 140);
    f.setVisible(true);
  }
}
