package uchicago.src.simbuilder.beans.gis;


import org.w3c.dom.Element;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.base.gis.AbstractOpenMapGISAgent;
import uchicago.src.simbuilder.beans.AbstractAgentProducer;
import uchicago.src.simbuilder.beans.EvolverAgentCompileable;
import uchicago.src.simbuilder.beans.EvolverBean;
import uchicago.src.simbuilder.beans.EvolverConstants;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;


public class RasterAgentProducer extends AbstractAgentProducer {

  public RasterAgentProducer() {
    super();
    name = "RasterAgent";
    groupName = "rasterAgentGroup";
  }

  public String getGroupName() {
    return groupName;
  }

  /**
   * Builds a DynamicType that represents the class compiled from this
   * NetworkNodeProducer.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator) {

    for (int i = 0; i < compileables.size(); i++) {
      EvolverAgentCompileable comp =
              (EvolverAgentCompileable) compileables.get(i);
      h = comp.buildType(h, generator);
    }

    String fqn = pkg + "." + name;
    NQPyDynamicClass me = new NQPyDynamicClass(fqn, AbstractOpenMapGISAgent.class);

    // add a no-op constructor
    me.addMethod(name, fqn, new String[]{});
    me = parameters.buildType(me);
    implementAsCustomProbeable(me);
    me = actions.buildType(me);
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    me.addField("model", modelName);
    me.addMethod("setModel", "void", new String[]{modelName});
    h.put(me.getJClassName(), me);
    generator.putDetail(EvolverBean.AGENT_NAME, fqn);

    return h;
  }

  /**
   * Compile the class described by this GridAgentProducer into
   * bytecode.
   */
  public void compile(String dir, ByteCodeGenerator generator) {
    for (int i = 0; i < compileables.size(); i++) {
      EvolverBean bean = (EvolverBean) compileables.get(i);
      bean.compile(dir, generator);
    }

    String fqn = pkg + "." + name;
    ArrayList list = new ArrayList();
    list.add("model");
    String[] params = new String[1];
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    params[0] = modelName;

    generator.createClass(fqn, AbstractOpenMapGISAgent.class);
    generator.createFieldsConstructor(list, params);
    compileAsCustomProbeable(generator, new String[]{});

    parameters.compile(generator, null);
    actions.compile(generator, fqn);
    generator.finishCtor();
    makeModelSet(generator);
    generator.finishClass();
    generator.writeToFile(fqn, dir);
  }

  /**
   * Saves an xml representation of this NetworkNodeProducer to the specified
   * output stream.
   *
   * @param out the BufferedWriter to write the xml to.
   */
  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"Compileable\" class=\"");
    b.append(getClass());
    b.append("\" name=\"");
    b.append(name);
    b.append("\">");
    out.write(b.toString());
    out.newLine();
    super.saveXML(out);
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
    out.newLine();
  }

  public void loadFromXML(Element nodeE) {
    // loads name, actions, parameters and schedule
    super.loadFromXML(nodeE);
  }
}
