package uchicago.src.simbuilder.beans.gis;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Nick Collier
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:27 $
 */
public class SymbolTableModel extends AbstractTableModel {

  private static final String[] NAMES = {"Symbol", "Value"};
  private static final Class[] CLASSES = {Icon.class, String.class};


  private List symbolList = new ArrayList();
  private VectorDataSource dataSource;
  private boolean isPoly = true;

  public SymbolTableModel(VectorDataSource dataSource) {
    this.dataSource = dataSource;
    isPoly = this.dataSource.isPolygon();
    List symbols = dataSource.getDisplaySymbolData();
    if (symbols == null || symbols.size() == 0) {
      symbolList.add(new SymbolValuePair(getDefaultIcon(), "default value", true));
    } else {
      isPoly = this.dataSource.isPolygon();
      symbolList = new ArrayList(symbols);
      Collections.sort(symbolList);
    }
  }

  private SymbolIcon getDefaultIcon() {
    if (isPoly)
      return new PolygonSymbolIcon(Color.RED);
    else
      return new LineSymbolIcon(Color.RED, 1);
  }

  public String getColumnName(int column) {
    return NAMES[column];
  }

  public Class getColumnClass(int columnIndex) {
    return CLASSES[columnIndex];
  }

  public int getColumnCount() {
    return 2;
  }

  public int getRowCount() {
    return symbolList.size();
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    SymbolValuePair pair = (SymbolValuePair) symbolList.get(rowIndex);
    if (columnIndex == 0) {
      return pair.getIcon();
    }

    return pair.getValue();
  }

  public List getCurrentSymbolList() {
    return new ArrayList(symbolList);
  }

  public SymbolValuePair getSymbolValuePair(int row) {
    SymbolValuePair pair = (SymbolValuePair) symbolList.get(row);
    return pair;
  }

  public void addValues(List newValues) {
    int start = symbolList.size();
    for (int i = 0, n = newValues.size(); i < n; i++) {
      String value = newValues.get(i).toString();
      symbolList.add(new SymbolValuePair(getDefaultIcon(), value));
    }

    fireTableRowsInserted(start, symbolList.size() - 1);
  }

  public void clear() {
    this.symbolList.clear();
    symbolList.add(new SymbolValuePair(getDefaultIcon(), "default value", true));
    fireTableDataChanged();
  }

  public void setSymbolList(List symbolList) {
    this.symbolList = new ArrayList(symbolList);
    Collections.sort(symbolList);
    fireTableDataChanged();
  }

  public void removeRows(int[] selectedRows) {
    List objsToRemove = new ArrayList();
    for (int i = 0; i < selectedRows.length; i++) {
      if (selectedRows[i] != 0) {
        objsToRemove.add(symbolList.get(selectedRows[i]));
      }
    }

    symbolList.removeAll(objsToRemove);
    fireTableDataChanged();
  }

  public void reset() {
    isPoly = dataSource.isPolygon();
    clear();
    List symbols = dataSource.getDisplaySymbolData();
    if (symbols != null && symbols.size() > 0) {
      symbolList = new ArrayList(symbols);
      Collections.sort(symbolList);
      fireTableDataChanged();
    }
  }

  public List getFieldNames() {
    List names = new ArrayList();
    for (int i = 0, n = symbolList.size(); i < n; i++) {
      SymbolValuePair pair = (SymbolValuePair) symbolList.get(i);
      names.add(pair.getValue());
    }

    return names;
  }
}
