package uchicago.src.simbuilder.beans.gis;

import anl.repast.gis.data.OpenMapData;
import layout.TableLayout;
import uchicago.src.guiUtils.GuiUtilities;
import uchicago.src.simbuilder.property.DoubleDocument;

import javax.swing.*;
import javax.swing.text.Document;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.*;

/**
 * @author Nick Collier
 * @version $Revision: 1.3 $ $Date: 2004/10/24 22:45:42 $
 */
public class SymbologyDialog extends JPanel {

  private JTable symbolTable;
  private JComboBox fieldsCmb;
  private JButton addValueBtn = new JButton("Add Value");
  private JButton removeValueBtn = new JButton("Remove Value");
  private JButton okBtn = new JButton("OK");
  private JButton cancelBtn = new JButton("Cancel");
  private VectorDataSource dataSource;
  // key is name, value is list of symbology pairs
  private Map nameSymbologyMap = new HashMap();
  private String curFieldName = "";
  private JDialog dialog;


  public SymbologyDialog(VectorDataSource dataSource) {
    super(new BorderLayout());
    this.dataSource = dataSource;
    symbolTable = new JTable(new SymbolTableModel(dataSource));
    fieldsCmb = new JComboBox(new Vector(dataSource.getSymbolFields()));
    if (dataSource.getSymbolFields().size() > 0) {
      String displayFieldName = dataSource.getDisplayFieldName();
      if (displayFieldName == null || displayFieldName.length() == 0) {
        fieldsCmb.setSelectedIndex(0);
      } else {
        fieldsCmb.setSelectedItem(displayFieldName);
      }
      curFieldName = (String) fieldsCmb.getSelectedItem();
    } else {
      fieldsCmb.setEnabled(false);
      addValueBtn.setEnabled(false);
      removeValueBtn.setEnabled(false);
      symbolTable.setEnabled(false);
    }

    JPanel main = new JPanel(new BorderLayout());
    this.add(main, BorderLayout.CENTER);

    double[][] sizes = {{6, TableLayout.FILL, 6},
                        {6, TableLayout.PREFERRED, 4, TableLayout.FILL, 6}};
    JPanel center = new JPanel(new TableLayout(sizes));

    sizes = new double[][]{{4, TableLayout.FILL, 4},
                           {0, TableLayout.PREFERRED, 4}};

    JPanel fieldsPanel = new JPanel(new TableLayout(sizes));
    fieldsPanel.add(fieldsCmb, "1, 1");
    fieldsPanel.setBorder(BorderFactory.createTitledBorder("Fields"));
    center.add(fieldsPanel, "1, 1");
    center.add(new JScrollPane(symbolTable), "1, 3");
    main.add(center, BorderLayout.CENTER);

    JPanel bottom = new JPanel();
    bottom.add(addValueBtn);
    bottom.add(removeValueBtn);
    main.add(bottom, BorderLayout.SOUTH);
    /*
    JPanel buttons = new JPanel();
    buttons.add(okBtn);
    buttons.add(cancelBtn);
    this.add(buttons, BorderLayout.SOUTH);
    */
    
    addListeners();
  }

  public void reset() {
    if (dataSource.getSymbolFields().size() > 0) {
      fieldsCmb.setModel(new DefaultComboBoxModel(new Vector(dataSource.getSymbolFields())));
      String displayFieldName = dataSource.getDisplayFieldName();
      if (displayFieldName == null || displayFieldName.length() == 0) {
        fieldsCmb.setSelectedIndex(0);
      } else {
        fieldsCmb.setSelectedItem(displayFieldName);
      }

      fieldsCmb.setEnabled(true);
      addValueBtn.setEnabled(true);
      removeValueBtn.setEnabled(true);
      symbolTable.setEnabled(true);
      curFieldName = (String) fieldsCmb.getSelectedItem();
      SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
      model.reset();
      nameSymbologyMap.clear();
    } else {
      fieldsCmb.setModel(new DefaultComboBoxModel(new Vector()));
      fieldsCmb.setEnabled(false);
      addValueBtn.setEnabled(false);
      removeValueBtn.setEnabled(false);
      symbolTable.setEnabled(false);
    }

  }

  private void addListeners() {
    fieldsCmb.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
        nameSymbologyMap.put(curFieldName, model.getCurrentSymbolList());
        curFieldName = (String) fieldsCmb.getSelectedItem();
        java.util.List symbolList = (java.util.List) nameSymbologyMap.get(curFieldName);
        if (symbolList == null) {
          model.clear();
        } else {
          Collections.sort(symbolList);
          model.setSymbolList(symbolList);
        }
      }
    });

    symbolTable.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2 && symbolTable.isEnabled()) {
          int col = symbolTable.columnAtPoint(e.getPoint());
          int row = symbolTable.rowAtPoint(e.getPoint());
          if (col == 0) {
            if (dataSource.isPolygon()) {
              SymbolIcon icon = (SymbolIcon) symbolTable.getModel().getValueAt(row, col);
              Color newColor = JColorChooser.showDialog(SymbologyDialog.this, "Color", icon.getColor());
              // return null on cancel
              if (newColor != null) {
                icon.setColor(newColor);
                symbolTable.invalidate();
                symbolTable.validate();
                symbolTable.repaint();
              }
            } else {
              // show line editor -- color and thickness
              SymbolIcon icon = (SymbolIcon) symbolTable.getModel().getValueAt(row, col);
              LineEditor editor = new LineEditor(icon.getColor(), icon.getThickness());
              JDialog dialog = new JDialog((JDialog) SymbologyDialog.this.getTopLevelAncestor(), "Edit Line", true);
              editor.display(dialog);
              // returns null on cancel
              if (editor.getColor() != null) {
                icon.setColor(editor.getColor());
                icon.setThickness((editor.getThickness()).intValue());
                symbolTable.invalidate();
                symbolTable.validate();
                symbolTable.repaint();
              }
            }
          }
        }
      }
    });

    addValueBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
        String fieldName = (String) fieldsCmb.getSelectedItem();
        Set valueSet = OpenMapData.getInstance().getAttributeValues(fieldName,
                dataSource.getShapefile());
        valueSet.removeAll(model.getFieldNames());
        Document doc = null;
        if (dataSource.isFieldTypeNumeric(fieldName)) doc = new DoubleDocument();
        ValueDialog vd = new ValueDialog(valueSet, doc);
        JDialog dialog = new JDialog((JDialog) SymbologyDialog.this.getTopLevelAncestor(), "Add Value", true);
        vd.display(dialog);
        java.util.List newValues = vd.getSelectedValues();
        if (newValues != null) {

          model.addValues(newValues);
        }
      }
    });

    removeValueBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        if (symbolTable.getSelectedRow() != -1) {
          int[] selectedRows = symbolTable.getSelectedRows();
          SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
          model.removeRows(selectedRows);
        }
      }
    });

    /*
    okBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
        dataSource.setDisplayData(curFieldName, model.getCurrentSymbolList());
      }
    });

    cancelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        dialog.dispose();
      }
    });
    */
  }

  public void setDisplayDataOnDataSource() {
    SymbolTableModel model = (SymbolTableModel) symbolTable.getModel();
    dataSource.setDisplayData(curFieldName, model.getCurrentSymbolList());
  }

  public void display(JDialog dialog) {
    this.dialog = dialog;
    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.setSize(300, 400);
    GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
  }


  public static void main(String[] args) {
    JFrame f = new JFrame();
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.getContentPane().setLayout(new BorderLayout());
    VectorDataSource source = new VectorDataSource();
    try {
      source.setShapefile("c:/src/agent_analyst/shapefiles/chicago/tomszips.shp");
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      System.exit(0);
    }
    f.getContentPane().add(new SymbologyDialog(source), BorderLayout.CENTER);
    f.setSize(300, 400);
    f.setVisible(true);
  }
}
