package uchicago.src.simbuilder.beans.gis;

import layout.TableLayout;
import uchicago.src.simbuilder.property.DoubleDocument;
import uchicago.src.guiUtils.GuiUtilities;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.Document;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.Collections;

/**
 * @author Nick Collier
 * @version $Revision: 1.2 $ $Date: 2004/10/24 22:44:20 $
 */
public class ValueDialog extends JPanel {

  private static final String addIcon = "/uchicago/src/simbuilder/images/add.png";
  private static final String addAllIcon = "/uchicago/src/simbuilder/images/addAll.png";
  private static final String removeIcon = "/uchicago/src/simbuilder/images/remove.png";
  private static final String removeAllIcon = "/uchicago/src/simbuilder/images/removeAll.png";

  private JList sourceList, targetList;
  private JTextField newValueFld;

  private JButton addBtn = new JButton("Add", new ImageIcon(ValueDialog.class.getResource(addIcon)));
  private JButton addAllBtn = new JButton("Add All", new ImageIcon(ValueDialog.class.getResource(addAllIcon)));
  private JButton removeBtn = new JButton("Remove", new ImageIcon(ValueDialog.class.getResource(removeIcon)));
  private JButton removeAllBtn = new JButton("Remove All", new ImageIcon(ValueDialog.class.getResource(removeAllIcon)));
  private JButton addNewBtn = new JButton("Add New", new ImageIcon(ValueDialog.class.getResource(addIcon)));
  private JButton okBtn = new JButton("OK");
  private JButton cancelBtn = new JButton("Cancel");
  private JDialog dialog;
  private boolean canceled = false;

  public ValueDialog(Set sourceValues, Document doc) {
    super(new BorderLayout());
    newValueFld = new JTextField();
    if (doc != null) newValueFld.setDocument(doc);

    DefaultListModel dataModel = new DefaultListModel();
    ArrayList list = new ArrayList(sourceValues);
    Collections.sort(list);
    for (Iterator iter = list.iterator(); iter.hasNext();) {
      dataModel.addElement(iter.next());
    }
    
    removeBtn.setEnabled(false);
    removeAllBtn.setEnabled(false);
    addBtn.setEnabled(sourceValues.size() > 0);
    addAllBtn.setEnabled(sourceValues.size() > 0);
    addNewBtn.setEnabled(false);

    sourceList = new JList(dataModel);
    targetList = new JList(new DefaultListModel());


    addBtn.setHorizontalTextPosition(JButton.LEFT);
    addAllBtn.setHorizontalTextPosition(JButton.LEFT);
    addNewBtn.setHorizontalTextPosition(JButton.LEFT);

    double[][] sizes = {{6, .33, 4, .34, 4, .33, 6},
                        {6, TableLayout.PREFERRED, 4, TableLayout.PREFERRED, 4, TableLayout.PREFERRED, 12,
                         TableLayout.PREFERRED, 4, TableLayout.PREFERRED, 12, TableLayout.PREFERRED, 4}};
    JPanel center = new JPanel(new TableLayout(sizes));
    center.add(new JLabel("Values:"), "1, 1");
    center.add(new JLabel("Selected Values:"), "5, 1");
    center.add(new JScrollPane(sourceList), "1, 3, 1, 9");
    center.add(addBtn, "3, 3");
    center.add(new JScrollPane(targetList), "5, 3, 5, 11");
    center.add(addAllBtn, "3, 5");
    center.add(removeBtn, "3, 7");
    center.add(removeAllBtn, "3, 9");
    center.add(addNewBtn, "3, 11");
    center.add(newValueFld, "1, 11");
    this.add(center, BorderLayout.CENTER);

    JPanel bottom = new JPanel();
    bottom.add(okBtn);
    bottom.add(cancelBtn);
    this.add(bottom, BorderLayout.SOUTH);

    addListeners();
  }

  private void addListeners() {
    addBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        if (sourceList.getSelectedIndex() != -1) {
          DefaultListModel sourceModel = (DefaultListModel) sourceList.getModel();
          DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
          Object[] vals = sourceList.getSelectedValues();
          for (int i = 0; i < vals.length; i++) {
            Object obj = vals[i];
            sourceModel.removeElement(obj);
            targetModel.addElement(obj);
          }

          addBtn.setEnabled(sourceModel.size() > 0);
          addAllBtn.setEnabled(sourceModel.size() > 0);
          removeBtn.setEnabled(targetModel.size() > 0);
          removeAllBtn.setEnabled(targetModel.size() > 0);
        }
      }
    });

    removeBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        if (targetList.getSelectedIndex() != -1) {
          DefaultListModel sourceModel = (DefaultListModel) sourceList.getModel();
          DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
          Object[] vals = targetList.getSelectedValues();
          for (int i = 0; i < vals.length; i++) {
            Object obj = vals[i];
            targetModel.removeElement(obj);
            sourceModel.addElement(obj);
          }

          addBtn.setEnabled(sourceModel.size() > 0);
          addAllBtn.setEnabled(sourceModel.size() > 0);
          removeBtn.setEnabled(targetModel.size() > 0);
          removeAllBtn.setEnabled(targetModel.size() > 0);
        }
      }
    });

    addAllBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        DefaultListModel sourceModel = (DefaultListModel) sourceList.getModel();
        DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
        ArrayList all = new ArrayList();
        for (int i = 0; i < sourceModel.size(); i++) {
          all.add(sourceModel.get(i));
        }
        sourceModel.removeAllElements();

        for (int i = 0, n = all.size(); i < n; i++) {
          targetModel.addElement(all.get(i));
        }

        addBtn.setEnabled(sourceModel.size() > 0);
        addAllBtn.setEnabled(sourceModel.size() > 0);
        removeBtn.setEnabled(targetModel.size() > 0);
        removeAllBtn.setEnabled(targetModel.size() > 0);
      }
    });

    removeAllBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        DefaultListModel sourceModel = (DefaultListModel) sourceList.getModel();
        DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
        ArrayList all = new ArrayList();
        for (int i = 0; i < targetModel.size(); i++) {
          all.add(targetModel.get(i));
        }
        targetModel.removeAllElements();

        for (int i = 0, n = all.size(); i < n; i++) {
          sourceModel.addElement(all.get(i));
        }

        addBtn.setEnabled(sourceModel.size() > 0);
        addAllBtn.setEnabled(sourceModel.size() > 0);
        removeBtn.setEnabled(targetModel.size() > 0);
        removeAllBtn.setEnabled(targetModel.size() > 0);
      }
    });

    addNewBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
        targetModel.addElement(newValueFld.getText().trim());
        removeBtn.setEnabled(targetModel.size() > 0);
        removeAllBtn.setEnabled(targetModel.size() > 0);
        newValueFld.setText("");
      }
    });

    newValueFld.addCaretListener(new CaretListener() {
      public void caretUpdate(CaretEvent e) {
        addNewBtn.setEnabled(newValueFld.getText().trim().length() > 0);
      }
    });

    okBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        canceled = false;
        dialog.dispose();

      }
    });

    cancelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        canceled = true;
        dialog.dispose();
      }
    });
  }

  public java.util.List getSelectedValues() {
    if (!canceled) {
      java.util.List values = new ArrayList();
      DefaultListModel targetModel = (DefaultListModel) targetList.getModel();
      for (int i = 0, n = targetModel.size(); i < n; i++) {
        values.add(targetModel.get(i));
      }
      return values;
    }
    return null;
  }

  public void display(JDialog dialog) {
    this.dialog = dialog;
    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    dialog.setSize(390, 275);
    GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
  }
}
