package uchicago.src.simbuilder.beans.gis;


import ViolinStrings.Strings;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.w3c.dom.Element;
import uchicago.src.nqpy.compiler.CompilerException;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.nqpy.compiler.NQPyClassFactory;
import uchicago.src.simbuilder.base.gis.AbstractOpenMapGISAgent;
import uchicago.src.simbuilder.beans.*;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.sim.util.RepastException;

import java.awt.*;
import java.beans.PropertyEditorManager;
import java.beans.PropertyChangeEvent;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.*;
import java.util.List;


public class VectorAgentProducer extends AbstractAgentProducer {

  static {
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.gis.VectorDataSource.class,
            uchicago.src.simbuilder.beans.gis.VectorDataSourcePropertyEditor.class);
  }

  private VectorDataSource dataSource = new VectorDataSource();
  
  public VectorAgentProducer() {
    super();
    name = "VectorAgent";
    groupName = "vectorAgentGroup";
  }

  public VectorDataSource getDatasource() {
    return dataSource;
  }

  public void setDatasource(VectorDataSource dataSource) {
    this.dataSource = dataSource;

  }

  /**
   * Adds this EvolverBean to the specified EvolverParent. This method is
   * corresponds to the accept method in the visitor pattern. Classes which
   * implement this method should call the appropriate method of the
   * EvolverParent with themselves as the parameter.
   *
   * @throws uchicago.src.sim.util.RepastException
   *          if the
   *          parent cannot successfully add the child
   */
  public void addToParent(EvolverParent parent) throws RepastException {
    super.addToParent(parent); 
    ((DefaultGISModelProducer)parent).addPropertyChangeListener(this);
    GISPackage gisPackage = (GISPackage)((DefaultGISModelProducer)parent).getGisPackage().getSelected();
    dataSource.setGisPackage(gisPackage);
  }

  /**
   * Removes this VectorAgentProducer from the specified parent.
   */
  public void removeFromParent(EvolverParent parent) {
    super.removeFromParent(parent);   
    ((DefaultGISModelProducer)parent).removePropertyChangeListener(this);
  }

  public void propertyChange(PropertyChangeEvent evt) {
    super.propertyChange(evt);    
    if (evt.getPropertyName() != null && evt.getPropertyName().equals(PropertyName.GIS_PACKAGE_NAME.getName())) {
      dataSource.setGisPackage((GISPackage)evt.getNewValue());
    }
  }

  public Actions getActions() {
    super.getActions();
    for (int i = 0, n = dataSource.fieldCount(); i < n; i++) {
      actions.addVarName(dataSource.getFieldNameAt(i));
    }

    return actions;
  }

  /**
   * Returns the nqpy that becomes part of the model's build (not the produced agent's)
   * method.
   *
   * @return
   */
  public String getNQPyInit() {
    if (dataSource.isValid()) {
      VelocityContext context = new VelocityContext();
      context.put("dataHandler", dataSource.getGisPackage().getDataHandlerString());
      context.put("groupName", getGroupName());
      context.put("fqnName", pkg + "." + name);
      context.put("shapefile", Strings.change(dataSource.getShapefile(), "\\", "/"));
      context.put("super", super.getNQPyInit());
     
      // render a template (in the classpath) giving fully qualified
      // path to the template file
      StringWriter stmt = new StringWriter();

      try {
        String template = getClass().getName().replace('.', '/') + "NQPyInit.vt";
        Velocity.mergeTemplate(template, "UTF-8", context, stmt);
      } catch (Exception e) {
        throw new CompilerException("Compiler Error: " + e);
      }

      return stmt.toString();
    }

    return super.getNQPyInit();
  }

  /**
   * Builds a DynamicType that represents the class compiled from this
   * NetworkNodeProducer.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator) {

    for (int i = 0; i < compileables.size(); i++) {
      EvolverAgentCompileable comp =
              (EvolverAgentCompileable) compileables.get(i);
      h = comp.buildType(h, generator);
    }

    String fqn = pkg + "." + name;
    NQPyDynamicClass dynClass = new NQPyDynamicClass(fqn, 
            getDatasource().getGisPackage().getAbstractBaseClassForProducer());

    // add a no-op constructor
    dynClass.addMethod(name, fqn, new String[]{});
    dynClass = parameters.buildType(dynClass);
    implementAsCustomProbeable(dynClass);
    dynClass = actions.buildType(dynClass);
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    dynClass.addField("model", modelName);
    dynClass.addMethod("setModel", "void", new String[]{modelName});
    dynClass = dataSource.buildType(dynClass);
    h.put(dynClass.getJClassName(), dynClass);
    generator.putDetail(EvolverBean.AGENT_NAME, fqn);

    return h;
  }

  private void createInitDrawMapMethod(ByteCodeGenerator generator) {
    if (dataSource.getDisplaySymbolData().size() > 0) {
      if (dataSource.isPolygon()) createPolyInitDrawMapMethod(generator);
      else if (dataSource.isLine()) createLineInitDrawMapMethod(generator);
    }
  }

  private void createLineInitDrawMapMethod(ByteCodeGenerator generator) {
    NQPyClassFactory.addImport("uchicago.src.simbuilder.base.gis.LineSetter");
    String selfName = pkg + "." + name;
    Hashtable dynTypes = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) dynTypes.get(selfName);
    NQPyBlockCompiler compiler = new NQPyBlockCompiler("initDrawingMap", void.class, generator.getClassFile());
    compiler.setSelf(dt);
    compiler.setBlockName(selfName + ".initDrawingMap");
    
    // need default color, default line setter data (e.g. Color(int), thickness), and
    // map where key is value, and value is line setter data.
    Map setterMap = new HashMap();
    Color defaultColor = Color.RED;
    int defaultThickness = 1;
    for (Iterator iter = dataSource.getDisplaySymbolData().iterator(); iter.hasNext();) {
      SymbolValuePair pair = (SymbolValuePair) iter.next();
      Color color =  pair.getIcon().getColor();
      int thickness = pair.getIcon().getThickness();
      if (pair.isDefault) {
        defaultColor = color;
        defaultThickness = thickness;
      } else {
        setterMap.put(pair.getValue(), "Color(" + color.getRGB() + "), " + thickness);
      }
    }
    
    VelocityContext context = new VelocityContext();
    context.put("lineSetters", setterMap);
    context.put("defColor", new Integer(defaultColor.getRGB()));
    context.put("defLineSetter", "Color(" + defaultColor.getRGB() + "), " + defaultThickness);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "LineInitDrawingMap.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    //System.out.println(stmt);
    compiler.compile(stmt.toString());
  }


  private void createPolyInitDrawMapMethod(ByteCodeGenerator generator) {
    String selfName = pkg + "." + name;
    Hashtable dynTypes = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) dynTypes.get(selfName);
    NQPyBlockCompiler compiler = new NQPyBlockCompiler("initDrawingMap", void.class, generator.getClassFile());
    compiler.setSelf(dt);
    compiler.setBlockName(selfName + ".initDrawingMap");

    Map colorMap = new HashMap();
    Color defaultColor = Color.RED;
    for (Iterator iter = dataSource.getDisplaySymbolData().iterator(); iter.hasNext();) {
      SymbolValuePair pair = (SymbolValuePair) iter.next();
      if (pair.isDefault) {
        defaultColor = pair.getIcon().getColor();
      } else {
        colorMap.put(pair.getValue(), new Integer(pair.getIcon().getColor().getRGB()));
      }
    }

    VelocityContext context = new VelocityContext();
    context.put("colors", colorMap);
    context.put("defColor", new Integer(defaultColor.getRGB()));
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "InitDrawingMap.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    compiler.compile(stmt.toString());
  }

  private void createSetupDrawMethod(ByteCodeGenerator generator) {
    String selfName = pkg + "." + name;
    Hashtable dynTypes = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) dynTypes.get(selfName);
    NQPyBlockCompiler compiler = new NQPyBlockCompiler("setupDraw", Color.class, generator.getClassFile());
    compiler.setSelf(dt);
    compiler.setBlockName(selfName + ".setupDraw");

    List valList = new ArrayList();
    List data = dataSource.getDisplaySymbolData();
    String firstValue = "";
    boolean manyColors = false;
    boolean isStringType = true;
    if (data.size() > 1) {
      isStringType = !dataSource.isFieldTypeNumeric(dataSource.getDisplayFieldName());
      manyColors = true;
      boolean firstNonDefault = true;
      for (Iterator iter = dataSource.getDisplaySymbolData().iterator(); iter.hasNext();) {
        SymbolValuePair pair = (SymbolValuePair) iter.next();
        if (!pair.isDefault) {
          if (firstNonDefault) firstValue = pair.getValue();
          else valList.add(pair.getValue());
          firstNonDefault = false;
        }
      }
    }
    
    VelocityContext context = new VelocityContext();
    context.put("manyColors", new Boolean(manyColors));
    context.put("value", firstValue);
    context.put("field", dataSource.getDisplayFieldName());
    context.put("values", valList);
    context.put("isStringType", new Boolean(isStringType));
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template;
      if (dataSource.isLine()) template = getClass().getName().replace('.', '/') + "LineSetupDraw.vt";
      else template = getClass().getName().replace('.', '/') + "SetupDraw.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    //System.out.println(stmt.toString());
    compiler.compile(stmt.toString());
  }

  /**
   * Compile the class described by this GridAgentProducer into
   * bytecode.
   */
  public void compile(String dir, ByteCodeGenerator generator) {
    for (int i = 0; i < compileables.size(); i++) {
      EvolverBean bean = (EvolverBean) compileables.get(i);
      bean.compile(dir, generator);
    }

    String fqn = pkg + "." + name;
    ArrayList list = new ArrayList();
    list.add("model");
    String[] params = new String[1];
    String modelName = (String) generator.getDetail(EvolverBean.MODEL_NAME);
    params[0] = modelName;

    generator.createClass(fqn, getDatasource().getGisPackage().getAbstractBaseClassForProducer());
    generator.createFieldsConstructor(list, params);
    compileAsCustomProbeable(generator, new String[]{});

    parameters.compile(generator, null);
    actions.compile(generator, fqn);
    dataSource.compile(generator);
    if (dataSource.getGisPackage().equals(GISPackage.OPEN_MAP)) {
      createInitDrawMapMethod(generator);
      createSetupDrawMethod(generator);
    }
    generator.finishCtor();
    makeModelSet(generator);
    generator.finishClass();
    generator.writeToFile(fqn, dir);
  }

  /**
   * Saves an xml representation of this NetworkNodeProducer to the specified
   * output stream.
   *
   * @param out the BufferedWriter to write the xml to.
   */
  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"Compileable\" class=\"");
    b.append(getClass());
    b.append("\" name=\"");
    b.append(name);
    b.append("\" groupName=\"");
    b.append(groupName);
    b.append("\">");
    out.write(b.toString());
    out.newLine();
    out.write(dataSource.toXML());
    out.newLine();
    super.saveXML(out);
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
    out.newLine();
  }

  public void loadFromXML(Element nodeE) {
    // loads name, actions, parameters and schedule
    super.loadFromXML(nodeE);
    setDatasource(VectorDataSource.createDataSource(nodeE));
    setGroupName(nodeE.getAttribute("groupName"));
  }
}
