package uchicago.src.simbuilder.beans.gis;

import ViolinStrings.Strings;
import anl.repast.gis.data.FieldNameAndType;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import anl.repast.gis.data.OpenMapData;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.simbuilder.codegen.Modifier;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.EvolverGui;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;
import java.awt.*;

/**
 * @author Nick Collier
 * @version $Revision: 1.3 $ $Date: 2004/12/10 22:04:16 $
 */
public class VectorDataSource {
  
  private static Set symbolFieldClasses = new HashSet();

  static {
    symbolFieldClasses.add(int.class);
    symbolFieldClasses.add(float.class);
    symbolFieldClasses.add(double.class);
    symbolFieldClasses.add(long.class);
    symbolFieldClasses.add(boolean.class);
    symbolFieldClasses.add(String.class);
    symbolFieldClasses.add(byte.class);
    symbolFieldClasses.add(char.class);
  }

  class FieldData implements Comparable {
    private String name;
    private Class type;
    private boolean enabled = true;

    public FieldData(String name, Class type) {
      this.name = Strings.change(name, ' ', '_');
      if (type.equals(Integer.class))
        this.type = int.class;
      else if (type.equals(Long.class))
        this.type = long.class;
      else if (type.equals(Boolean.class))
        this.type = boolean.class;
      else if (type.equals(Float.class))
        this.type = float.class;
      else if (type.equals(Double.class))
        this.type = double.class;
      else
        this.type = type;
    }

    public String getName() {
      return name;
    }

    public Class getType() {
      return type;
    }

    public int compareTo(Object o) {
      FieldData pair = (FieldData) o;
      return this.name.compareTo(pair.name);
    }

    public boolean isEnabled() {
      return enabled;
    }

    public void setEnabled(boolean enabled) {
      this.enabled = enabled;
    }
  }

  private String shapefile = "";
  private ArrayList fields = new ArrayList();
  private Set shapefiles = new HashSet();
  private List displaySymbolData = new ArrayList();
  private String displayFieldName = null;
  private GISPackage gisPackage = GISPackage.OPEN_MAP;

  public VectorDataSource(VectorDataSource source) {
    this.shapefile = source.shapefile;
    for (int i = 0; i < source.fields.size(); i++) {
      FieldData pair = (FieldData) source.fields.get(i);
      FieldData copy = new FieldData(pair.getName(), pair.getType());
      copy.setEnabled(pair.isEnabled());
      this.fields.add(copy);
    }
    this.shapefiles.addAll(source.shapefiles);
    this.displayFieldName = source.displayFieldName;
    this.displaySymbolData.addAll(source.displaySymbolData);
    this.gisPackage = source.gisPackage;
  }

  public VectorDataSource() {
  }

  public GISPackage getGisPackage() {
    return gisPackage;
  }

  public void setGisPackage(GISPackage gisPackage) {
    if (!this.gisPackage.equals(gisPackage)) {
      this.gisPackage = gisPackage;
      if (isValid()) {
        try {
          setShapefile(this.shapefile);
        } catch (IOException ex) {
          // its OK to catch this and not really do anything
          // because isValid() will be false if there is a
          // problem. If its true we know we can set the shapefile
          // without any io problems.
          ex.printStackTrace();
        }
      }
    }
  }

  public void addShapefile(String file) {
    shapefiles.add(new File(file));
  }

  public Set getShapefiles() {
    return shapefiles;
  }
  
  public void setDisplayData(String displayFieldName, List symbolList) {
    this.displayFieldName = displayFieldName;
    this.displaySymbolData = new ArrayList(symbolList);
  }
  
  public String getDisplayFieldName() {
    return displayFieldName;
  }
  
  public List getDisplaySymbolData() {
    return displaySymbolData;
  }
  
  public boolean isPolygon() {
    if (shapefile == null || shapefile.trim().length() == 0) {
      return false;
    }
    return OpenMapData.getInstance().getShpType(shapefile) == OpenMapData.SHP_POLYGON;
  }
  
  public boolean isLine() {
    if (shapefile == null || shapefile.trim().length() == 0) {
      return false;
    }
    return OpenMapData.getInstance().getShpType(shapefile) == OpenMapData.SHP_POLYLINE;
  }

  public String getShapefileDisplayName() {
    if (shapefile == null || shapefile.length() == 0) return "None Selected";
    return shapefile.substring(shapefile.lastIndexOf(File.separator) + 1, shapefile.length());
  }

  public String getShapefile() {
    return shapefile;
  }


  public void setShapefile(String shapefile) throws IOException {
    this.shapefile = shapefile;
    fields.clear();
    if (shapefile == null || shapefile.trim().length() == 0) {
      return;
    }
    
    FieldNameAndType[] fns = gisPackage.interrogate(shapefile);
    for (int i = 0; i < fns.length; i++) {
      FieldNameAndType fn = fns[i];
      try {
        fields.add(new FieldData(fn.getFieldName(), Class.forName(fn.getFieldType())));
      } catch (ClassNotFoundException e) {
        e.printStackTrace();
      }
    }
    Collections.sort(fields);
  }

  public boolean isValid() {
    if (shapefile == null || shapefile.trim().length() == 0) return false;
    try {
      gisPackage.interrogate(shapefile);
    } catch (IOException e) {
      return false;
    }

    return true;
  }

  public int fieldCount() {
    return fields.size();
  }

  public String getFieldNameAt(int index) {
    FieldData pair = (FieldData) fields.get(index);
    return pair.getName();
  }

  public Class getFieldClassAt(int index) {
    FieldData pair = (FieldData) fields.get(index);
    return pair.getType();
  }

  public boolean isFieldEnabled(int index) {
    FieldData pair = (FieldData) fields.get(index);
    return pair.isEnabled();
  }

  public void setFieldEnabled(int index, boolean enabled) {
    FieldData pair = (FieldData) fields.get(index);
    pair.setEnabled(enabled);
  }

  public void setFieldEnabled(String name, boolean enabled) {
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData data = (FieldData) fields.get(i);
      if (data.getName().equals(name)) {
        data.setEnabled(enabled);
        break;
      }
    }
  }
  
  public List getSymbolFields() {
    List symFields = new ArrayList();
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData fd = (FieldData) fields.get(i);
      if (symbolFieldClasses.contains(fd.getType()) && fd.isEnabled()) {
        symFields.add(fd.getName());
      }
    }
    
    Collections.sort(symFields);
    return symFields;
  }

  public NQPyDynamicClass buildType(NQPyDynamicClass dynClass) {
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData data = (FieldData) fields.get(i);
      if (data.isEnabled()) {
        String fieldName = data.getName();
        Class fieldClass = data.getType();
        dynClass.addField(fieldName, fieldClass);
        String accessorName = Character.toUpperCase(fieldName.charAt(0)) + fieldName.substring(1);
        dynClass.addMethod("set" + accessorName, void.class, new Class[]{fieldClass});
        dynClass.addMethod("get" + accessorName, fieldClass, new Class[]{});
      }
    }

    return dynClass;
  }

  public String toXML() {
    StringBuffer b = new StringBuffer("<VectorDataSource shapefile=\"");
    if (shapefile == null)
      b.append("");
    else
      b.append(shapefile);
    
    b.append("\" displayField=\"");
    b.append(displayFieldName);
    b.append("\" gisPackage=\"");
    b.append(gisPackage.toString());
    b.append("\">\n");
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData data = (FieldData) fields.get(i);
      if (!data.isEnabled()) {
        String fieldName = data.getName();
        b.append("<VectorDataSourceField name=\"");
        b.append(fieldName);
        b.append("\" enabled=\"false\"/>\n");
      }
    }
    
    for (int i = 0, n = displaySymbolData.size(); i < n; i++) {
      SymbolValuePair pair = (SymbolValuePair) displaySymbolData.get(i);
      b.append("<VectorDataSourceDisplayValue field=\"");
      b.append(pair.getValue());
      b.append("\" color=\"");
      b.append(pair.getIcon().getColor().getRGB());
      b.append("\" isDefault=\"");
      b.append(pair.isDefault());
      b.append("\" thickness=\"");
      b.append(pair.getIcon().getThickness());
      b.append("\" />\n");
    }
    b.append("</VectorDataSource>\n");
    return b.toString();
  }

  public void compile(ByteCodeGenerator generator) {
    // generate private fields for each enabled data source field
    // and accessor methods.
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData pair = (FieldData) fields.get(i);
      if (pair.isEnabled()) {
        String fieldName = pair.getName();
        Class fieldClass = pair.getType();
        generator.createAccessor(fieldName, fieldClass, Modifier.PRIVATE);
      }
    }
  }
  
  public boolean isFieldTypeNumeric(String fieldName) {
    for (int i = 0, n = fields.size(); i < n; i++) {
      FieldData pair = (FieldData) fields.get(i);
      if (pair.getName().equals(fieldName)) {
        Class type = pair.getType();
        return symbolFieldClasses.contains(type) && !(type.equals(String.class));
      }
    }
    
    return false;
  }

  public static VectorDataSource createDataSource(Element parent) {
    VectorDataSource dataSource = new VectorDataSource();
    Element nodeE = (Element) parent.getElementsByTagName("VectorDataSource").item(0);
    String gisPackage = nodeE.getAttribute("gisPackage");
    if (gisPackage != null && gisPackage.length() > 0) {
      dataSource.setGisPackage(GISPackage.createPackage(gisPackage));
    }
    boolean shapefileOK = true;
    try {
      String shapefile = nodeE.getAttribute("shapefile");
      dataSource.setShapefile(shapefile);
      dataSource.addShapefile(shapefile);
    } catch (IOException e) {
      //throw new RuntimeException("Shapefile '" + nodeE.getAttribute("shapefile") + "' does not exist");
      e.printStackTrace();
      EvolverGui.showError("Invalid shapefile", e);
      shapefileOK = false;
    }

    if (shapefileOK) {
      NodeList xFields = nodeE.getElementsByTagName("VectorDataSourceField");
      for (int i = 0; i < xFields.getLength(); i++) {
        Element xField = (Element) xFields.item(i);
        dataSource.setFieldEnabled(xField.getAttribute("name"), Boolean.getBoolean(xField.getAttribute("enabled")));
      }
      
      dataSource.displayFieldName = nodeE.getAttribute("displayField");
      NodeList xPairs = nodeE.getElementsByTagName("VectorDataSourceDisplayValue");
      List displayFields = new ArrayList();
      for (int i = 0; i < xPairs.getLength(); i++) {
        Element xPair = (Element) xPairs.item(i);
        Color color = new Color(Integer.parseInt(xPair.getAttribute("color")));
        String field = xPair.getAttribute("field");
        boolean isDefault = Boolean.valueOf(xPair.getAttribute("isDefault")).booleanValue();
        String attribute = xPair.getAttribute("thickness");
        int thickness = -1;
        if (attribute.length() != 0) thickness = Integer.parseInt(attribute);
        SymbolIcon icon;
        if (thickness == -1) icon = new PolygonSymbolIcon(color);
        else icon = new LineSymbolIcon(color, thickness);
        SymbolValuePair pair = new SymbolValuePair(icon , field, isDefault);
        displayFields.add(pair);
      }
      
      Collections.sort(displayFields);
      dataSource.displaySymbolData = displayFields;
    }
    
    return dataSource;
  }
}
