package uchicago.src.simbuilder.beans.network;

import layout.TableLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import uchicago.src.simbuilder.util.RepastPyUtilities;

/**
 * @author Nick Collier
 * @version $Revision: 1.2 $ $Date: 2004/10/12 14:51:10 $
 */
public class ActionNameDialog extends JPanel {
  
  private JTextField nameFld = new JTextField();
  private JComboBox returnTypeCmb; 
  private JButton okBtn = new JButton("OK");
  private JButton cancelBtn = new JButton("Cancel");
  private JDialog dialog;
  private boolean retVal;
  private boolean showReturnType;
  
  public ActionNameDialog(String name, String returnType) {
    this(name, returnType, true);
  }
  
  public ActionNameDialog(String name, String returnType, boolean showReturnType) {
    super(new BorderLayout());
    returnTypeCmb = new JComboBox(new String[]{"None", "int", "boolean", "long", "double", "float", "String"});
    returnTypeCmb.setEditable(true);
    this.showReturnType = showReturnType;
    nameFld.setText(name);
    if (returnType.length() > 0) {
      if (returnType.equals("void")) returnTypeCmb.setSelectedIndex(0);
      else returnTypeCmb.setSelectedItem(returnType);
    }
    else returnTypeCmb.setSelectedIndex(0);
    double[][] layout = {{6, TableLayout.PREFERRED, 4, TableLayout.FILL, 6},
                         {6, TableLayout.PREFERRED, 4, TableLayout.PREFERRED, 6}};
    JPanel center = new JPanel(new TableLayout(layout));
    center.setBorder(BorderFactory.createEtchedBorder());
    center.add(new JLabel("Name:"), "1, 1");
    center.add(nameFld, "3, 1");
    if (showReturnType) {
      center.add(new JLabel("Return Type:"), "1, 3");
      center.add(returnTypeCmb, "3, 3");
    }
    this.add(center, BorderLayout.CENTER);
    
    JPanel bottom = new JPanel();
    bottom.add(okBtn);
    bottom.add(cancelBtn);
    this.add(bottom, BorderLayout.SOUTH);
    addListeners();
  }
  
  private void addListeners() {
    okBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        if (!RepastPyUtilities.isValidJavaIdentifier(getName())) {
          String message = "Invalid name. Names may only contain letters, numbers, or '_'";
          JOptionPane.showMessageDialog(dialog,  message, "Name Error", 
                  JOptionPane.OK_OPTION);
          return;
        }
        retVal = true;
        dialog.dispose();
      }
    });
    
    cancelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        retVal = false;
        dialog.dispose();
      }
    });
  }
  
  public String getName() {
    return nameFld.getText().trim();
  }
  
  public String getReturnType() {
    String ret = returnTypeCmb.getSelectedItem().toString().trim();
    if (ret.length() == 0) ret = "void";
    if (ret.equalsIgnoreCase("none")) ret = "void";
    return ret;
  }
  
  public boolean display(Dialog owner) {
    return display(owner, "Action Signature");
  }
  
  public boolean display(Dialog owner, String title) {
    dialog = new JDialog(owner, title, true);
    dialog.getContentPane().setLayout(new BorderLayout());
    dialog.getContentPane().add(this, BorderLayout.CENTER);
    if (showReturnType) dialog.setSize(275, 125);
    else dialog.setSize(275, 100);
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        retVal = false;
        dialog.dispose();
      }
    });
    uchicago.src.guiUtils.GuiUtilities.centerComponentOnScreen(dialog);
    dialog.setVisible(true);
    return retVal;
  }
  
  public static void main(String[] args) {
    JFrame f = new JFrame();
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    ActionNameDialog dialog = new ActionNameDialog("foo", "bar", false);
    f.getContentPane().setLayout(new BorderLayout());
    f.getContentPane().add(dialog, BorderLayout.CENTER);
    f.setSize(275, 125);
    f.setVisible(true);
    
  }
  
}
