package uchicago.src.simbuilder.beans.network;

import uchicago.src.simbuilder.beans.EvolverConstants;

import java.util.HashMap;

public class KamadaLayoutProperty implements LayoutProperty {

  private boolean rescale = true;
  private boolean animate = true;
  private int updates = 0;
  private double minEpsilon = 1;
  private double springConst = 1;
  private int maxPass = 5000;


  public KamadaLayoutProperty() {
  }

  public KamadaLayoutProperty(KamadaLayoutProperty prop) {
    rescale = prop.rescale;
    animate = prop.animate;
    updates = prop.updates;
    minEpsilon = prop.minEpsilon;
    springConst = prop.springConst;
    maxPass = prop.maxPass;
  }

  public LayoutProperty copy() {
    return new KamadaLayoutProperty(this);
  }

  public boolean isEditable() {
    return true;
  }

  public boolean getRescale() {
    return rescale;
  }

  public void setRescale(boolean val) {
    rescale = val;
  }

  public boolean getAnimate() {
    return animate;
  }

  public void setAnimate(boolean val) {
    animate = val;
  }

  public int getUpdateEveryN() {
    return updates;
  }

  public void setUpdateEveryN(int val) {
    updates = val;
  }

  public int getMaxPasses() {
    return maxPass;
  }

  public void setMaxPasses(int val) {
    maxPass = val;
  }

  public double getMinEpsilon() {
    return minEpsilon;
  }

  public void setMinEpsilon(double val) {
    minEpsilon = val;
  }

  public double getSpringConstant() {
    return springConst;
  }

  public void setSpringConstant(double val) {
    springConst = val;
  }

  public int boolToInt(boolean val) {
    return val ? 1 : 0;
  }

  public String getInitScript(String varName, String ctorArgs, String dispName) {
    StringBuffer b = new StringBuffer(varName);
    b.append(" = KamadaGraphLayout(");
    b.append(ctorArgs);
    b.append(")\n");
    b.append(varName);
    b.append(".setAnimateTransitions(");
    b.append(boolToInt(animate));
    b.append(")\n");
    b.append(varName);
    b.append(".setRescaleLayout(");
    b.append(boolToInt(rescale));
    b.append(")\n");
    b.append(varName);
    b.append(".setUpdateEveryN(");
    b.append(updates);
    b.append(")\n");
    b.append(varName);
    b.append(".setMinEpsilon(");
    b.append(minEpsilon);
    b.append(")\n");
    b.append(varName);
    b.append(".setSpringConst(");
    b.append(springConst);
    b.append(")\n");
    b.append(varName);
    b.append(".setMaxPasses(");
    b.append(maxPass);
    b.append(")\n");
    b.append(varName);
    b.append(".setDisplay(");
    b.append(dispName);
    b.append(")\n");
    return b.toString();
  }

  public String[] getProperties() {
    return new String[]{"Rescale", "Animate", "UpdateEveryN", "MaxPasses",
                        "MinEpsilon", "SpringConstant"};
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_display";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"");
    b.append(getClass().getName());
    b.append("\"");
    b.append(" rescale=\"");
    b.append(rescale);
    b.append("\" animate=\"");
    b.append(animate);
    b.append("\" updateN=\"");
    b.append(updates);
    b.append("\" maxPasses=\"");
    b.append(maxPass);
    b.append("\" springConstant=\"");
    b.append(springConst);
    b.append("\" minEpsilon=\"");
    b.append(minEpsilon);
    b.append("\" />");
    return b.toString();
  }

  public void fromMap(HashMap map) {
    Boolean b = Boolean.valueOf((String) map.get("rescale"));
    setRescale(b.booleanValue());
    b = Boolean.valueOf((String) map.get("animate"));
    setAnimate(b.booleanValue());
    Integer i = Integer.valueOf((String) map.get("updateN"));
    setUpdateEveryN(i.intValue());
    i = Integer.valueOf((String) map.get("maxPasses"));
    setMaxPasses(i.intValue());
    Double d = Double.valueOf((String) map.get("springConstant"));
    setSpringConstant(d.doubleValue());
    d = Double.valueOf((String) map.get("minEpsilon"));
    setMinEpsilon(d.doubleValue());
  }

  public String getClassName() {
    return "uchicago.src.sim.gui.KamadaGraphLayout";
  }
}
