package uchicago.src.simbuilder.beans.network;

import uchicago.src.simbuilder.property.EvolverPropertyEditor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Vector;

/**
 * Property Editor for ModelParameter properties.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:28 $
 */
public class LayoutEditor extends EvolverPropertyEditor {

  private HashMap map = new HashMap();

  private JButton b = EvolverPropertyEditor.getEditButton();
  private JComboBox box;
  private LayoutProperty curSource;

  public LayoutEditor() {
    Vector v = new Vector();
    v.add(new CircularFactory());
    map.put(CircularLayoutProperty.class, v.get(0));
    v.add(new FruchReinFactory());
    map.put(FruchLayoutProperty.class, v.get(1));
    v.add(new KamadaFactory());
    map.put(KamadaLayoutProperty.class, v.get(2));
    v.add(new RandomFactory());
    map.put(RandomLayoutProperty.class, v.get(3));
    box = new JComboBox(v);

    panel = new JPanel();
    panel.setBackground(Color.WHITE);
    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
    b.setToolTipText("Edit Layout");
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(box);
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(b);

    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showDialog();
      }
    });

    box.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
          LayoutFactory fac = (LayoutFactory) box.getSelectedItem();
          if (!fac.isSourceFor(curSource)) {
            curSource = fac.createSource();
            firePropertyChange();
          }
        }
      }
    });
  }

  public void setValue(Object val) {
    curSource = (LayoutProperty) val;
    if (map.get(curSource.getClass()) != null) {
      b.setEnabled(curSource.isEditable());
      box.setSelectedItem(map.get(curSource.getClass()));
    }
  }

  public Object getValue() {
    return curSource;
  }

  public void showDialog() {
    LayoutPropertyPanel p = new LayoutPropertyPanel(curSource.copy());
    p.display((JFrame) panel.getTopLevelAncestor(), this);
  }

  public void setPropertyFromValue(Object bean) {
    NetworkDisplayProducer producer = (NetworkDisplayProducer) bean;
    producer.setLayout((LayoutProperty)getValue());
  }
}

interface LayoutFactory {

  public boolean isSourceFor(LayoutProperty source);

  public LayoutProperty createSource();

}

class FruchReinFactory implements LayoutFactory {

  public boolean isSourceFor(LayoutProperty source) {
    return source.getClass().equals(FruchLayoutProperty.class);
  }

  public LayoutProperty createSource() {
    return new FruchLayoutProperty();
  }

  public String toString() {
    return "Fruchman-Reingold";
  }
}

class CircularFactory implements LayoutFactory {

  public boolean isSourceFor(LayoutProperty source) {
    return source.getClass().equals(CircularLayoutProperty.class);
  }

  public LayoutProperty createSource() {
    return new CircularLayoutProperty();
  }

  public String toString() {
    return "Circular";
  }
}

class KamadaFactory implements LayoutFactory {

  public boolean isSourceFor(LayoutProperty source) {
    return source.getClass().equals(KamadaLayoutProperty.class);
  }

  public LayoutProperty createSource() {
    return new KamadaLayoutProperty();
  }

  public String toString() {
    return "Kamada-Kawai";
  }
}


class RandomFactory implements LayoutFactory {

  public boolean isSourceFor(LayoutProperty source) {
    return source.getClass().equals(RandomLayoutProperty.class);
  }

  public LayoutProperty createSource() {
    return new RandomLayoutProperty();
  }

  public String toString() {
    return "Random";
  }
}
