package uchicago.src.simbuilder.beans.network;

import uchicago.src.reflector.DescriptorContainer;
import uchicago.src.reflector.FilePropertyDescriptor;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.Fields;
import uchicago.src.nqpy.compiler.CompilerException;

import java.util.Hashtable;
import java.io.StringWriter;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import ViolinStrings.Strings;

public class MatrixFileNetworkSource implements NetworkSource,
        NetworkConstants, DescriptorContainer {

  private String fileName = "";
  private Hashtable descriptors = new Hashtable();
  private int type = BINARY;

  public MatrixFileNetworkSource() {
    setupDescriptors();
  }

  // copy constructor
  public MatrixFileNetworkSource(MatrixFileNetworkSource source) {
    fileName = source.fileName;
    type = source.type;
    setupDescriptors();
  }

  private void setupDescriptors() {
    Hashtable h = new Hashtable();
    h.put(new Integer(BINARY), "Binary (Adjacency cell value is 1 or 0)");
    h.put(new Integer(SMALL), "Small (Adjacency cell value is between -128 and 128");
    h.put(new Integer(LARGE), "Large (Adjacency cell value is a double precision number)");
    ListPropertyDescriptor pd = new ListPropertyDescriptor("Type", h);
    descriptors.put("Type", pd);
    FilePropertyDescriptor fd = new FilePropertyDescriptor("FileName");
    descriptors.put("FileName", fd);
  }

  public NetworkSource copy() {
    return new MatrixFileNetworkSource(this);
  }

  public String getFileName() {
    return fileName;
  }

  public void setFileName(String val) {
    //System.out.println("setting filename: " + val);
    fileName = val;
  }

  public Fields fixParameters(NetworkNodeProducer producer, Fields params) {
    params.removeImplicitFields(producer);
    return params;
  }

  public int getType() {
    return type;
  }

  public void setType(int val) {
    type = val;
  }

  public String[] getProperties() {
    return new String[]{"FileName", "Type"};
  }

  public Hashtable getParameterDescriptors() {
    return descriptors;
  }

  public String getInitScript(String agentName, String groupName) {
    VelocityContext context = new VelocityContext();
    context.put("agentName", agentName);
    context.put("filename", Strings.change(fileName, "\\", "/"));
    context.put("type", new Integer(type));
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Init.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    return stmt.toString();
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_source";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"uchicago.src.simbuilder.beans.network.MatrixFileNetworkSource\" fileName=\"");
    b.append(fileName);
    b.append("\" type=\"");
    b.append(type);
    b.append("\" />");

    return b.toString();
  }
}
