package uchicago.src.simbuilder.beans.network;

import org.w3c.dom.Element;
import uchicago.src.sim.analysis.NetSeqTableRow;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.XMLLoader;

import java.awt.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class NetStats {

  private HashMap stats = new HashMap();

  public NetStats() {

    StatView row = new StatView("Clust Coef", Color.red, "ClusterCoefficient");
    stats.put("ClusterCoefficient", row);

    row = new StatView("Density", Color.green, "Density");
    stats.put("Density", row);

    row = new StatView("Diameter", Color.blue, "Diameter");
    stats.put("Diameter", row);

    row = new StatView("Components", Color.magenta, "ComponentCount");
    stats.put("ComponentCount", row);

    row = new StatView("Avg. Path Length", Color.black, "AvgPathLength");
    stats.put("AvgPathLength", row);

    row = new StatView("Symmetry", Color.orange, "Symmetry");
    stats.put("Symmetry", row);
  }

  public NetStats(NetStats netStats) {
    for (Iterator iter = netStats.stats.keySet().iterator(); iter.hasNext();) {
      Object key = iter.next();
      StatView view = (StatView) netStats.stats.get(key);
      stats.put(key, new StatView(view));
    }
  }

  public int size() {
    return stats.size();
  }

  public String toBePlotted() {
    StringBuffer buffer = new StringBuffer();
    if (stats.size() == 0) buffer.append("None");
    boolean first = true;
    for (Iterator iter = stats.values().iterator(); iter.hasNext();) {
      StatView view = (StatView) iter.next();
      if (view.show()) {
        if (!first) buffer.append(", ");
        buffer.append(view.getName());
        first = false;
      }
    }

    return buffer.toString();
  }

  ArrayList getRows() {
    return new ArrayList(stats.values());
  }

  public String getNQPyForSetup(String plotName) {
    StringBuffer b = new StringBuffer();
    for (Iterator iter = stats.values().iterator(); iter.hasNext();) {
      StatView view = (StatView) iter.next();
      if (view.show()) {
        b.append(plotName);
        b.append(".");
        b.append(view.getNQPy());
      }
    }
    return b.toString();
  }

  public String toXML() {
    StringBuffer b = new StringBuffer();
    for (Iterator iter = stats.values().iterator(); iter.hasNext();) {
      StatView view = (StatView) iter.next();
      if (view.show()) {
        b.append(view.toXML());
        b.append("\n");
      }
    }

    return b.toString();
  }

  private Color makeColor(Element elem) {
    return new Color(XMLLoader.getIntVal(elem, "red"),
            XMLLoader.getIntVal(elem, "green"),
            XMLLoader.getIntVal(elem, "blue"));
  }

  public void loadFromXML(Element elem) {
    ArrayList views = XMLLoader.getChildElementsByName(elem,
            EvolverConstants.XMLNS +
            ":net_stat");
    for (Iterator iter = views.iterator(); iter.hasNext();) {
      Element el = (Element) iter.next();
      StatView view = new StatView(el.getAttribute("name"), makeColor(el),
              el.getAttribute("plotCmd"),
              XMLLoader.getIntVal(el, "markType"),
              XMLLoader.getBooleanVal(el, "show"));
      stats.put(view.getPlotCmd(), view);
    }
  }
}

class StatView extends NetSeqTableRow {

  private String cmd;

  public StatView(String name, Color color, String plotCmd) {
    super(name, null, color);
    cmd = plotCmd;
  }

  public StatView(String name, Color color, String plotCmd, int markType,
                  boolean show) {
    super(name, null, color);
    cmd = plotCmd;
    this.markType = markType;
    this.show = show;
  }

  public StatView(StatView view) {
    super(view.name, null, view.color);
    name = view.name;
    sequence = view.sequence;
    color = view.color;
    show = view.show;
    hasRun = view.hasRun;
    markType = view.markType;
    cmd = view.cmd;
  }

  public boolean show() {
    return show;
  }

  public String getPlotCmd() {
    return cmd;
  }

  public String getName() {
    return name;
  }

  public String getNQPy() {
    if (show) {
      StringBuffer b = new StringBuffer("plot");
      b.append(cmd);
      b.append("(\"");
      b.append(name);
      b.append("\", Color(");
      b.append(color.getRed());
      b.append(", ");
      b.append(color.getGreen());
      b.append(", ");
      b.append(color.getBlue());
      b.append("), ");
      b.append(markType);
      b.append(")\n");
      return b.toString();
    }

    return "";
  }

  public String toXML() {
    String xmlName = EvolverConstants.XMLNS + ":net_stat";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlName);

    b.append(" name=\"");
    b.append(name);

    b.append("\" red=\"");
    b.append(color.getRed());
    b.append("\" green=\"");
    b.append(color.getGreen());
    b.append("\" blue=\"");
    b.append(color.getBlue());

    b.append("\" show=\"");
    b.append(show);

    b.append("\" markType=\"");
    b.append(markType);

    b.append("\" plotCmd=\"");
    b.append(cmd);
    b.append("\" />");
    return b.toString();
  }
}
