package uchicago.src.simbuilder.beans.network;

import org.w3c.dom.Element;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.simbuilder.EvolverModelCompiler;
import uchicago.src.simbuilder.beans.*;
import uchicago.src.simbuilder.base.DefaultSimModelImpl;
import uchicago.src.xml.XMLFactory;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;


/**
 * A default model EvolverModel for network simulations constructed in
 * Evolver. This loads the agents from a dl format file. Most of the
 * EvovlerModel functionality is provided in the base class,
 * AbstractEvolverModel.
 * <p/>
 * $Revision: 1.2 $ $Date: 2004/12/10 22:05:37 $
 */

public class NetworkModelProducer extends AbstractEvolverModel {

  public NetworkModelProducer() {
    className = "NetworkModel";
    dispName = "Network Model";
    setParameters(parameters);
    pcs = new PropertyChangeSupport(this);
    setParameters(parameters);
    actions.createDefaultAction("initAgents", "");
  }

  /**
   * Returns nqpy statements that become part of the _build method in
   * the class compiled from this EvolverModel.
   */
  public String buildStmts() {
    StringBuffer stmt = new StringBuffer();
    if (compileables.size() > 0) {
      stmt.append("self.initAgents()\n");
    }
    
    for (Iterator iter = compileables.iterator(); iter.hasNext(); ) {
      Object o = iter.next();
      if (o instanceof AgentProducer) {
        stmt.append("self.agentList.addAll(self.");
        stmt.append(((AgentProducer)o).getGroupName());
        stmt.append(")\n");
      }
    }

    return stmt.toString();
  }

  // incorporates the EvolverComponents that are children of this model
  // into the DynamicType the represents the compilied class for this
  // model. In general EvolverComponents add instance variables
  // (fields) into the compiled class, and the DynamicType needs to
  // represent these fields during compilation.
  private void processComponents(NQPyDynamicClass me) {
    for (int i = 0; i < components.size(); i++) {
      EvolverComponent comp = (EvolverComponent) components.get(i);
      Hashtable h = comp.fields();
      Enumeration e = h.keys();
      while (e.hasMoreElements()) {
        String compName = (String) e.nextElement();
        String className = (String) h.get(compName);
        me.addField(compName, className);
      }
    }
  }

  /**
   * Creates a DynamicType that represents the compiled class produced
   * by this NetworkModelProducer. This DynamicType is added to the
   * passed-in Hashtable (key is class name, value is DynamicType
   * which is then returned.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator) {
    // check that we don't have conflicting names and groups names in our agent producers.
    sanityCheck();

    String fqn = pkg + "." + className;
    NQPyDynamicClass me = new NQPyDynamicClass(fqn,
            DefaultSimModelImpl.class);
    me.addMethod(className, fqn, new String[]{});
    processComponents(me);
    processAgentProducers(me);
    me = parameters.buildType(me);
    me = actions.buildType(me);
    h.put(me.getJClassName(), me);

    for (int i = 0; i < compileables.size(); i++) {
      EvolverCompileable bean = (EvolverCompileable) compileables.get(i);
      h = bean.buildType(h, generator);
    }

    generator.putDetail(EvolverBean.MODEL_NAME, fqn);
    return h;
  }

  public Class getDynamicSuperclass() {
    return DefaultSimModelImpl.class;
  }


  /**
   * Compile this NetworkModelProducer. Most of the actual compilation
   * work is done in the EvolverModelCompiler.
   */
  public void compile(String dir, ByteCodeGenerator generator) {
    for (int i = 0; i < compileables.size(); i++) {
      EvolverBean bean = (EvolverBean) compileables.get(i);
      bean.compile(dir, generator);
    }

    EvolverModelCompiler c = new EvolverModelCompiler();
    c.init(this, pkg);
    c.compile(generator, dir);
  }

  public void loadFromXML(Element modelE) {
    super.loadFromXML(modelE);
  }


  public void saveXML(BufferedWriter out) throws IOException {
    XMLFactory fac = new XMLFactory(EvolverConstants.XML_MODEL_FULL,
            this, xmlProps);
    try {
      out.write(fac.getXMLStartAttrib());
      out.write(" >");
      out.newLine();
      super.saveXML(out);

      out.newLine();
      out.write(fac.getXMLEnd());
      out.newLine();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void propertyChange(PropertyChangeEvent evt) {
    // let super class handle the events it is interested in.
    super.propertyChange(evt);
    if (evt.getPropertyName() != null && evt.getPropertyName().equals(PropertyName.NETWORK_SOURCE.getName())) {
      NetworkSource newNetworkSource = (NetworkSource) evt.getNewValue();
      setParameters(newNetworkSource.fixParameters((NetworkNodeProducer)evt.getSource(), parameters));
    }
  }
}


