package uchicago.src.simbuilder.beans.network;

import org.w3c.dom.Element;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.simbuilder.beans.ChartComponent;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.ScheduleDetail;
import uchicago.src.simbuilder.beans.ScheduleDetails;
import uchicago.src.simbuilder.util.BooleanMessage;
import uchicago.src.simbuilder.util.RepastPyUtilities;
import uchicago.src.nqpy.compiler.CompilerException;

import java.beans.PropertyEditorManager;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Component for producing NetSequenceGraphs-s.
 */

public class NetworkPlotProducer extends ChartComponent {

  private NetStats stats = new NetStats();

  static {
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.ScheduleDetails.class,
            uchicago.src.simbuilder.beans.ScheduleDetailsPropertyEditor.class);
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.network.NetStats.class,
            uchicago.src.simbuilder.beans.network.NetStatsPropertyEditor.class);
    PropertyEditorManager.registerEditor(uchicago.src.simbuilder.beans.FileProperty.class,
            uchicago.src.simbuilder.beans.FilePropertyEditor.class);

    uchicago.src.nqpy.compiler.NQPyClassFactory.addImport("java.awt.Color");
  }

  /**
   * Creates a NetworkPlotProducer.
   */
  public NetworkPlotProducer() {
    ArrayList a = new ArrayList();
    a.add("step");
    //a.add("writeToFile");
    details.setActionNames(a);
    name = "networkPlot";
    title = "Network Stats";
  }
  
  
    /**
   * Performs a precompile check making sure that the component is ready to compile.
   *
   * @return
   */
  public BooleanMessage sanityCheck() {
    if (!RepastPyUtilities.isValidJavaIdentifier(name)) {
      return new BooleanMessage("Invalid name '" + name + "'" +
              ". Valid names contain only letters, numbers, or '_'",
                false);
    }
    
    return BooleanMessage.TRUE;
  }

  /**
   * Gets the NetStats that describe the statistics is to be plotted.
   */
  public NetStats getStatsToPlot() {
    return stats;
  }

  /**
   * Sets the NetStats that describe the statistics is to be plotted.
   */
  public void setStatsToPlot(NetStats val) {
    stats = val;
  }


  /**
   * Gets the NQPy to be added to this NetworkPlotProducer's parent's build method.
   * This returns null as NetworkPlotProducer doesn't require anything to be added
   * to build in order to add a NetSequenceGraph to a model.
   */
  public String buildStmts() {
    return null;
  }

  /**
   * Gets the NQPy to be added to this NetworkPlotProducer's parent's setup method.
   */
  public String setupStmts() {
    VelocityContext context = new VelocityContext();
    context.put("name", name);
    context.put("doWrite", new Boolean(fileProperty.doWrite()));
    context.put("filename", fileProperty.getFileName());
    context.put("title", title);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Setup.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }
    return stmt.toString();
  }

  /**
   * Gets the NQPy to be added to this NetworkPlotProducer's parent's display method.
   */
  public String displayStmts() {
    
    VelocityContext context = new VelocityContext();
    context.put("name", name);
    context.put("doWrite", new Boolean(fileProperty.doWrite()));
    context.put("filename", fileProperty.getFileName());
    context.put("title", title);
    context.put("yAxis", new Double(yAxis));
    context.put("xAxis", new Double(xAxis));
    context.put("stats", stats.getNQPyForSetup("self." + name));
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Display.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }
    return stmt.toString();
  }

  /**
   * Gets a hashtable describing the fields required by this NetworkPlotProducer
   * to add a NetSequenceGraph to a model. The hashtable's key is the
   * name of field (the ivar) and the value is the type. These fields become
   * part of the class produced by the model.
   */
  public Hashtable fields() {
    Hashtable h = new Hashtable();
    h.put(name, "uchicago.src.sim.analysis.NetSequenceGraph");
    return h;
  }

  /**
   * Gets an ArrayList of ActionCode objects. These ActionCode objects become
   * methods in the model that are used by the NetSequenceGraph produced by
   * this NetworkPlotProducer. This returns NULL as no such methods are necessary.
   */
  public ArrayList methods() {
    return null;
  }

  // EvolverBean (extended by EvolverComponent) interface
  public void compile(String dir, ByteCodeGenerator generator) {
  }

  /**
   * Gets the ScheduleDetails for this NetworkPlotProducer. These ScheduleDetails
   * describe a schedule of excecution for the NetSequenceGraph produced
   * by this NetworkPlotProducer.
   */
  public ScheduleDetails schedule() {
    for (int i = 0; i < details.size(); i++) {
      ScheduleDetail d = details.getDetail(i);
      StringBuffer b = new StringBuffer("self.");
      b.append(name);
      if (d.getActionName().equals("step")) {
        b.append(".step()");
      } else if (d.getActionName().equals("writeToFile") &&
              fileProperty.doWrite()) {
        b.append(".writeToFile()");
      }

      d.setNQPy(b.toString());
      d.setTarget(name);
    }

    return details;
  }

  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"Component\" class=\"");
    b.append(getClass());
    b.append("\" ");
    b.append(super.getCommonXML());
    b.append(" >");

    out.write(b.toString());
    out.newLine();

    getSchedule();
    out.write(details.toXML());
    out.newLine();

    out.write(stats.toXML());
    out.newLine();

    out.write(fileProperty.toXML());
    out.newLine();
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
  }

  public void loadFromXML(Element elem) {
    super.loadCommonXML(elem);
    ArrayList a = new ArrayList();
    a.add("step");
    //a.add("writeToFile");
    details.setActionNames(a);

    stats = new NetStats();
    stats.loadFromXML(elem);
    setStatsToPlot(stats);
  }
}
