package uchicago.src.simbuilder.beans.network;

import uchicago.src.simbuilder.property.EvolverPropertyEditor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Vector;

/**
 * Property Editor for ModelParameter properties.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:29 $
 */
public class NetworkSourceEditor extends EvolverPropertyEditor {

  private HashMap map = new HashMap();

  private JButton b = EvolverPropertyEditor.getEditButton();
  private JComboBox box;
  private NetworkSource curSource;

  public NetworkSourceEditor() {
    Vector v = new Vector();
    v.add(new MatrixFileFactory());
    map.put(MatrixFileNetworkSource.class, v.get(0));
    v.add(new RandomDensityNetFactory());
    map.put(RandomDensityNetworkSource.class, v.get(1));
    v.add(new SquareLatticeNetFactory());
    map.put(SquareLatticeNetworkSource.class, v.get(2));
    v.add(new UnlinkedNodesNetFactory());
    map.put(UnlinkedNodeNetworkSource.class, v.get(3));
    v.add(new WattsStrogatzNetFactory());
    map.put(WattsStrogatzNetworkSource.class, v.get(4));
    box = new JComboBox(v);

    panel = new JPanel();
    panel.setBackground(Color.WHITE);
    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
    b.setToolTipText("Edit Network Source");
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(box);
    panel.add(Box.createRigidArea(new Dimension(5, 0)));
    panel.add(b);

    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showDialog();
      }
    });

    box.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
          NetworkSourceFactory fac =
                  (NetworkSourceFactory) box.getSelectedItem();
          if (!fac.isSourceFor(curSource)) {
            curSource = fac.createSource();
            firePropertyChange();
          }
        }
      }
    });
  }

  public void setValue(Object val) {
    curSource = (NetworkSource) val;
    if (map.get(curSource.getClass()) != null) {
      box.setSelectedItem(map.get(curSource.getClass()));
    }
  }

  public Object getValue() {
    //return params.copy();
    return curSource;
  }

  public void showDialog() {
    NetworkSourcePanel p = new NetworkSourcePanel(curSource.copy());
    p.display((JFrame) panel.getTopLevelAncestor(), this);
  }

  /**
   * Sets the bean's property from the current value in the editor.
   *
   * @param bean
   */
  public void setPropertyFromValue(Object bean) {
    NetworkNodeProducer producer = (NetworkNodeProducer) bean;
    producer.setNetworkSource((NetworkSource) getValue());
  }
}

interface NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source);

  public NetworkSource createSource();

}

class MatrixFileFactory implements NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source) {
    return source.getClass().equals(MatrixFileNetworkSource.class);
  }

  public NetworkSource createSource() {
    return new MatrixFileNetworkSource();
  }

  public String toString() {
    return "Matrix File";
  }
}

class RandomDensityNetFactory implements NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source) {
    return source.getClass().equals(RandomDensityNetworkSource.class);
  }

  public NetworkSource createSource() {
    return new RandomDensityNetworkSource();
  }

  public String toString() {
    return "Random Density";
  }
}

class SquareLatticeNetFactory implements NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source) {
    return source.getClass().equals(SquareLatticeNetworkSource.class);
  }

  public NetworkSource createSource() {
    return new SquareLatticeNetworkSource();
  }

  public String toString() {
    return "Square Lattice";
  }
}

class WattsStrogatzNetFactory implements NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source) {
    return source.getClass().equals(WattsStrogatzNetworkSource.class);
  }

  public NetworkSource createSource() {
    return new WattsStrogatzNetworkSource();
  }

  public String toString() {
    return "Watts-Strogatz Small World";
  }
}

class UnlinkedNodesNetFactory implements NetworkSourceFactory {

  public boolean isSourceFor(NetworkSource source) {
    return source.getClass().equals(UnlinkedNodeNetworkSource.class);
  }

  public NetworkSource createSource() {
    return new UnlinkedNodeNetworkSource();
  }

  public String toString() {
    return "Unlinked Nodes";
  }
}
