package uchicago.src.simbuilder.beans.network;

import org.w3c.dom.Element;
import uchicago.src.simbuilder.codegen.ByteCodeGenerator;
import uchicago.src.nqpy.compiler.NQPyBlockCompiler;
import uchicago.src.nqpy.compiler.NQPyDynamicClass;
import uchicago.src.sim.network.AbstractProbabilityRule;
import uchicago.src.sim.util.RepastException;
import uchicago.src.simbuilder.base.AbstractEvProbRule;
import uchicago.src.simbuilder.beans.*;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeEvent;
import java.io.BufferedWriter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

/**
 * @version $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:29 $
 */


public class NodeLinker extends AbstractEvolverCompileable implements
        EvolverAgentCompileable {

  private static final String PROB_METHOD = "getProbability";
  private static final String UPDATE_METHOD = "update";
  public static final String GEN_CODE_TAG = "### uchicago.src.simbuilder.beans." +
          "NodeLinker Generated Code ###";

  private Fields parameters = new Fields();
  private Actions actions = new Actions();
  private String name = "NodeLinker";
  private ActionCode probCode;
  private ActionCode updateCode;
  private float startWeight = 1.0f;

  private EvolverCompileable evComp;

  public NodeLinker() {
    actions.createDefaultAction(PROB_METHOD, "");
    actions.createDefaultAction(UPDATE_METHOD, "");
    actions.addParameterName(PROB_METHOD, "otherNode");
    actions.addParameterName(UPDATE_METHOD, "otherNode");
    actions.addParameterName(UPDATE_METHOD, "amtToUpdate");
    pcs = new PropertyChangeSupport(this);
  }

  /**
   * Sets the model's parameters.
   */
  public void setParameters(Fields newParameters) {
    parameters = newParameters;
    actions.clearVarNames();
    Iterator i = parameters.namesIter();
    while (i.hasNext()) {
      actions.addVarName((String) i.next());
    }
  }

  /**
   * Gets this model's parameters.
   */
  public Fields getParameters() {
    return parameters;
  }

  public Actions getActions() {
    return actions;
  }

  public void setActions(Actions val) {
    actions = val;
  }

  public String getName() {
    return name;
  }

  public void setName(String val) {
    if (!name.equals(val)) {
      name = val;
      Actions compA = evComp.getActions();
      String initCode = initAction();
      compA.removeEvCode("init", GEN_CODE_TAG, 2);
      compA.prependToAction("init", initCode);
      evComp.setActions(compA);
    }
  }

  public float getStartWeight() {
    return startWeight;
  }

  public void setStartWeight(float val) {
    startWeight = val;
    if (evComp != null) {
      Fields p = evComp.getParameters();
      p.addField("startWeight", "float",
              String.valueOf(startWeight), true);
      evComp.setParameters(p);
    }
  }

  public void setEvolverCompileable(EvolverCompileable comp) {
    evComp = comp;
    Fields compP = evComp.getParameters();
    compP.addFields(agentParameters());
    evComp.setParameters(compP);

    Actions compA = evComp.getActions();
    String initCode = initAction();
    compA.removeEvCode("init", GEN_CODE_TAG, 2);
    compA.prependToAction("init", initCode);
    evComp.setActions(compA);
  }

  public Fields agentParameters() {
    Fields p = new Fields();
    p.addField("_linker", "uchicago.src.sim.network.Linker",
            "", false);
    p.addField("startWeight", "float", String.valueOf(startWeight), true);
    return p;
  }

  public String initAction() {
    StringBuffer b = new StringBuffer(GEN_CODE_TAG);
    b.append("\nrule = ");
    b.append(name);
    b.append("(self.model.getAgentList(), self.startWeight, self)\n");
    b.append("self._linker = Linker(rule)\n\n");
    return b.toString();
  }

  /**
   * Builds a Type object corresponding to this EvolverCompileable and
   * places it in the specified Hashtable. The key of the Hashtable is
   * the Class name of this Type and the value is the Type object itself.
   */
  public Hashtable buildType(Hashtable h, ByteCodeGenerator generator) {
    String fqn = pkg + "." + name;
    NQPyDynamicClass me =
            new NQPyDynamicClass(fqn, AbstractEvProbRule.class);
    me = parameters.buildType(me);
    me.addMethod(PROB_METHOD, double.class, new Class[]{Object.class});
    me.addMethod(UPDATE_METHOD, void.class, new Class[]{Object.class,
                                                        float.class});

    // add the constructor
    String[] params = {"java.util.List", "float", "java.lang.Object"};
    me.addMethod(name, fqn, params);
    probCode = (ActionCode) actions.remove("getProbability");
    updateCode = (ActionCode) actions.remove("update");
    me = actions.buildType(me);
    h.put(me.getJClassName(), me);

    return h;
  }

  /**
   * Gets the class name for this EvolverCompileable.
   */
  public String className() {
    return name;
  }

  /**
   * Perform any post-compile clean up. Removing non-user defined
   * internally created actions, for example.
   */
  public void postCompile() {
  }

  /**
   * Compile this EvolverBean.
   */
  public void compile(String dir, ByteCodeGenerator generator) {
    String fqn = pkg + "." + name;
    generator.createClass(fqn, AbstractEvProbRule.class);
    Class[] params = {List.class, float.class, Object.class};
    generator.createConstructorWithSuper(params);
    parameters.compile(generator, null);
    actions.compile(generator, fqn);
    actions.addAction(probCode);
    actions.addAction(updateCode);
    generator.finishCtor();

    Class c = AbstractProbabilityRule.class;
    Method probMeth = null;
    Method updateMeth = null;
    try {
      probMeth = c.getMethod(PROB_METHOD, new Class[]{Object.class});
      updateMeth = c.getMethod(UPDATE_METHOD, new Class[]{Object.class,
                                                          float.class});
    } catch (Exception ex) {
      ex.printStackTrace();
    }

    compileMethod(generator, probCode, probMeth);
    compileMethod(generator, updateCode, updateMeth);

    generator.finishClass();
    generator.writeToFile(fqn, dir);


  }

  protected void compileMethod(ByteCodeGenerator generator, ActionCode code,
                               Method method) {
    String fqn = pkg + "." + name;
    Hashtable h = (Hashtable) generator.getDetail(EvolverBean.DYN_TYPE_LIST);
    NQPyDynamicClass dt = (NQPyDynamicClass) h.get(fqn);

    NQPyBlockCompiler compiler =
            new NQPyBlockCompiler(method, generator.getClassFile());
    compiler.setSelf(dt);
    compiler.setBlockName(fqn + "." + method.getName());
    ArrayList imports = code.getImports();
    for (int j = 0; j < imports.size(); j++) {
      //System.out.println("ADDING IMPORT " + (String)imports.get(j));
      compiler.addImport((String) imports.get(j));
    }

    compiler.createMethodParams(code.getParameterNames());
    compiler.setReturnType(method.getReturnType());
    compiler.compile(code.getCode());
  }

  /**
   * Returns ScheduleDetails that are used as the basis of the model's
   * _schedule() method.
   */
  public ScheduleDetails schedule() {
    return null;
  }

  /**
   * Adds this EvolverBean to the specified EvolverParent. This method is
   * corresponds to the accept method in the visitor pattern. Classes which
   * implement this method should call the appropriate method of the
   * EvolverParent with themselves as the parameter.
   *
   * @throws uchicago.src.sim.util.RepastException
   *          if the
   *          parent cannot successfully add the child
   */
  public void addToParent(EvolverParent parent) throws RepastException {
    parent.addCompileable(this);
  }

  /**
   * Removes this EvolverBean from the specified EvolverParent. This method is
   * corresponds to the accept method in the visitor pattern. Classes which
   * implement this method should call the appropriate method of the
   * EvolverParent with themselves as the parameter.
   */
  public void removeFromParent(EvolverParent parent) {
    parent.removeCompileable(this);
  }

  /**
   * Saves an xml representation of this NetworkNodeProducer to the specified
   * output stream.
   *
   * @param out the BufferedWriter to write the xml to.
   */
  public void saveXML(BufferedWriter out) throws IOException {
    StringBuffer b = new StringBuffer("<");
    b.append(EvolverConstants.XML_BEAN_FULL);
    b.append(" type=\"AgentCompileable\" class=\"");
    b.append(getClass());
    b.append("\" name=\"");
    b.append(name);
    b.append("\" startWeight=\"");
    b.append(startWeight);
    b.append("\" >");
    out.write(b.toString());
    out.newLine();
    out.write(actions.toXML());
    out.newLine();
    out.write(parameters.toXML());
    out.newLine();
    out.write("</" + EvolverConstants.XML_BEAN_FULL + ">");
    out.newLine();
  }

  public void loadFromXML(Element nodeE) {
    setName(nodeE.getAttribute("name"));
    setActions(XMLLoader.getActions(nodeE));
    setParameters(XMLLoader.getParameters(nodeE));
    setStartWeight((float) XMLLoader.getDoubleVal(nodeE, "startWeight"));
  }

  public void propertyChange(PropertyChangeEvent evt) {
    
  }
}
