package uchicago.src.simbuilder.beans.network;

import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.Fields;
import uchicago.src.nqpy.compiler.CompilerException;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import java.io.StringWriter;

public class RandomDensityNetworkSource implements uchicago.src.simbuilder.beans.network.NetworkSource,
        NetworkConstants {

  private int size = 40;
  private double density = 0.3;
  private boolean allowLoops = false;
  private boolean isSymmetric = false;

  public RandomDensityNetworkSource() {
  }

  public RandomDensityNetworkSource(RandomDensityNetworkSource source) {
    size = source.size;
    density = source.density;
    allowLoops = source.allowLoops;
    isSymmetric = source.isSymmetric;
  }

  public NetworkSource copy() {
    return new RandomDensityNetworkSource(this);
  }

  public Fields fixParameters(NetworkNodeProducer producer, Fields params) {
    String groupName = producer.getGroupName();
    params.removeImplicitFields(producer);
    params.addImplicitField(producer, groupName + "NumAgents", "int", String.valueOf(size),
            true);
    params.addImplicitField(producer, groupName + "Density", "double",
            String.valueOf(density), true);
    params.addImplicitField(producer, groupName + "AllowLoops", "boolean",
            String.valueOf(allowLoops), true);
    params.addImplicitField(producer, groupName + "IsSymmetric", "boolean",
            String.valueOf(isSymmetric), true);
    return params;
  }

  public int getSize() {
    return size;
  }

  public void setSize(int val) {
    size = val;
  }

  public double getDensity() {
    return density;
  }

  public void setDensity(double val) {
    density = val;
  }

  public boolean isAllowLoops() {
    return allowLoops;
  }

  public void setAllowLoops(boolean val) {
    allowLoops = val;
  }

  public boolean isSymmetric() {
    return isSymmetric;
  }

  public void setSymmetric(boolean val) {
    isSymmetric = val;
  }

  public String[] getProperties() {
    return new String[]{"Size", "Density", "AllowLoops", "Symmetric"};
  }

  private int boolToInt(boolean val) {
    return val ? 1 : 0;
  }

  public String getInitScript(String agentName, String groupName) {
    VelocityContext context = new VelocityContext();
    context.put("agentName", agentName);
    context.put("groupName", groupName);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Init.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    return stmt.toString();
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_source";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"uchicago.src.simbuilder.beans.network.RandomDensityNetworkSource\"");
    b.append(" size=\"");
    b.append(size);
    b.append("\" density=\"");
    b.append(density);
    b.append("\" allowLoops=\"");
    b.append(allowLoops);
    b.append("\" symmetric=\"");
    b.append(isSymmetric);
    b.append("\" />");

    return b.toString();
  }


}
