package uchicago.src.simbuilder.beans.network;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import uchicago.src.nqpy.compiler.CompilerException;
import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.Fields;

import java.io.StringWriter;

public class SquareLatticeNetworkSource implements NetworkSource,
        NetworkConstants {

  private int cols = 10;
  private int rows = 5;
  private int connectRadius = 1;
  private boolean wrapAround = false;

  public SquareLatticeNetworkSource() {
  }

  public SquareLatticeNetworkSource(SquareLatticeNetworkSource source) {
    cols = source.cols;
    rows = source.rows;
    connectRadius = source.connectRadius;
    wrapAround = source.wrapAround;
  }

  public NetworkSource copy() {
    return new SquareLatticeNetworkSource(this);
  }

  public Fields fixParameters(NetworkNodeProducer producer, Fields params) {
    String groupName = producer.getGroupName();
    params.removeImplicitFields(producer);
    params.addImplicitField(producer, groupName + "NetworkCols", "int",
            String.valueOf(cols), true);
    params.addImplicitField(producer, groupName + "NetworkRows", "int",
            String.valueOf(rows), true);
    params.addImplicitField(producer, groupName + "ConnectRadius", "int",
            String.valueOf(connectRadius), true);
    params.addImplicitField(producer, groupName + "WrapAround", "boolean",
            String.valueOf(wrapAround), true);
    return params;
  }

  public int getCols() {
    return cols;
  }

  public void setCols(int val) {
    cols = val;
  }

  public int getRows() {
    return rows;
  }

  public void setRows(int val) {
    rows = val;
  }

  public int getConnectRadius() {
    return connectRadius;
  }

  public void setConnectRadius(int val) {
    connectRadius = val;
  }

  public boolean getWrapAround() {
    return wrapAround;
  }

  public void setWrapAround(boolean val) {
    wrapAround = val;
  }

  public String[] getProperties() {
    return new String[]{"Cols", "Rows", "ConnectRadius", "WrapAround"};
  }

  public String getInitScript(String agentName, String groupName) {
    VelocityContext context = new VelocityContext();
    context.put("agentName", agentName);
    context.put("groupName", groupName);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Init.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    return stmt.toString();
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_source";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"uchicago.src.simbuilder.beans.network.SquareLatticeNetworkSource\"");
    b.append(" cols=\"");
    b.append(cols);
    b.append("\" rows=\"");
    b.append(rows);
    b.append("\" connectRadius=\"");
    b.append(connectRadius);
    b.append("\" wrapAround=\"");
    b.append(wrapAround);
    b.append("\" />");

    return b.toString();
  }
}
