package uchicago.src.simbuilder.beans.network;

import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.Fields;
import uchicago.src.nqpy.compiler.CompilerException;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import java.io.StringWriter;

public class UnlinkedNodeNetworkSource implements NetworkSource {

  private int numNodes = 10;

  public UnlinkedNodeNetworkSource() {
  }

  public UnlinkedNodeNetworkSource(UnlinkedNodeNetworkSource source) {
    this.numNodes = source.numNodes;
  }

  public Fields fixParameters(NetworkNodeProducer producer, Fields params) {
    String groupName = producer.getGroupName();
    params.removeImplicitFields(producer);
    params.addImplicitField(producer, groupName + "NumAgents", "int",
            String.valueOf(numNodes), true);
    return params;
  }

  public int getNumberOfNodes() {
    return numNodes;
  }

  public void setNumberOfNodes(int val) {
    numNodes = val;
  }

  public String[] getProperties() {
    return new String[]{"NumberOfNodes"};
  }

  public uchicago.src.simbuilder.beans.network.NetworkSource copy() {
    return new UnlinkedNodeNetworkSource(this);
  }

  public String getInitScript(String agentName, String groupName) {
   VelocityContext context = new VelocityContext();
    context.put("agentName", agentName);
    context.put("groupName", groupName);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Init.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    return stmt.toString();
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_source";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"uchicago.src.simbuilder.beans.network.UnlinkedNodeNetworkSource\" numberOfNodes=\"");
    b.append(numNodes);
    b.append("\" />");
    return b.toString();
  }
}
