package uchicago.src.simbuilder.beans.network;

import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.simbuilder.beans.EvolverConstants;
import uchicago.src.simbuilder.beans.Fields;
import uchicago.src.nqpy.compiler.CompilerException;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import java.io.StringWriter;

public class WattsStrogatzNetworkSource implements NetworkSource,
        NetworkConstants {

  private int size = 40;
  private int connectRadius = 1;
  private double rewireProb = .4;

  public WattsStrogatzNetworkSource() {
  }

  public WattsStrogatzNetworkSource(WattsStrogatzNetworkSource source) {
    size = source.size;
    connectRadius = source.connectRadius;
    rewireProb = source.rewireProb;
  }

  public uchicago.src.simbuilder.beans.network.NetworkSource copy() {
    return new WattsStrogatzNetworkSource(this);
  }

  public Fields fixParameters(NetworkNodeProducer producer, Fields params) {
    String groupName = producer.getGroupName();
    params.removeImplicitFields(producer);
    params.addImplicitField(producer, groupName + "NumAgents", "int", String.valueOf(size),
            true);
    params.addImplicitField(producer, groupName + "ConnectRadius", "int",
            String.valueOf(connectRadius), true);
    params.addImplicitField(producer, groupName + "RewireProbability", "double",
            String.valueOf(rewireProb), true);

    return params;
  }

  public int getSize() {
    return size;
  }

  public void setSize(int val) {
    size = val;
  }

  public int getConnectRadius() {
    return connectRadius;
  }

  public void setConnectRadius(int val) {
    connectRadius = val;
  }

  public double getRewireProb() {
    return rewireProb;
  }

  public void setRewireProb(double val) {
    rewireProb = val;
  }

  public String[] getProperties() {
    return new String[]{"Size", "RewireProb", "ConnectRadius"};
  }

  public String getInitScript(String agentName, String groupName) {
    VelocityContext context = new VelocityContext();
    context.put("agentName", agentName);
    context.put("groupName", groupName);
     
    // render a template (in the classpath) giving fully qualified
    // path to the template file
    StringWriter stmt = new StringWriter();

    try {
      String template = getClass().getName().replace('.', '/') + "Init.vt";
      Velocity.mergeTemplate(template, "UTF-8", context, stmt);
    } catch (Exception e) {
      throw new CompilerException("Compiler Error" + e);
    }

    return stmt.toString();
  }

  public String toXML() {
    String xmlString = EvolverConstants.XMLNS + ":network_source";
    StringBuffer b = new StringBuffer("<");
    b.append(xmlString);
    b.append(" class=\"uchicago.src.simbuilder.beans.network.WattsStrogatzNetworkSource\"");
    b.append(" size=\"");
    b.append(size);
    b.append("\" connectRadius=\"");
    b.append(connectRadius);
    b.append("\" rewireProb=\"");
    b.append(rewireProb);
    b.append("\" />");

    return b.toString();
  }
}
