package uchicago.src.simbuilder.property;

import javax.swing.*;
import java.awt.*;
import java.beans.PropertyEditorManager;

/**
 * A JPanel containing a JTable that displays an EvolverBeans properties.
 * Most of the functionality of this table comes via the PropertyTableModel
 * class.
 * <p/>
 * $Revision: 1.1.1.1 $ $Date: 2004/09/30 03:40:50 $
 */

public class PropertyTablePanel extends JPanel {

  private static final int ROW_HEIGHT = 22;

  private JTable table;
  private PropertyTableModel model;

  public PropertyTablePanel() {
    super(new BorderLayout());
    model = new PropertyTableModel();
    table = new JTable(model, new PropertyColumnModel());
    table.setRowHeight(ROW_HEIGHT);
    add(new JScrollPane(table), BorderLayout.CENTER);

    registerPropertyEditors();
  }

  // Registers the primitive and String property editors.
  // Registering the editors with PropertyEditorManager ensures
  // that these types of properties are displayed with the
  // appropriate editor.
  private void registerPropertyEditors() {
    PropertyEditorManager.registerEditor(Boolean.class,
            uchicago.src.simbuilder.property.BooleanPropertyEditor.class);
    PropertyEditorManager.registerEditor(boolean.class,
            uchicago.src.simbuilder.property.BooleanPropertyEditor.class);
    PropertyEditorManager.registerEditor(String.class,
            uchicago.src.simbuilder.property.StringPropertyEditor.class);
    PropertyEditorManager.registerEditor(Integer.class,
            uchicago.src.simbuilder.property.IntegerPropertyEditor.class);
    PropertyEditorManager.registerEditor(int.class,
            uchicago.src.simbuilder.property.IntegerPropertyEditor.class);
    PropertyEditorManager.registerEditor(float.class,
            uchicago.src.simbuilder.property.FloatPropertyEditor.class);
    PropertyEditorManager.registerEditor(Float.class,
            uchicago.src.simbuilder.property.FloatPropertyEditor.class);
    PropertyEditorManager.registerEditor(long.class,
            uchicago.src.simbuilder.property.LongPropertyEditor.class);
    PropertyEditorManager.registerEditor(Long.class,
            uchicago.src.simbuilder.property.LongPropertyEditor.class);
    PropertyEditorManager.registerEditor(double.class,
            uchicago.src.simbuilder.property.DoublePropertyEditor.class);
    PropertyEditorManager.registerEditor(Double.class,
            uchicago.src.simbuilder.property.DoublePropertyEditor.class);
  }

  /**
   * This is called when a user clicks on an EvolverBean in the project
   * tree. That bean becomes the current object in the PropertyTableModel.
   */
  public void setObject(Object bean) {
    model.setObject(bean);
    // this is necessary to avoid getting cell editors from one
    // table displaying in another table.
    table.setColumnModel(new PropertyColumnModel());
  }
}
