package uchicago.src.simbuilder.util;

import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Map;
import java.util.prefs.Preferences;
import java.awt.*;

/**
 * @author Nick Collier
 * @version $Revision: 1.2 $ $Date: 2004/11/03 22:29:04 $
 */
public class RepastPyUtilities {
  
  static interface Validator {
    public boolean isValid(String value);
  }
  
  static class IntegerValidator implements Validator {
    public boolean isValid(String value) {
      try {
        Integer.parseInt(value);
      } catch (NumberFormatException ex) {
        return false;
      }
      return true;
    }
  }
  
  static class LongValidator implements Validator {
    public boolean isValid(String value) {
      try {
        Long.parseLong(value);
      } catch (NumberFormatException ex) {
        return false;
      }
      return true;
    }
  }
  
  static class FloatValidator implements Validator {
    public boolean isValid(String value) {
      try {
        Float.parseFloat(value);
      } catch (NumberFormatException ex) {
        return false;
      }
      return true;
    }
  }
  
  static class DoubleValidator implements Validator {
    public boolean isValid(String value) {
      try {
        Double.parseDouble(value);
      } catch (NumberFormatException ex) {
        return false;
      }
      return true;
    }
  }
  
  private static Map numberTypes = new HashMap();
  
  static {
    numberTypes.put("int", new IntegerValidator());
    numberTypes.put("long", new LongValidator());
    numberTypes.put("float", new FloatValidator());
    numberTypes.put("double", new DoubleValidator());
    numberTypes.put("java.lang.int", new IntegerValidator());
    numberTypes.put("java.lang.long", new LongValidator());
    numberTypes.put("java.lang.float", new FloatValidator());
    numberTypes.put("java.lang.double", new DoubleValidator());
  }
  
  public static boolean isValidJavaIdentifier(String name) {
    char[] chars = name.toCharArray();
    for (int i = 0; i < chars.length; i++) {
      char c = chars[i];
      if (i == 0 && !Character.isJavaIdentifierStart(c)) return false;
      else if (!Character.isJavaIdentifierPart(c)) return false;
    }
    
    return true;
  }
  
  public static boolean isNumericType(String type) {
    return numberTypes.containsKey(type);
  }
  
  public static boolean isValidBooleanValue(String value) {
    return value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false");
  }
  
  public static boolean isValidValueForNumericType(String value, String type) {
    Validator validator = (Validator) numberTypes.get(type);
    if (validator == null) return false;
    return validator.isValid(value);
  }
  
  public static Rectangle getPreferredWindowBounds(String frameKey, int defaultWidth, int defaultHeight) {
    Preferences userPrefs = Preferences.userNodeForPackage(RepastPyUtilities.class);
    int width = userPrefs.getInt(frameKey + "_WIDTH", defaultWidth);
    int height = userPrefs.getInt(frameKey + "_HEIGHT", defaultHeight);
    int x = userPrefs.getInt(frameKey + "_X", 0);
    int y = userPrefs.getInt(frameKey + "_Y", 0);

    return new Rectangle(x, y, width, height);
  }

  public static void savePreferredWindowBounds(String frameKey, Rectangle bounds) {
    Preferences userPrefs = Preferences.userNodeForPackage(RepastPyUtilities.class);
    userPrefs.putInt(frameKey + "_WIDTH", (int) bounds.getWidth());
    userPrefs.putInt(frameKey + "_HEIGHT", (int) bounds.getHeight());
    userPrefs.putInt(frameKey + "_X", (int) bounds.getX());
    userPrefs.putInt(frameKey + "_Y", (int) bounds.getY());
  }
}
